@@ -1,13 +1,13 @@
-#ifndef __BACKEND_H__
-#define __BACKEND_H__
+#ifndef FILEZILLA_ENGINE_BACKEND_HEADER
+#define FILEZILLA_ENGINE_BACKEND_HEADER
 
 #include "ratelimiter.h"
 #include "socket.h"
 
-class CBackend : public CRateLimiterObject, public CSocketEventSource
+class CBackend : public CRateLimiterObject, public fz::socket_event_source
 {
 public:
-	CBackend(fz::event_handler* pEvtHandler);
+	explicit CBackend(fz::event_handler* pEvtHandler);
 	virtual ~CBackend();
 
 	CBackend(CBackend const&) = delete;
@@ -23,22 +23,25 @@ protected:
 	fz::event_handler* const m_pEvtHandler;
 };
 
+namespace fz {
 class CSocket;
+}
+
 class CSocketBackend final : public CBackend
 {
 public:
-	CSocketBackend(fz::event_handler* pEvtHandler, CSocket & socket, CRateLimiter& rateLimiter);
+	CSocketBackend(fz::event_handler* pEvtHandler, fz::socket & socket, CRateLimiter& rateLimiter);
 	virtual ~CSocketBackend();
 	// Backend definitions
-	virtual int Read(void *buffer, unsigned int size, int& error);
-	virtual int Peek(void *buffer, unsigned int size, int& error);
-	virtual int Write(const void *buffer, unsigned int size, int& error);
+	virtual int Read(void *buffer, unsigned int size, int& error) override;
+	virtual int Peek(void *buffer, unsigned int size, int& error) override;
+	virtual int Write(const void *buffer, unsigned int size, int& error) override;
 
 protected:
-	virtual void OnRateAvailable(CRateLimiter::rate_direction direction);
+	virtual void OnRateAvailable(CRateLimiter::rate_direction direction) override;
 
-	CSocket &socket_;
+	fz::socket &socket_;
 	CRateLimiter& m_rateLimiter;
 };
 
-#endif //__BACKEND_H__
+#endif
