@@ -1,15 +1,17 @@
 #include <filezilla.h>
 #include "directorycache.h"
 
+#include <assert.h>
+
 CDirectoryCache::CDirectoryCache()
 {
 }
 
 CDirectoryCache::~CDirectoryCache()
 {
-	for( auto & serverEntry : m_serverList ) {
-		for (auto & cacheEntry : serverEntry.cacheList ) {
-#ifdef __WXDEBUG__
+	for (auto & serverEntry : m_serverList) {
+		for (auto & cacheEntry : serverEntry.cacheList) {
+#ifndef NDEBUG
 			m_totalFileCount -= cacheEntry.listing.GetCount();
 #endif
 			tLruList::iterator* lruIt = (tLruList::iterator*)cacheEntry.lruIt;
@@ -19,17 +21,17 @@ CDirectoryCache::~CDirectoryCache()
 			}
 		}
 	}
-#ifdef __WXDEBUG__
-	wxASSERT(m_totalFileCount == 0);
+#ifndef NDEBUG
+	assert(m_totalFileCount == 0);
 #endif
 }
 
-void CDirectoryCache::Store(const CDirectoryListing &listing, const CServer &server)
+void CDirectoryCache::Store(CDirectoryListing const& listing, CServer const& server)
 {
 	fz::scoped_lock lock(mutex_);
 
 	tServerIter sit = CreateServerEntry(server);
-	wxASSERT(sit != m_serverList.end());
+	assert(sit != m_serverList.end());
 
 	m_totalFileCount += listing.GetCount();
 
@@ -52,13 +54,14 @@ void CDirectoryCache::Store(const CDirectoryListing &listing, const CServer &ser
 	Prune();
 }
 
-bool CDirectoryCache::Lookup(CDirectoryListing &listing, const CServer &server, const CServerPath &path, bool allowUnsureEntries, bool& is_outdated)
+bool CDirectoryCache::Lookup(CDirectoryListing &listing, CServer const& server, const CServerPath &path, bool allowUnsureEntries, bool& is_outdated)
 {
 	fz::scoped_lock lock(mutex_);
 
 	tServerIter sit = GetServerEntry(server);
-	if (sit == m_serverList.end())
+	if (sit == m_serverList.end()) {
 		return false;
+	}
 
 	tCacheIter iter;
 	if (Lookup(iter, sit, path, allowUnsureEntries, is_outdated)) {
@@ -69,7 +72,7 @@ bool CDirectoryCache::Lookup(CDirectoryListing &listing, const CServer &server,
 	return false;
 }
 
-bool CDirectoryCache::Lookup(tCacheIter &cacheIter, tServerIter &sit, const CServerPath &path, bool allowUnsureEntries, bool& is_outdated)
+bool CDirectoryCache::Lookup(tCacheIter &cacheIter, tServerIter &sit, CServerPath const& path, bool allowUnsureEntries, bool& is_outdated)
 {
 	static CCacheEntry dummy;
 	dummy.listing.path = path;
@@ -81,10 +84,11 @@ bool CDirectoryCache::Lookup(tCacheIter &cacheIter, tServerIter &sit, const CSer
 		if (entry.listing.path == path) {
 			UpdateLru(sit, cacheIter);
 
-			if (!allowUnsureEntries && entry.listing.get_unsure_flags())
+			if (!allowUnsureEntries && entry.listing.get_unsure_flags()) {
 				return false;
+			}
 
-			is_outdated = (fz::monotonic_clock::now() - entry.listing.m_firstListTime).get_seconds() > CACHE_TIMEOUT;
+			is_outdated = (fz::monotonic_clock::now() - entry.listing.m_firstListTime) > ttl_;
 			return true;
 		}
 	}
@@ -92,13 +96,14 @@ bool CDirectoryCache::Lookup(tCacheIter &cacheIter, tServerIter &sit, const CSer
 	return false;
 }
 
-bool CDirectoryCache::DoesExist(const CServer &server, const CServerPath &path, int &hasUnsureEntries, bool &is_outdated)
+bool CDirectoryCache::DoesExist(CServer const& server, CServerPath const& path, int &hasUnsureEntries, bool &is_outdated)
 {
 	fz::scoped_lock lock(mutex_);
 
 	tServerIter sit = GetServerEntry(server);
-	if (sit == m_serverList.end())
+	if (sit == m_serverList.end()) {
 		return false;
+	}
 
 	tCacheIter iter;
 	if (Lookup(iter, sit, path, true, is_outdated)) {
@@ -109,7 +114,7 @@ bool CDirectoryCache::DoesExist(const CServer &server, const CServerPath &path,
 	return false;
 }
 
-bool CDirectoryCache::LookupFile(CDirentry &entry, const CServer &server, const CServerPath &path, const wxString& file, bool &dirDidExist, bool &matchedCase)
+bool CDirectoryCache::LookupFile(CDirentry &entry, CServer const& server, CServerPath const& path, std::wstring const& filename, bool &dirDidExist, bool &matchedCase)
 {
 	fz::scoped_lock lock(mutex_);
 
@@ -130,13 +135,13 @@ bool CDirectoryCache::LookupFile(CDirentry &entry, const CServer &server, const
 	const CCacheEntry &cacheEntry = *iter;
 	const CDirectoryListing &listing = cacheEntry.listing;
 
-	int i = listing.FindFile_CmpCase(file);
+	int i = listing.FindFile_CmpCase(filename);
 	if (i >= 0) {
 		entry = listing[i];
 		matchedCase = true;
 		return true;
 	}
-	i = listing.FindFile_CmpNoCase(file);
+	i = listing.FindFile_CmpNoCase(filename);
 	if (i >= 0) {
 		entry = listing[i];
 		matchedCase = false;
@@ -146,26 +151,29 @@ bool CDirectoryCache::LookupFile(CDirentry &entry, const CServer &server, const
 	return false;
 }
 
-bool CDirectoryCache::InvalidateFile(const CServer &server, const CServerPath &path, const wxString& filename, bool *wasDir /*=false*/)
+bool CDirectoryCache::InvalidateFile(CServer const& server, CServerPath const& path, std::wstring const& filename, bool *wasDir)
 {
 	fz::scoped_lock lock(mutex_);
 
 	tServerIter sit = GetServerEntry(server);
-	if (sit == m_serverList.end())
+	if (sit == m_serverList.end()) {
 		return false;
+	}
 
 	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); ++iter) {
 		auto & entry = const_cast<CCacheEntry&>(*iter);
-		if (path.CmpNoCase(entry.listing.path))
+		if (path.CmpNoCase(entry.listing.path)) {
 			continue;
+		}
 
 		UpdateLru(sit, iter);
 
 		for (unsigned int i = 0; i < entry.listing.GetCount(); i++) {
-			if (!filename.CmpNoCase(((const CCacheEntry&)entry).listing[i].name)) {
-				if (wasDir)
+			if (!fz::stricmp(filename, entry.listing[i].name)) {
+				if (wasDir) {
 					*wasDir = entry.listing[i].is_dir();
-				entry.listing[i].flags |= CDirentry::flag_unsure;
+				}
+				entry.listing.get(i).flags |= CDirentry::flag_unsure;
 			}
 		}
 		entry.listing.m_flags |= CDirectoryListing::unsure_unknown;
@@ -175,28 +183,30 @@ bool CDirectoryCache::InvalidateFile(const CServer &server, const CServerPath &p
 	return true;
 }
 
-bool CDirectoryCache::UpdateFile(const CServer &server, const CServerPath &path, const wxString& filename, bool mayCreate, Filetype type, int64_t size)
+bool CDirectoryCache::UpdateFile(CServer const& server, CServerPath const& path, std::wstring const& filename, bool mayCreate, Filetype type, int64_t size)
 {
 	fz::scoped_lock lock(mutex_);
 
 	tServerIter sit = GetServerEntry(server);
-	if (sit == m_serverList.end())
+	if (sit == m_serverList.end()) {
 		return false;
+	}
 
 	bool updated = false;
 
 	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); ++iter) {
 		auto & entry = const_cast<CCacheEntry&>(*iter);
-		if (path.CmpNoCase(entry.listing.path))
+		if (path.CmpNoCase(entry.listing.path)) {
 			continue;
+		}
 
 		UpdateLru(sit, iter);
 
 		bool matchCase = false;
 		unsigned int i;
 		for (i = 0; i < entry.listing.GetCount(); ++i) {
-			if (!filename.CmpNoCase(entry.listing[i].name)) {
-				entry.listing[i].flags |= CDirentry::flag_unsure;
+			if (!fz::stricmp(filename, entry.listing[i].name)) {
+				entry.listing.get(i).flags |= CDirentry::flag_unsure;
 				if (entry.listing[i].name == filename) {
 					matchCase = true;
 					break;
@@ -206,20 +216,25 @@ bool CDirectoryCache::UpdateFile(const CServer &server, const CServerPath &path,
 
 		if (matchCase) {
 			Filetype old_type = entry.listing[i].is_dir() ? dir : file;
-			if (type != old_type)
+			if (type != old_type) {
 				entry.listing.m_flags |= CDirectoryListing::unsure_invalid;
-			else if (type == dir)
+			}
+			else if (type == dir) {
 				entry.listing.m_flags |= CDirectoryListing::unsure_dir_changed;
-			else
+			}
+			else {
 				entry.listing.m_flags |= CDirectoryListing::unsure_file_changed;
+			}
 		}
 		else if (type != unknown && mayCreate) {
 			CDirentry direntry;
 			direntry.name = filename;
-			if (type == dir)
+			if (type == dir) {
 				direntry.flags = CDirentry::flag_dir | CDirentry::flag_unsure;
-			else
+			}
+			else {
 				direntry.flags = CDirentry::flag_unsure;
+			}
 			direntry.size = size;
 			switch (type) {
 			case dir:
@@ -236,8 +251,9 @@ bool CDirectoryCache::UpdateFile(const CServer &server, const CServerPath &path,
 
 			++m_totalFileCount;
 		}
-		else
+		else {
 			entry.listing.m_flags |= CDirectoryListing::unsure_unknown;
+		}
 		entry.modificationTime = fz::monotonic_clock::now();
 
 		updated = true;
@@ -246,41 +262,47 @@ bool CDirectoryCache::UpdateFile(const CServer &server, const CServerPath &path,
 	return updated;
 }
 
-bool CDirectoryCache::RemoveFile(const CServer &server, const CServerPath &path, const wxString& filename)
+bool CDirectoryCache::RemoveFile(CServer const& server, CServerPath const& path, std::wstring const& filename)
 {
 	fz::scoped_lock lock(mutex_);
 
 	tServerIter sit = GetServerEntry(server);
-	if (sit == m_serverList.end())
+	if (sit == m_serverList.end()) {
 		return false;
+	}
 
 	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); ++iter) {
 		auto & entry = const_cast<CCacheEntry&>(*iter);
-		if (path.CmpNoCase(entry.listing.path))
+		if (path.CmpNoCase(entry.listing.path)) {
 			continue;
+		}
 
 		UpdateLru(sit, iter);
 
 		bool matchCase = false;
 		for (unsigned int i = 0; i < entry.listing.GetCount(); ++i) {
-			if (entry.listing[i].name == filename)
+			if (entry.listing[i].name == filename) {
 				matchCase = true;
+			}
 		}
 
 		if (matchCase) {
 			unsigned int i;
-			for (i = 0; i < entry.listing.GetCount(); i++)
-				if (entry.listing[i].name == filename)
+			for (i = 0; i < entry.listing.GetCount(); ++i) {
+				if (entry.listing[i].name == filename) {
 					break;
-			wxASSERT(i != entry.listing.GetCount());
+				}
+			}
+			assert(i != entry.listing.GetCount());
 
 			entry.listing.RemoveEntry(i); // This does set m_hasUnsureEntries
 			--m_totalFileCount;
 		}
 		else {
 			for (unsigned int i = 0; i < entry.listing.GetCount(); ++i) {
-				if (!filename.CmpNoCase(entry.listing[i].name))
-					entry.listing[i].flags |= CDirentry::flag_unsure;
+				if (!fz::stricmp(filename, entry.listing[i].name)) {
+					entry.listing.get(i).flags |= CDirentry::flag_unsure;
+				}
 			}
 			entry.listing.m_flags |= CDirectoryListing::unsure_invalid;
 		}
@@ -290,20 +312,18 @@ bool CDirectoryCache::RemoveFile(const CServer &server, const CServerPath &path,
 	return true;
 }
 
-void CDirectoryCache::InvalidateServer(const CServer& server)
+void CDirectoryCache::InvalidateServer(CServer const& server)
 {
 	fz::scoped_lock lock(mutex_);
 
-	for (auto iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
-	{
-		if (iter->server != server)
+	for (auto iter = m_serverList.begin(); iter != m_serverList.end(); ++iter) {
+		if (iter->server != server) {
 			continue;
+		}
 
-		for (tCacheIter cit = iter->cacheList.begin(); cit != iter->cacheList.end(); ++cit)
-		{
+		for (tCacheIter cit = iter->cacheList.begin(); cit != iter->cacheList.end(); ++cit) {
 			tLruList::iterator* lruIt = (tLruList::iterator*)cit->lruIt;
-			if (lruIt)
-			{
+			if (lruIt) {
 				m_leastRecentlyUsedList.erase(*lruIt);
 				delete lruIt;
 			}
@@ -316,13 +336,14 @@ void CDirectoryCache::InvalidateServer(const CServer& server)
 	}
 }
 
-bool CDirectoryCache::GetChangeTime(fz::monotonic_clock& time, const CServer &server, const CServerPath &path)
+bool CDirectoryCache::GetChangeTime(fz::monotonic_clock& time, CServer const& server, CServerPath const& path)
 {
 	fz::scoped_lock lock(mutex_);
 
 	tServerIter sit = GetServerEntry(server);
-	if (sit == m_serverList.end())
+	if (sit == m_serverList.end()) {
 		return false;
+	}
 
 	tCacheIter iter;
 	bool unused;
@@ -334,7 +355,7 @@ bool CDirectoryCache::GetChangeTime(fz::monotonic_clock& time, const CServer &se
 	return false;
 }
 
-void CDirectoryCache::RemoveDir(const CServer& server, const CServerPath& path, const wxString& filename, const CServerPath&)
+void CDirectoryCache::RemoveDir(CServer const& server, CServerPath const& path, std::wstring const& filename, CServerPath const&)
 {
 	fz::scoped_lock lock(mutex_);
 
@@ -342,12 +363,14 @@ void CDirectoryCache::RemoveDir(const CServer& server, const CServerPath& path,
 	// Perhaps just throw away the complete cache?
 
 	tServerIter sit = GetServerEntry(server);
-	if (sit == m_serverList.end())
+	if (sit == m_serverList.end()) {
 		return;
+	}
 
 	CServerPath absolutePath = path;
-	if (!absolutePath.AddSegment(filename))
+	if (!absolutePath.AddSegment(filename)) {
 		absolutePath.clear();
+	}
 
 	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); ) {
 		auto & entry = const_cast<CCacheEntry&>(*iter);
@@ -369,40 +392,37 @@ void CDirectoryCache::RemoveDir(const CServer& server, const CServerPath& path,
 	RemoveFile(server, path, filename);
 }
 
-void CDirectoryCache::Rename(const CServer& server, const CServerPath& pathFrom, const wxString& fileFrom, const CServerPath& pathTo, const wxString& fileTo)
+void CDirectoryCache::Rename(CServer const& server, CServerPath const& pathFrom, std::wstring const& fileFrom, CServerPath const& pathTo, std::wstring const& fileTo)
 {
 	fz::scoped_lock lock(mutex_);
 
 	tServerIter sit = GetServerEntry(server);
-	if (sit == m_serverList.end())
+	if (sit == m_serverList.end()) {
 		return;
+	}
 
 	tCacheIter iter;
 	bool is_outdated = false;
 	bool found = Lookup(iter, sit, pathFrom, true, is_outdated);
 	if (found) {
 		auto & listing = const_cast<CDirectoryListing&>(iter->listing);
-		if (pathFrom == pathTo)
-		{
+		if (pathFrom == pathTo) {
 			RemoveFile(server, pathFrom, fileTo);
 			unsigned int i;
-			for (i = 0; i < listing.GetCount(); i++)
-			{
-				if (listing[i].name == fileFrom)
+			for (i = 0; i < listing.GetCount(); ++i) {
+				if (listing[i].name == fileFrom) {
 					break;
+				}
 			}
-			if (i != listing.GetCount())
-			{
-				if (listing[i].is_dir())
-				{
+			if (i != listing.GetCount()) {
+				if (listing[i].is_dir()) {
 					RemoveDir(server, pathFrom, fileFrom, CServerPath());
 					RemoveDir(server, pathFrom, fileTo, CServerPath());
 					UpdateFile(server, pathFrom, fileTo, true, dir);
 				}
-				else
-				{
-					listing[i].name = fileTo;
-					listing[i].flags |= CDirentry::flag_unsure;
+				else {
+					listing.get(i).name = fileTo;
+					listing.get(i).flags |= CDirentry::flag_unsure;
 					listing.m_flags |= CDirectoryListing::unsure_unknown;
 					listing.ClearFindMap();
 				}
@@ -411,9 +431,10 @@ void CDirectoryCache::Rename(const CServer& server, const CServerPath& pathFrom,
 		}
 		else {
 			unsigned int i;
-			for (i = 0; i < listing.GetCount(); i++) {
-				if (listing[i].name == fileFrom)
+			for (i = 0; i < listing.GetCount(); ++i) {
+				if (listing[i].name == fileFrom) {
 					break;
+				}
 			}
 			if (i != listing.GetCount()) {
 				if (listing[i].is_dir()) {
@@ -433,23 +454,25 @@ void CDirectoryCache::Rename(const CServer& server, const CServerPath& pathFrom,
 	InvalidateServer(server);
 }
 
-CDirectoryCache::tServerIter CDirectoryCache::CreateServerEntry(const CServer& server)
+CDirectoryCache::tServerIter CDirectoryCache::CreateServerEntry(CServer const& server)
 {
 	for (tServerIter iter = m_serverList.begin(); iter != m_serverList.end(); ++iter) {
-		if (iter->server == server)
+		if (iter->server == server) {
 			return iter;
+		}
 	}
 	m_serverList.emplace_back(server);
 
 	return --m_serverList.end();
 }
 
-CDirectoryCache::tServerIter CDirectoryCache::GetServerEntry(const CServer& server)
+CDirectoryCache::tServerIter CDirectoryCache::GetServerEntry(CServer const& server)
 {
 	tServerIter iter;
 	for (iter = m_serverList.begin(); iter != m_serverList.end(); ++iter) {
-		if (iter->server == server)
+		if (iter->server == server) {
 			break;
+		}
 	}
 
 	return iter;
@@ -481,9 +504,23 @@ void CDirectoryCache::Prune()
 		m_totalFileCount -= pos.second->listing.GetCount();
 
 		pos.first->cacheList.erase(pos.second);
-		if (pos.first->cacheList.empty())
+		if (pos.first->cacheList.empty()) {
 			m_serverList.erase(pos.first);
+		}
 
 		m_leastRecentlyUsedList.pop_front();
 	}
 }
+
+void CDirectoryCache::SetTtl(fz::duration const& ttl)
+{
+	if (ttl < fz::duration::from_seconds(30)) {
+		ttl_ = fz::duration::from_seconds(30);
+	}
+	else if (ttl > fz::duration::from_days(1)) {
+		ttl_ = fz::duration::from_days(1);
+	}
+	else {
+		ttl_ = ttl;
+	}
+}
