@@ -70,7 +70,7 @@ bool CLocalPath::SetPath(std::wstring const& path, std::wstring* file)
 					return false;
 				}
 				in++;
-				if (((*in >= 'a' && *in <= 'z') || (*in >= 'A' || *in <= 'Z')) && *(in+1) == ':') {
+				if (((*in >= 'a' && *in <= 'z') || (*in >= 'A' && *in <= 'Z')) && *(in+1) == ':') {
 					// It's \\?\c:\foo\bar
 					goto parse_regular;
 				}
@@ -104,7 +104,7 @@ bool CLocalPath::SetPath(std::wstring const& path, std::wstring* file)
 
 			segments.push_back(out);
 		}
-		else if ((*in >= 'a' && *in <= 'z') || (*in >= 'A' || *in <= 'Z')) {
+		else if ((*in >= 'a' && *in <= 'z') || (*in >= 'A' && *in <= 'Z')) {
 parse_regular:
 			// Regular path
 			*out++ = *in++;
@@ -555,11 +555,13 @@ bool CLocalPath::IsParentOf(const CLocalPath &path) const
 	}
 
 #ifdef FZ_WINDOWS
-	if (fz::stricmp(*m_path, path.m_path->substr(0, m_path->size())))
+	if (fz::stricmp(*m_path, path.m_path->substr(0, m_path->size()))) {
 		return false;
+	}
 #else
-	if (*m_path != path.m_path->substr(0, m_path->size()))
+	if (*m_path != path.m_path->substr(0, m_path->size())) {
 		return false;
+	}
 #endif
 
 	return true;
@@ -567,18 +569,22 @@ bool CLocalPath::IsParentOf(const CLocalPath &path) const
 
 bool CLocalPath::IsSubdirOf(const CLocalPath &path) const
 {
-	if (empty() || path.empty())
+	if (empty() || path.empty()) {
 		return false;
+	}
 
-	if (path.m_path->size() > m_path->size())
+	if (path.m_path->size() > m_path->size()) {
 		return false;
+	}
 
 #ifdef FZ_WINDOWS
-	if (fz::stricmp(*path.m_path, m_path->substr(0, path.m_path->size())))
+	if (fz::stricmp(*path.m_path, m_path->substr(0, path.m_path->size()))) {
 		return false;
+	}
 #else
-	if (*path.m_path != m_path->substr(0, path.m_path->size()))
+	if (*path.m_path != m_path->substr(0, path.m_path->size())) {
 		return false;
+	}
 #endif
 
 	return true;
@@ -623,7 +629,7 @@ bool CLocalPath::Create(CLocalPath *last_successful)
 		local_path.AddSegment(*iter);
 
 	#ifdef FZ_WINDOWS
-		BOOL res = CreateDirectory(local_path.GetPath().c_str(), 0);
+		BOOL res = CreateDirectory(local_path.GetPath().c_str(), nullptr);
 		if (!res && GetLastError() != ERROR_ALREADY_EXISTS) {
 			ret = false;
 			break;
