@@ -96,7 +96,7 @@ bool CLogging::InitLogFile(fz::scoped_lock& l) const
 		return false;
 
 #ifdef FZ_WINDOWS
-	m_log_fd = CreateFile(m_file.c_str(), FILE_APPEND_DATA, FILE_SHARE_DELETE | FILE_SHARE_WRITE | FILE_SHARE_READ, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
+	m_log_fd = CreateFile(m_file.c_str(), FILE_APPEND_DATA, FILE_SHARE_DELETE | FILE_SHARE_WRITE | FILE_SHARE_READ, nullptr, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, nullptr);
 	if (m_log_fd == INVALID_HANDLE_VALUE) {
 		DWORD err = GetLastError();
 #else
@@ -172,7 +172,7 @@ void CLogging::LogToFile(MessageType nMessageType, std::wstring const& msg) cons
 
 			// m_log_fd might no longer be the original file.
 			// Recheck on a new handle. Proteced with a mutex against other processes
-			HANDLE hMutex = ::CreateMutexW(0, true, L"FileZilla 3 Logrotate Mutex");
+			HANDLE hMutex = ::CreateMutexW(nullptr, true, L"FileZilla 3 Logrotate Mutex");
 			if (!hMutex) {
 				DWORD err = GetLastError();
 				l.unlock();
@@ -180,7 +180,7 @@ void CLogging::LogToFile(MessageType nMessageType, std::wstring const& msg) cons
 				return;
 			}
 
-			HANDLE hFile = CreateFileW(m_file.c_str(), FILE_APPEND_DATA, FILE_SHARE_DELETE | FILE_SHARE_WRITE | FILE_SHARE_READ, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
+			HANDLE hFile = CreateFileW(m_file.c_str(), FILE_APPEND_DATA, FILE_SHARE_DELETE | FILE_SHARE_WRITE | FILE_SHARE_READ, nullptr, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, nullptr);
 			if (hFile == INVALID_HANDLE_VALUE) {
 				DWORD err = GetLastError();
 
@@ -216,7 +216,7 @@ void CLogging::LogToFile(MessageType nMessageType, std::wstring const& msg) cons
 					}
 				}
 				MoveFileExW(m_file.c_str(), (m_file + L".1").c_str(), MOVEFILE_REPLACE_EXISTING);
-				m_log_fd = CreateFileW(m_file.c_str(), FILE_APPEND_DATA, FILE_SHARE_DELETE | FILE_SHARE_WRITE | FILE_SHARE_READ, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
+				m_log_fd = CreateFileW(m_file.c_str(), FILE_APPEND_DATA, FILE_SHARE_DELETE | FILE_SHARE_WRITE | FILE_SHARE_READ, nullptr, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, nullptr);
 				if (m_log_fd == INVALID_HANDLE_VALUE) {
 					// If this function would return bool, I'd return FILE_NOT_FOUND here.
 					err = GetLastError();
@@ -240,7 +240,7 @@ void CLogging::LogToFile(MessageType nMessageType, std::wstring const& msg) cons
 	}
 	DWORD len = out.size();
 	DWORD written;
-	BOOL res = WriteFile(m_log_fd, out.c_str(), len, &written, 0);
+	BOOL res = WriteFile(m_log_fd, out.c_str(), len, &written, nullptr);
 	if (!res || written != len) {
 		DWORD err = GetLastError();
 		CloseHandle(m_log_fd);
