@@ -31,21 +31,22 @@ enum handshake_state
 	socks4_handshake
 };
 
-CProxySocket::CProxySocket(fz::event_handler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner)
+CProxySocket::CProxySocket(fz::event_handler* pEvtHandler, fz::socket* pSocket, CControlSocket* pOwner)
 	: fz::event_handler(pOwner->event_loop_)
 	, CBackend(pEvtHandler)
-	, m_pSocket(pSocket)
+	, socket_(pSocket)
 	, m_pOwner(pOwner)
 {
-	m_pSocket->SetEventHandler(this);
+	socket_->set_event_handler(this);
 }
 
 CProxySocket::~CProxySocket()
 {
 	remove_handler();
 
-	if (m_pSocket)
-		m_pSocket->SetEventHandler(0);
+	if (socket_) {
+		socket_->set_event_handler(nullptr);
+	}
 	delete [] m_pSendBuffer;
 	delete [] m_pRecvBuffer;
 }
@@ -100,10 +101,11 @@ int CProxySocket::Handshake(CProxySocket::ProxyType type, std::wstring const& ho
 		std::string host_raw = fz::to_utf8(host);
 		m_pSendBuffer = new char[70 + host_raw.size() * 2 + 2*5 + auth.size() + 23];
 
-		m_sendBufferLen = sprintf(m_pSendBuffer, "CONNECT %s:%u HTTP/1.1\r\nHost: %s:%u\r\n%sUser-Agent: FileZilla\r\n\r\n",
+		m_sendBufferLen = sprintf(m_pSendBuffer, "CONNECT %s:%u HTTP/1.1\r\nHost: %s:%u\r\n%sUser-Agent: %s\r\n\r\n",
 			host_raw.c_str(), port,
 			host_raw.c_str(), port,
-			auth.c_str());
+			auth.c_str(),
+			fz::replaced_substrings(PACKAGE_STRING, " ", "/").c_str());
 
 		m_pRecvBuffer = new char[4096];
 		m_recvBufferLen = 4096;
@@ -125,10 +127,10 @@ int CProxySocket::Handshake(CProxySocket::ProxyType type, std::wstring const& ho
 			hints.ai_socktype = SOCK_STREAM;
 
 			addrinfo * result{};
-			int res = getaddrinfo(fz::to_string(m_host).c_str(), 0, &hints, &result);
+			int res = getaddrinfo(fz::to_string(m_host).c_str(), nullptr, &hints, &result);
 			if (!res && result) {
 				if (result->ai_family == AF_INET) {
-					ip = CSocket::AddressToString(result->ai_addr, result->ai_addrlen, false);
+					ip = fz::socket::address_to_string(result->ai_addr, result->ai_addrlen, false);
 				}
 				freeaddrinfo(result);
 			}
@@ -197,60 +199,64 @@ int CProxySocket::Handshake(CProxySocket::ProxyType type, std::wstring const& ho
 
 void CProxySocket::operator()(fz::event_base const& ev)
 {
-	fz::dispatch<CSocketEvent, CHostAddressEvent>(ev, this,
+	fz::dispatch<fz::socket_event, fz::hostaddress_event>(ev, this,
 		&CProxySocket::OnSocketEvent,
 		&CProxySocket::OnHostAddress);
 }
 
-void CProxySocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error)
+void CProxySocket::OnSocketEvent(socket_event_source*, fz::socket_event_flag t, int error)
 {
 	switch (t) {
-	case SocketEventType::connection_next:
-		if (error)
-			m_pOwner->LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\", trying next address."), CSocket::GetErrorDescription(error));
-		break;
-	case SocketEventType::connection:
+	case fz::socket_event_flag::connection_next:
 		if (error) {
-			if (m_proxyState == handshake)
+			m_pOwner->LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\", trying next address."), fz::socket::error_description(error));
+		}
+		break;
+	case fz::socket_event_flag::connection:
+		if (error) {
+			if (m_proxyState == handshake) {
 				m_proxyState = noconn;
-			m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::connection, error);
+			}
+			m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::connection, error);
 		}
 		else {
 			m_pOwner->LogMessage(MessageType::Status, _("Connection with proxy established, performing handshake..."));
 		}
 		break;
-	case SocketEventType::read:
+	case fz::socket_event_flag::read:
 		OnReceive();
 		break;
-	case SocketEventType::write:
+	case fz::socket_event_flag::write:
 		OnSend();
 		break;
-	case SocketEventType::close:
+	case fz::socket_event_flag::close:
 		OnReceive();
 		break;
 	}
 }
 
-void CProxySocket::OnHostAddress(CSocketEventSource*, std::string const& address)
+void CProxySocket::OnHostAddress(socket_event_source*, std::string const& address)
 {
 	m_pOwner->LogMessage(MessageType::Status, _("Connecting to %s..."), address);
 }
 
 void CProxySocket::Detach()
 {
-	if (!m_pSocket)
+	if (!socket_) {
 		return;
+	}
 
-	m_pSocket->SetEventHandler(0);
-	m_pSocket = 0;
+	socket_->set_event_handler(nullptr);
+	socket_ = nullptr;
 }
 
 void CProxySocket::OnReceive()
 {
 	m_can_read = true;
 
-	if (m_proxyState != handshake)
+	if (m_proxyState != handshake) {
 		return;
+	}
 
 	switch (m_handshakeState)
 	{
@@ -258,31 +264,34 @@ void CProxySocket::OnReceive()
 		for (;;) {
 			int error;
 			int do_read = m_recvBufferLen - m_recvBufferPos - 1;
-			char* end = 0;
+			char* end = nullptr;
 			for (int i = 0; i < 2; ++i) {
 				int read;
-				if (!i)
-					read = m_pSocket->Peek(m_pRecvBuffer + m_recvBufferPos, do_read, error);
-				else
-					read = m_pSocket->Read(m_pRecvBuffer + m_recvBufferPos, do_read, error);
+				if (!i) {
+					read = socket_->peek(m_pRecvBuffer + m_recvBufferPos, do_read, error);
+				}
+				else {
+					read = socket_->read(m_pRecvBuffer + m_recvBufferPos, do_read, error);
+				}
 				if (read == -1) {
 					if (error != EAGAIN) {
 						m_proxyState = noconn;
-						m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, error);
+						m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, error);
 					}
-					else
+					else {
 						m_can_read = false;
+					}
 					return;
 				}
 				if (!read) {
 					m_proxyState = noconn;
-					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 					return;
 				}
 				if (m_pSendBuffer) {
 					m_proxyState = noconn;
 					m_pOwner->LogMessage(MessageType::Debug_Warning, L"Incoming data before request fully sent");
-					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 					return;
 				}
 
@@ -294,27 +303,29 @@ void CProxySocket::OnReceive()
 						if (m_recvBufferPos + read + 1 == m_recvBufferLen) {
 							m_proxyState = noconn;
 							m_pOwner->LogMessage(MessageType::Debug_Warning, L"Incoming header too large");
-							m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ENOMEM);
+							m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ENOMEM);
 							return;
 						}
 						do_read = read;
 					}
-					else
+					else {
 						do_read = end - m_pRecvBuffer + 4 - m_recvBufferPos;
+					}
 				}
 				else {
 					if (read != do_read) {
 						m_proxyState = noconn;
 						m_pOwner->LogMessage(MessageType::Debug_Warning, "Could not read what got peeked");
-						m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+						m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 						return;
 					}
 					m_recvBufferPos += read;
 				}
 			}
 
-			if (!end)
+			if (!end) {
 				continue;
+			}
 
 			end = strchr(m_pRecvBuffer, '\r'); // Never fails as old value of end exists and starts with CR, we just look for an earlier case.
 			*end = 0;
@@ -323,38 +334,40 @@ void CProxySocket::OnReceive()
 
 			if (reply.substr(0, 10) != L"HTTP/1.1 2" && reply.substr(0, 10) != L"HTTP/1.0 2") {
 				m_proxyState = noconn;
-				m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNRESET);
+				m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNRESET);
 				return;
 			}
 
 			m_proxyState = conn;
-			m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::connection, 0);
+			m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::connection, 0);
 			return;
 		}
 	case socks4_handshake:
 		while (m_recvBufferLen && m_can_read && m_proxyState == handshake) {
 			int read_error;
-			int read = m_pSocket->Read(m_pRecvBuffer + m_recvBufferPos, m_recvBufferLen, read_error);
+			int read = socket_->read(m_pRecvBuffer + m_recvBufferPos, m_recvBufferLen, read_error);
 			if (read == -1) {
 				if (read_error != EAGAIN) {
 					m_proxyState = noconn;
-					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, read_error);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, read_error);
 				}
-				else
+				else {
 					m_can_read = false;
+				}
 				return;
 			}
 
 			if (!read) {
 				m_proxyState = noconn;
-				m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+				m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 				return;
 			}
 			m_recvBufferPos += read;
 			m_recvBufferLen -= read;
 
-			if (m_recvBufferLen)
+			if (m_recvBufferLen) {
 				continue;
+			}
 
 			m_recvBufferPos = 0;
 
@@ -376,11 +389,11 @@ void CProxySocket::OnReceive()
 				}
 				m_pOwner->LogMessage(MessageType::Error, _("Proxy request failed: %s"), error);
 				m_proxyState = noconn;
-				m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+				m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 				return;
 			}
 			m_proxyState = conn;
-			m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::connection, 0);
+			m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::connection, 0);
 		}
 		return;
 	case socks5_method:
@@ -388,30 +401,33 @@ void CProxySocket::OnReceive()
 	case socks5_request:
 	case socks5_request_addrtype:
 	case socks5_request_address:
-		if (m_pSendBuffer)
+		if (m_pSendBuffer) {
 			return;
+		}
 		while (m_recvBufferLen && m_can_read && m_proxyState == handshake) {
 			int error;
-			int read = m_pSocket->Read(m_pRecvBuffer + m_recvBufferPos, m_recvBufferLen, error);
+			int read = socket_->read(m_pRecvBuffer + m_recvBufferPos, m_recvBufferLen, error);
 			if (read == -1) {
 				if (error != EAGAIN) {
 					m_proxyState = noconn;
-					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, error);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, error);
 				}
-				else
+				else {
 					m_can_read = false;
+				}
 				return;
 			}
 			if (!read) {
 				m_proxyState = noconn;
-				m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+				m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 				return;
 			}
 			m_recvBufferPos += read;
 			m_recvBufferLen -= read;
 
-			if (m_recvBufferLen)
+			if (m_recvBufferLen) {
 				continue;
+			}
 
 			m_recvBufferPos = 0;
 
@@ -421,7 +437,7 @@ void CProxySocket::OnReceive()
 				if (m_pRecvBuffer[0] != 5) {
 					m_pOwner->LogMessage(MessageType::Error, _("Unknown SOCKS protocol version: %d"), (int)m_pRecvBuffer[0]);
 					m_proxyState = noconn;
-					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 					return;
 				}
 				break;
@@ -429,7 +445,7 @@ void CProxySocket::OnReceive()
 				if (m_pRecvBuffer[0] != 1) {
 					m_pOwner->LogMessage(MessageType::Error, _("Unknown protocol version of SOCKS Username/Password Authentication subnegotiation: %d"), (int)m_pRecvBuffer[0]);
 					m_proxyState = noconn;
-					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 					return;
 				}
 				break;
@@ -453,7 +469,7 @@ void CProxySocket::OnReceive()
 					default:
 						m_pOwner->LogMessage(MessageType::Error, _("No supported SOCKS5 auth method"));
 						m_proxyState = noconn;
-						m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+						m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 						return;
 					}
 				}
@@ -462,7 +478,7 @@ void CProxySocket::OnReceive()
 				if (m_pRecvBuffer[1] != 0) {
 					m_pOwner->LogMessage(MessageType::Error, _("Proxy authentication failed"));
 					m_proxyState = noconn;
-					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 					return;
 				}
 				m_handshakeState = socks5_request;
@@ -503,7 +519,7 @@ void CProxySocket::OnReceive()
 
 					m_pOwner->LogMessage(MessageType::Error, _("Proxy request failed. Reply from proxy: %s"), errorMsg);
 					m_proxyState = noconn;
-					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 					return;
 				}
 				m_handshakeState = socks5_request_addrtype;
@@ -527,7 +543,7 @@ void CProxySocket::OnReceive()
 				default:
 					m_pOwner->LogMessage(MessageType::Error, _("Proxy request failed: Unknown address type in CONNECT reply"));
 					m_proxyState = noconn;
-					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 					return;
 				}
 				m_handshakeState = socks5_request_address;
@@ -536,7 +552,7 @@ void CProxySocket::OnReceive()
 				{
 					// We're done
 					m_proxyState = conn;
-					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::connection, 0);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::connection, 0);
 					return;
 				}
 			default:
@@ -548,13 +564,15 @@ void CProxySocket::OnReceive()
 			{
 			case socks5_auth:
 				{
-					m_sendBufferLen = m_user.size() + m_pass.size() + 3;
+					auto ulen = static_cast<unsigned char>(std::min(m_user.size(), size_t(255)));
+					auto plen = static_cast<unsigned char>(std::min(m_pass.size(), size_t(255)));
+					m_sendBufferLen = ulen + plen + 3;
 					m_pSendBuffer = new char[m_sendBufferLen];
 					m_pSendBuffer[0] = 1;
-					m_pSendBuffer[1] = m_user.size();
-					memcpy(m_pSendBuffer + 2, m_user.c_str(), m_user.size());
-					m_pSendBuffer[m_user.size() + 2] = m_pass.size();
-					memcpy(m_pSendBuffer + m_user.size() + 3, m_pass.c_str(), m_pass.size());
+					m_pSendBuffer[1] = ulen;
+					memcpy(m_pSendBuffer + 2, m_user.c_str(), ulen);
+					m_pSendBuffer[ulen + 2] = plen;
+					memcpy(m_pSendBuffer + ulen + 3, m_pass.c_str(), plen);
 					m_recvBufferLen = 2;
 				}
 				break;
@@ -598,9 +616,11 @@ void CProxySocket::OnReceive()
 					}
 					else {
 						m_pSendBuffer[3] = 3; // Domain name
-						m_pSendBuffer[4] = host.size();
-						memcpy(m_pSendBuffer + 5, host.c_str(), host.size());
-						addrlen = host.size() + 1;
+
+						auto hlen = static_cast<unsigned char>(std::min(host.size(), size_t(255)));
+						m_pSendBuffer[4] = hlen;
+						memcpy(m_pSendBuffer + 5, host.c_str(), hlen);
+						addrlen = hlen + 1;
 					}
 
 					m_pSendBuffer[addrlen + 4] = (m_port >> 8) & 0xFF; // Port in network order
@@ -618,14 +638,15 @@ void CProxySocket::OnReceive()
 				assert(false);
 				break;
 			}
-			if (m_pSendBuffer && m_can_write)
+			if (m_pSendBuffer && m_can_write) {
 				OnSend();
+			}
 		}
 		break;
 	default:
 		m_proxyState = noconn;
 		m_pOwner->LogMessage(MessageType::Debug_Warning, L"Unhandled handshake state %d", m_handshakeState);
-		m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+		m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 		return;
 	}
 }
@@ -633,29 +654,32 @@ void CProxySocket::OnReceive()
 void CProxySocket::OnSend()
 {
 	m_can_write = true;
-	if (m_proxyState != handshake || !m_pSendBuffer)
+	if (m_proxyState != handshake || !m_pSendBuffer) {
 		return;
+	}
 
 	for (;;) {
 		int error;
-		int written = m_pSocket->Write(m_pSendBuffer, m_sendBufferLen, error);
+		int written = socket_->write(m_pSendBuffer, m_sendBufferLen, error);
 		if (written == -1) {
 			if (error != EAGAIN) {
 				m_proxyState = noconn;
-				m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, error);
+				m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, error);
 			}
-			else
+			else {
 				m_can_write = false;
+			}
 
 			return;
 		}
 
 		if (written == m_sendBufferLen) {
 			delete [] m_pSendBuffer;
-			m_pSendBuffer = 0;
+			m_pSendBuffer = nullptr;
 
-			if (m_can_read)
+			if (m_can_read) {
 				OnReceive();
+			}
 			return;
 		}
 		memmove(m_pSendBuffer, m_pSendBuffer + written, m_sendBufferLen - written);
