@@ -1,42 +1,44 @@
 #include <filezilla.h>
 #include "serverpath.h"
 
-#define FTP_MVS_DOUBLE_QUOTE (wxChar)0xDC
+#define FTP_MVS_DOUBLE_QUOTE (wchar_t)0xDC
 
 struct CServerTypeTraits
 {
-	const wxChar* separators;
+	wchar_t const* separators;
 	bool has_root; // Root = simply separator nothing else
-	wxChar left_enclosure; // Example: VMS paths: [FOO.BAR]
-	wxChar right_enclosure;
+	wchar_t left_enclosure; // Example: VMS paths: [FOO.BAR]
+	wchar_t right_enclosure;
 	bool filename_inside_enclosure; // MVS
 	int prefixmode; //0 = normal prefix, 1 = suffix
-	wxChar separatorEscape;
+	wchar_t separatorEscape;
 	bool has_dots; // Special meaning for .. (parent) and . (self)
 	bool separator_after_prefix;
 };
 
 static const CServerTypeTraits traits[SERVERTYPE_MAX] = {
-	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  false }, // Failsafe
-	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  false },
-	{ _T("."),   false,  '[',  ']',    false, 0, '^', false, false },
-	{ _T("\\/"), false,    0,    0,    false, 0, 0,   true,  false }, // DOS with backslashes
-	{ _T("."),   false, '\'', '\'',     true, 1, 0,   false, false },
-	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  false },
-	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  false }, // Same as Unix
-	{ _T("."),   false,    0,    0,    false, 0, 0,   false, false },
-	{ _T("\\"),  true,     0,    0,    false, 0, 0,   true,  false },
-	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  true  },  // Cygwin is like Unix but has optional prefix of form "//server"
-	{ _T("/\\"), false,    0,    0,    false, 0, 0,   true,  false } // DOS with forwardslashes
+	{ L"/",   true,     0,    0,    false, 0, 0,   true,  false }, // Failsafe
+	{ L"/",   true,     0,    0,    false, 0, 0,   true,  false },
+	{ L".",   false,  '[',  ']',    false, 0, '^', false, false },
+	{ L"\\/", false,    0,    0,    false, 0, 0,   true,  false }, // DOS with backslashes
+	{ L".",   false, '\'', '\'',     true, 1, 0,   false, false },
+	{ L"/",   true,     0,    0,    false, 0, 0,   true,  false },
+	{ L"/",   true,     0,    0,    false, 0, 0,   true,  false }, // Same as Unix
+	{ L".",   false,    0,    0,    false, 0, 0,   false, false },
+	{ L"\\",  true,     0,    0,    false, 0, 0,   true,  false },
+	{ L"/",   true,     0,    0,    false, 0, 0,   true,  true  },  // Cygwin is like Unix but has optional prefix of form "//server"
+	{ L"/\\", false,    0,    0,    false, 0, 0,   true,  false } // DOS with forwardslashes
 };
 
-bool CServerPathData::operator==(const CServerPathData& cmp) const
+bool CServerPathData::operator==(CServerPathData const& cmp) const
 {
-	if (m_prefix != cmp.m_prefix)
+	if (m_prefix != cmp.m_prefix) {
 		return false;
+	}
 
-	if (m_segments != cmp.m_segments)
+	if (m_segments != cmp.m_segments) {
 		return false;
+	}
 
 	return true;
 }
@@ -46,7 +48,7 @@ CServerPath::CServerPath()
 {
 }
 
-CServerPath::CServerPath(CServerPath const& path, wxString subdir)
+CServerPath::CServerPath(CServerPath const& path, std::wstring subdir)
 	: m_type(path.m_type)
 	, m_data(path.m_data)
 {
@@ -59,7 +61,7 @@ CServerPath::CServerPath(CServerPath const& path, wxString subdir)
 	}
 }
 
-CServerPath::CServerPath(wxString const& path, ServerType type /*=DEFAULT*/)
+CServerPath::CServerPath(std::wstring const& path, ServerType type)
 	: m_type(type)
 {
 	SetPath(path);
@@ -70,119 +72,141 @@ void CServerPath::clear()
 	m_data.clear();
 }
 
-bool CServerPath::SetPath(wxString newPath)
+bool CServerPath::SetPath(std::wstring newPath)
 {
 	return SetPath(newPath, false);
 }
 
-bool CServerPath::SetPath(wxString &newPath, bool isFile)
+bool CServerPath::SetPath(std::wstring &newPath, bool isFile)
 {
-	wxString path = newPath;
-	wxString file;
+	std::wstring path = newPath;
 
 	if (path.empty()) {
 		return false;
 	}
 
 	if (m_type == DEFAULT) {
-		int pos1 = path.Find(_T(":["));
-		if (pos1 != -1) {
-			int pos2 = path.Find(']', true);
-			if (pos2 != -1 && static_cast<size_t>(pos2) == (path.Length() - 1) && !isFile)
+		size_t pos1 = path.find(L":[");
+		if (pos1 != std::wstring::npos) {
+			size_t pos2 = path.rfind(']');
+			if (pos2 != std::string::npos && pos2 == (path.size() - 1) && !isFile) {
 				m_type = VMS;
-			else if (isFile && pos2 > pos1)
+			}
+			else if (isFile && pos2 > pos1) {
 				m_type = VMS;
+			}
 		}
-		else if (path.Length() >= 3 &&
-			((path.c_str()[0] >= 'A' && path.c_str()[0] <= 'Z') || (path.c_str()[0] >= 'a' && path.c_str()[0] <= 'z')) &&
-			path.c_str()[1] == ':' && (path.c_str()[2] == '\\' || path.c_str()[2] == '/'))
-				m_type = DOS;
-		else if (path.c_str()[0] == FTP_MVS_DOUBLE_QUOTE && path.Last() == FTP_MVS_DOUBLE_QUOTE)
+		else if (path.size() >= 3 &&
+			((path[0] >= 'A' && path[0] <= 'Z') || (path[0] >= 'a' && path[0] <= 'z')) &&
+			path[1] == ':' && (path[2] == '\\' || path[2] == '/'))
+		{
+			m_type = DOS;
+		}
+		else if (path[0] == FTP_MVS_DOUBLE_QUOTE && path.back() == FTP_MVS_DOUBLE_QUOTE) {
 			m_type = MVS;
-		else if (path[0] == ':' && (pos1 = path.Mid(1).Find(':')) > 0) {
-			int slash = path.Find('/');
-			if (slash == -1 || slash > pos1)
-				m_type = VXWORKS;
 		}
-		else if (path[0] == '\\')
+		else if (path[0] == ':' && (pos1 = path.find(':'), 2) != std::wstring::npos) {
+			size_t slash = path.find('/');
+			if (slash == std::wstring::npos || slash > pos1) {
+				m_type = VXWORKS;
+			}
+		}
+		else if (path[0] == '\\') {
 			m_type = DOS_VIRTUAL;
+		}
 
-		if (m_type == DEFAULT)
+		if (m_type == DEFAULT) {
 			m_type = UNIX;
+		}
 	}
 
 	m_data.clear();
 
-	if (!ChangePath(path, isFile))
+	if (!ChangePath(path, isFile)) {
 		return false;
+	}
 
-	if (isFile)
+	if (isFile) {
 		newPath = path;
+	}
 	return true;
 }
 
-wxString CServerPath::GetPath() const
+std::wstring CServerPath::GetPath() const
 {
-	if (empty())
-		return wxString();
+	if (empty()) {
+		return std::wstring();
+	}
 
-	wxString path;
+	std::wstring path;
 
-	if (!traits[m_type].prefixmode && m_data->m_prefix)
+	if (!traits[m_type].prefixmode && m_data->m_prefix) {
 		path = *m_data->m_prefix;
+	}
 
-	if (traits[m_type].left_enclosure != 0)
+	if (traits[m_type].left_enclosure != 0) {
 		path += traits[m_type].left_enclosure;
-	if (m_data->m_segments.empty() && (!traits[m_type].has_root || !m_data->m_prefix || traits[m_type].separator_after_prefix))
+	}
+	if (m_data->m_segments.empty() && (!traits[m_type].has_root || !m_data->m_prefix || traits[m_type].separator_after_prefix)) {
 		path += traits[m_type].separators[0];
+	}
 
 	for (auto iter = m_data->m_segments.cbegin(); iter != m_data->m_segments.cend(); ++iter) {
 		std::wstring const& segment = *iter;
-		if (iter != m_data->m_segments.cbegin())
+		if (iter != m_data->m_segments.cbegin()) {
 			path += traits[m_type].separators[0];
+		}
 		else if (traits[m_type].has_root) {
-			if (!m_data->m_prefix || traits[m_type].separator_after_prefix)
+			if (!m_data->m_prefix || traits[m_type].separator_after_prefix) {
 				path += traits[m_type].separators[0];
+			}
 		}
 
 		if (traits[m_type].separatorEscape) {
-			wxString tmp = segment;
+			std::wstring tmp = segment;
 			EscapeSeparators(m_type, tmp);
 			path += tmp;
 		}
-		else
+		else {
 			path += segment;
+		}
 	}
 
-	if (traits[m_type].prefixmode && m_data->m_prefix)
+	if (traits[m_type].prefixmode && m_data->m_prefix) {
 		path += *m_data->m_prefix;
+	}
 
-	if (traits[m_type].right_enclosure != 0)
+	if (traits[m_type].right_enclosure != 0) {
 		path += traits[m_type].right_enclosure;
+	}
 
 	// DOS is strange.
 	// C: is current working dir on drive C, C:\ the drive root.
-	if ((m_type == DOS || m_type == DOS_FWD_SLASHES) && m_data->m_segments.size() == 1)
+	if ((m_type == DOS || m_type == DOS_FWD_SLASHES) && m_data->m_segments.size() == 1) {
 		path += traits[m_type].separators[0];
+	}
 
 	return path;
 }
 
 bool CServerPath::HasParent() const
 {
-	if (empty())
+	if (empty()) {
 		return false;
+	}
 
-	if (!traits[m_type].has_root)
+	if (!traits[m_type].has_root) {
 		return m_data->m_segments.size() > 1;
+	}
 
 	return !m_data->m_segments.empty();
 }
 
 CServerPath CServerPath::GetParent() const
 {
-	if (empty() || !HasParent())
+	if (empty() || !HasParent()) {
 		return CServerPath();
+	}
 
 	CServerPath parent(*this);
 	CServerPathData& parent_data = parent.m_data.get();
@@ -190,37 +214,52 @@ CServerPath CServerPath::GetParent() const
 	parent_data.m_segments.pop_back();
 
 	if (m_type == MVS) {
-		parent_data.m_prefix = fz::sparse_optional<std::wstring>(_T("."));
+		parent_data.m_prefix = fz::sparse_optional<std::wstring>(L".");
 	}
 
 	return parent;
 }
 
-wxString CServerPath::GetLastSegment() const
+std::wstring CServerPath::GetFirstSegment() const
 {
-	if (empty() || !HasParent())
-		return wxString();
+	if (empty() || !HasParent()) {
+		return std::wstring();
+	}
 
-	if (!m_data->m_segments.empty())
+	if (!m_data->m_segments.empty()) {
+		return m_data->m_segments.front();
+	}
+	else {
+		return std::wstring();
+	}
+}
+
+std::wstring CServerPath::GetLastSegment() const
+{
+	if (empty() || !HasParent()) {
+		return std::wstring();
+	}
+
+	if (!m_data->m_segments.empty()) {
 		return m_data->m_segments.back();
-	else
-		return wxString();
+	}
+	else {
+		return std::wstring();
+	}
 }
 
 // libc sprintf can be so slow at times...
-wxChar* fast_sprint_number(wxChar* s, size_t n)
+wchar_t* fast_sprint_number(wchar_t* s, size_t n)
 {
-	wxChar tmp[20]; // Long enough for 2^64-1
+	wchar_t tmp[20]; // Long enough for 2^64-1
 
-	wxChar* c = tmp;
-	do
-	{
+	wchar_t* c = tmp;
+	do {
 		*(c++) = n % 10 + '0';
 		n /= 10;
 	} while( n > 0 );
 
-	do
-	{
+	do {
 		*(s++) = *(--c);
 	} while (c != tmp);
 
@@ -229,10 +268,11 @@ wxChar* fast_sprint_number(wxChar* s, size_t n)
 
 #define tstrcpy wcscpy
 
-wxString CServerPath::GetSafePath() const
+std::wstring CServerPath::GetSafePath() const
 {
-	if (empty())
-		return wxString();
+	if (empty()) {
+		return std::wstring();
+	}
 
 	#define INTLENGTH 20 // 2^64 - 1
 
@@ -240,40 +280,39 @@ wxString CServerPath::GetSafePath() const
 		+ INTLENGTH; // Max length of prefix
 
 	len += m_data->m_prefix ? m_data->m_prefix->size() : 0;
-	for( auto const& segment : m_data->m_segments ) {
+	for (auto const& segment : m_data->m_segments) {
 		len += segment.size() + 2 + INTLENGTH;
 	}
 
-	wxString safepath;
-	{
-		wxStringBuffer start(safepath, len);
-		wxChar* t = start;
+	std::wstring safepath;
+	safepath.resize(len);
+	wchar_t * const start = &safepath[0];
+	wchar_t * t = &safepath[0];
 
-		t = fast_sprint_number(t, m_type);
+	t = fast_sprint_number(t, m_type);
+	*(t++) = ' ';
+	t = fast_sprint_number(t, m_data->m_prefix ? m_data->m_prefix->size() : 0);
+
+	if (m_data->m_prefix) {
 		*(t++) = ' ';
-		t = fast_sprint_number(t, m_data->m_prefix ? m_data->m_prefix->size() : 0);
-
-		if (m_data->m_prefix) {
-			*(t++) = ' ';
-			tstrcpy(t, m_data->m_prefix->c_str());
-			t += m_data->m_prefix->size();
-		}
-
-		for( auto const& segment : m_data->m_segments ) {
-			*(t++) = ' ';
-			t = fast_sprint_number(t, segment.size());
-			*(t++) = ' ';
-			tstrcpy(t, segment.c_str());
-			t += segment.size();
-		}
-		*t = 0;
+		tstrcpy(t, m_data->m_prefix->c_str());
+		t += m_data->m_prefix->size();
 	}
-	safepath.Shrink();
+
+	for (auto const& segment : m_data->m_segments) {
+		*(t++) = ' ';
+		t = fast_sprint_number(t, segment.size());
+		*(t++) = ' ';
+		tstrcpy(t, segment.c_str());
+		t += segment.size();
+	}
+	safepath.resize(t - start);
+	safepath.shrink_to_fit();
 
 	return safepath;
 }
 
-bool CServerPath::SetSafePath(const wxString& path)
+bool CServerPath::SetSafePath(std::wstring const& path)
 {
 	bool const ret = DoSetSafePath(path);
 	if (!ret) {
@@ -283,7 +322,7 @@ bool CServerPath::SetSafePath(const wxString& path)
 	return ret;
 }
 
-bool CServerPath::DoSetSafePath(const wxString& path)
+bool CServerPath::DoSetSafePath(std::wstring const& path)
 {
 	CServerPathData& data = m_data.get();
 	data.m_prefix.clear();
@@ -293,10 +332,10 @@ bool CServerPath::DoSetSafePath(const wxString& path)
 	// Before the optimization this function was responsible for
 	// most CPU cycles used during loading of transfer queues
 	// from file
-	wxChar const* const begin = path.c_str();
-	wxChar const* const end = begin + path.size();
-	
-	wxChar const* p = begin;
+	wchar_t const* const begin = path.c_str();
+	wchar_t const* const end = begin + path.size();
+
+	wchar_t const* p = begin;
 
 	int type = 0;
 	do {
@@ -373,7 +412,7 @@ bool CServerPath::DoSetSafePath(const wxString& path)
 		if (segment_len > end - p) {
 			return false;
 		}
-		data.m_segments.emplace_back(wxString(p, p + segment_len));
+		data.m_segments.emplace_back(p, p + segment_len);
 
 		p += segment_len + 1;
 	}
@@ -399,14 +438,17 @@ ServerType CServerPath::GetType() const
 
 bool CServerPath::IsSubdirOf(const CServerPath &path, bool cmpNoCase) const
 {
-	if (empty() || path.empty())
+	if (empty() || path.empty()) {
 		return false;
+	}
 
-	if (m_type != path.m_type)
+	if (m_type != path.m_type) {
 		return false;
+	}
 
-	if (!HasParent())
+	if (!HasParent()) {
 		return false;
+	}
 
 	if (traits[m_type].prefixmode != 1) {
 		if (cmpNoCase ) {
@@ -456,13 +498,13 @@ bool CServerPath::IsParentOf(const CServerPath &path, bool cmpNoCase) const
 	return path.IsSubdirOf(*this, cmpNoCase);
 }
 
-bool CServerPath::ChangePath(wxString const& subdir)
+bool CServerPath::ChangePath(std::wstring const& subdir)
 {
-	wxString subdir2 = subdir;
+	std::wstring subdir2 = subdir;
 	return ChangePath(subdir2, false);
 }
 
-bool CServerPath::ChangePath(wxString &subdir, bool isFile)
+bool CServerPath::ChangePath(std::wstring &subdir, bool isFile)
 {
 	bool ret = DoChangePath(subdir, isFile);
 	if (!ret) {
@@ -472,16 +514,18 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 	return ret;
 }
 
-bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
+bool CServerPath::DoChangePath(std::wstring &subdir, bool isFile)
 {
-	wxString dir = subdir;
-	wxString file;
+	std::wstring dir = subdir;
+	std::wstring file;
 
 	if (dir.empty()) {
-		if (empty() || isFile)
+		if (empty() || isFile) {
 			return false;
-		else
+		}
+		else {
 			return true;
+		}
 	}
 
 	bool const was_empty = empty();
@@ -491,236 +535,275 @@ bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 	{
 	case VMS:
 		{
-			int pos1 = dir.Find(traits[m_type].left_enclosure);
-			if (pos1 == -1)
-			{
-				int pos2 = dir.Find(traits[m_type].right_enclosure, true);
-				if (pos2 != -1)
+			size_t pos1 = dir.find(traits[m_type].left_enclosure);
+			if (pos1 == std::wstring::npos) {
+				size_t pos2 = dir.rfind(traits[m_type].right_enclosure);
+				if (pos2 != std::wstring::npos) {
 					return false;
+				}
 
-				if (isFile)
-				{
-					if (was_empty)
+				if (isFile) {
+					if (was_empty) {
 						return false;
+					}
 
 					file = dir;
 					break;
 				}
 			}
-			else
-			{
-				int pos2 = dir.Find(traits[m_type].right_enclosure, true);
-				if (pos2 == -1 || pos2 <= pos1 + 1)
+			else {
+				size_t pos2 = dir.rfind(traits[m_type].right_enclosure);
+				if (pos2 == std::wstring::npos || pos2 <= pos1 + 1) {
 					return false;
+				}
 
-				bool enclosure_is_last = static_cast<size_t>(pos2) == (dir.Length() - 1);
-				if (isFile == enclosure_is_last)
+				bool enclosure_is_last = pos2 == (dir.size() - 1);
+				if (isFile == enclosure_is_last) {
 					return false;
+				}
 
-				if (isFile)
-					file = dir.Mid(pos2 + 1);
-				dir = dir.Left(pos2);
+				if (isFile) {
+					file = dir.substr(pos2 + 1);
+				}
+				dir = dir.substr(0, pos2);
 
-				if (pos1)
-					data.m_prefix = fz::sparse_optional<std::wstring>(dir.Left(pos1).ToStdWstring());
-				dir = dir.Mid(pos1 + 1);
+				if (pos1) {
+					data.m_prefix = fz::sparse_optional<std::wstring>(dir.substr(0, pos1));
+				}
+				dir = dir.substr(pos1 + 1);
 
 				data.m_segments.clear();
 			}
 
-			if (!Segmentize(dir.ToStdWstring(), data.m_segments))
+			if (!Segmentize(dir, data.m_segments)) {
 				return false;
-			if (data.m_segments.empty() && was_empty)
+			}
+			if (data.m_segments.empty() && was_empty) {
 				return false;
+			}
 		}
 		break;
 	case DOS:
 	case DOS_FWD_SLASHES:
 		{
-			bool is_relative = false;
-			int sep = dir.find_first_of(traits[m_type].separators);
-			if (sep == -1)
-				sep = dir.Len();
-			int colon = dir.Find(':');
-			if (colon > 0 && colon == sep - 1)
-				is_relative = true;
+			bool is_absolute = false;
+			size_t sep = dir.find_first_of(traits[m_type].separators);
+			if (sep == std::wstring::npos) {
+				sep = dir.size();
+			}
+			size_t colon = dir.find(':');
+			if (colon != std::wstring::npos && colon > 0 && colon == sep - 1) {
+				is_absolute = true;
+			}
 
-			if (is_relative)
+			if (is_absolute) {
 				data.m_segments.clear();
-			else if (wxString(traits[m_type].separators).Contains(dir[0]))
-			{
-				if (data.m_segments.empty())
+			}
+			else if (IsSeparator(dir[0])) {
+				// Drive-relative path
+				if (data.m_segments.empty()) {
 					return false;
+				}
 				std::wstring first = data.m_segments.front();
 				data.m_segments.clear();
 				data.m_segments.push_back(first);
-				dir = dir.Mid(1);
+				dir = dir.substr(1);
+			}
+			// else: Any other relative path
+
+			if (isFile && !ExtractFile(dir, file)) {
+				return false;
 			}
 
-			if (isFile && !ExtractFile(dir, file))
+			if (!Segmentize(dir, data.m_segments)) {
 				return false;
-
-			if (!Segmentize(dir.ToStdWstring(), data.m_segments))
-				return false;
-			if (data.m_segments.empty() && was_empty)
+			}
+			if (data.m_segments.empty() && was_empty) {
 				return false;
+			}
 		}
 		break;
 	case MVS:
 		{
 			// Remove the double quoation some servers send in PWD reply
-			int i = 0;
-			wxChar c = dir.c_str()[i];
-			while (c == FTP_MVS_DOUBLE_QUOTE)
-				c = dir.c_str()[++i];
-			dir.Remove(0, i);
+			size_t i = 0;
+			wchar_t c = dir[i];
+			while (c == FTP_MVS_DOUBLE_QUOTE) {
+				c = dir[++i];
+			}
+			dir.erase(0, i);
 
 			while (!dir.empty()) {
-				c = dir.Last();
-				if (c != FTP_MVS_DOUBLE_QUOTE)
+				c = dir.back();
+				if (c != FTP_MVS_DOUBLE_QUOTE) {
 					break;
-				else
-					dir.RemoveLast();
+				}
+				else {
+					dir.pop_back();
+				}
 			}
 		}
-		if (dir.empty())
+		if (dir.empty()) {
 			return false;
+		}
 
-		if (dir.c_str()[0] == traits[m_type].left_enclosure) {
-			if (dir.Last() != traits[m_type].right_enclosure)
+		if (dir[0] == traits[m_type].left_enclosure) {
+			if (dir.back() != traits[m_type].right_enclosure) {
 				return false;
+			}
 
-			dir.RemoveLast();
-			dir = dir.Mid(1);
+			dir = dir.substr(1, dir.size() - 2);
 
 			data.m_segments.clear();
 		}
-		else if (dir.Last() == traits[m_type].right_enclosure)
+		else if (dir.back() == traits[m_type].right_enclosure) {
 			return false;
-		else if (was_empty)
+		}
+		else if (was_empty) {
 			return false;
+		}
 
-		if (!dir.empty() && dir.Last() == ')') {
+		if (!dir.empty() && dir.back() == ')') {
 			// Partitioned dataset member
-			if (!isFile)
+			if (!isFile) {
 				return false;
+			}
 
-			int pos = dir.Find('(');
-			if (pos == -1)
+			size_t pos = dir.find('(');
+			if (pos == std::wstring::npos) {
 				return false;
-			dir.RemoveLast();
-			file = dir.Mid(pos + 1);
-			dir = dir.Left(pos);
+			}
+			dir.pop_back();
+			file = dir.substr(pos + 1);
+			dir = dir.substr(0, pos);
 
-			if (!was_empty && !data.m_prefix && !dir.empty())
+			if (!was_empty && !data.m_prefix && !dir.empty()) {
 				return false;
+			}
 
 			data.m_prefix.clear();
 		}
 		else {
-			if (!was_empty && !data.m_prefix)
-			{
-				if (dir.Find('.') != -1 || !isFile)
+			if (!was_empty && !data.m_prefix) {
+				if (dir.find('.') != std::wstring::npos || !isFile) {
 					return false;
+				}
 			}
 
 			if (isFile) {
-				if (!ExtractFile(dir, file))
+				if (!ExtractFile(dir, file)) {
 					return false;
-				data.m_prefix = fz::sparse_optional<std::wstring>(_T("."));
+				}
+				data.m_prefix = fz::sparse_optional<std::wstring>(L".");
 			}
-			else if (!dir.empty() && dir.Last() == '.')
-				data.m_prefix = fz::sparse_optional<std::wstring>(_T("."));
-			else
+			else if (!dir.empty() && dir.back() == '.') {
+				data.m_prefix = fz::sparse_optional<std::wstring>(L".");
+			}
+			else {
 				data.m_prefix.clear();
+			}
 		}
 
-		if (!Segmentize(dir.ToStdWstring(), data.m_segments))
+		if (!Segmentize(dir, data.m_segments)) {
 			return false;
+		}
 		break;
 	case HPNONSTOP:
-		if (dir[0] == '\\')
+		if (dir[0] == '\\') {
 			data.m_segments.clear();
+		}
 
-		if (isFile && !ExtractFile(dir, file))
+		if (isFile && !ExtractFile(dir, file)) {
 			return false;
+		}
 
-		if (!Segmentize(dir.ToStdWstring(), data.m_segments))
+		if (!Segmentize(dir, data.m_segments)) {
 			return false;
-		if (data.m_segments.empty() && was_empty)
+		}
+		if (data.m_segments.empty() && was_empty) {
 			return false;
+		}
 
 		break;
 	case VXWORKS:
 		{
-			if (dir[0] != ':')
-			{
-				if (was_empty)
+			if (dir[0] != ':') {
+				if (was_empty) {
 					return false;
+				}
 			}
-			else
-			{
-				int colon2;
-				if ((colon2 = dir.Mid(1).Find(':')) < 1)
+			else {
+				size_t colon2 =  dir.find(':', 1);
+				if (colon2 == std::wstring::npos || colon2 == 1) {
 					return false;
-				data.m_prefix = fz::sparse_optional<std::wstring>(dir.Left(colon2 + 2).ToStdWstring());
-				dir = dir.Mid(colon2 + 2);
+				}
+				data.m_prefix = fz::sparse_optional<std::wstring>(dir.substr(0, colon2 + 1));
+				dir = dir.substr(colon2 + 1);
 
 				data.m_segments.clear();
 			}
 
-			if (isFile && !ExtractFile(dir, file))
+			if (isFile && !ExtractFile(dir, file)) {
 				return false;
+			}
 
-			if (!Segmentize(dir.ToStdWstring(), data.m_segments))
+			if (!Segmentize(dir, data.m_segments)) {
 				return false;
+			}
 		}
 		break;
 	case CYGWIN:
 		{
-			if (wxString(traits[m_type].separators).Contains(dir[0])) {
+			if (IsSeparator(dir[0])) {
 				data.m_segments.clear();
 				data.m_prefix.clear();
 			}
-			else if (was_empty)
+			else if (was_empty) {
 				return false;
-			if (dir.Left(2) == _T("//")) {
+			}
+			if (dir[0] == '/' && dir[1] == '/') {
 				data.m_prefix = fz::sparse_optional<std::wstring>(std::wstring(1, traits[m_type].separators[0]));
-				dir = dir.Mid(1);
+				dir = dir.substr(1);
 			}
 
-			if (isFile && !ExtractFile(dir, file))
+			if (isFile && !ExtractFile(dir, file)) {
 				return false;
+			}
 
-			if (!Segmentize(dir.ToStdWstring(), data.m_segments))
+			if (!Segmentize(dir, data.m_segments)) {
 				return false;
+			}
 		}
 		break;
 	default:
 		{
-			if (wxString(traits[m_type].separators).Contains(dir[0]))
+			if (IsSeparator(dir[0])) {
 				data.m_segments.clear();
-			else if (was_empty)
+			}
+			else if (was_empty) {
 				return false;
+			}
 
-			if (isFile && !ExtractFile(dir, file))
+			if (isFile && !ExtractFile(dir, file)) {
 				return false;
+			}
 
-			if (!Segmentize(dir.ToStdWstring(), data.m_segments))
+			if (!Segmentize(dir, data.m_segments)) {
 				return false;
+			}
 		}
 		break;
 	}
 
-	if (!traits[m_type].has_root && data.m_segments.empty())
+	if (!traits[m_type].has_root && data.m_segments.empty()) {
 		return false;
+	}
 
-	if (isFile)
-	{
-		if (traits[m_type].has_dots)
-		{
-			if (file == _T("..") || file == _T("."))
+	if (isFile) {
+		if (traits[m_type].has_dots) {
+			if (file == L".." || file == L".") {
 				return false;
+			}
 		}
 		subdir = file;
 	}
@@ -730,12 +813,15 @@ bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 
 bool CServerPath::operator==(const CServerPath &op) const
 {
-	if (empty() != op.empty())
+	if (empty() != op.empty()) {
 		return false;
-	else if (m_type != op.m_type)
+	}
+	else if (m_type != op.m_type) {
 		return false;
-	else if (m_data != op.m_data)
+	}
+	else if (m_data != op.m_data) {
 		return false;
+	}
 
 	return true;
 }
@@ -748,25 +834,27 @@ bool CServerPath::operator!=(const CServerPath &op) const
 bool CServerPath::operator<(const CServerPath &op) const
 {
 	if (empty()) {
-		if (!op.empty())
-			return false;
+		return !op.empty();
+	}
+	else if (op.empty()) {
+		return false;
 	}
-	else if (op.empty())
-		return true;
 
-	if( m_data->m_prefix || op.m_data->m_prefix ) {
-		if( m_data->m_prefix < op.m_data->m_prefix ) {
+	if (m_data->m_prefix || op.m_data->m_prefix) {
+		if (m_data->m_prefix < op.m_data->m_prefix) {
 			return true;
 		}
-		else if( op.m_data->m_prefix < m_data->m_prefix ) {
+		else if (op.m_data->m_prefix < m_data->m_prefix) {
 			return false;
 		}
 	}
 
-	if (m_type > op.m_type)
+	if (m_type > op.m_type) {
 		return false;
-	else if (m_type < op.m_type)
+	}
+	else if (m_type < op.m_type) {
 		return true;
+	}
 
 	tConstSegmentIter iter1, iter2;
 	for (iter1 = m_data->m_segments.begin(), iter2 = op.m_data->m_segments.begin(); iter1 != m_data->m_segments.end(); ++iter1, ++iter2) {
@@ -786,43 +874,47 @@ bool CServerPath::operator<(const CServerPath &op) const
 	return iter2 != op.m_data->m_segments.end();
 }
 
-wxString CServerPath::FormatFilename(const wxString &filename, bool omitPath /*=false*/) const
+std::wstring CServerPath::FormatFilename(std::wstring const& filename, bool omitPath) const
 {
-	if (empty())
+	if (empty() || filename.empty()) {
 		return filename;
+	}
 
-	if (filename.empty())
-		return wxString();
-
-	if (omitPath && (!traits[m_type].prefixmode || (m_data->m_prefix && *m_data->m_prefix == _T("."))))
+	if (omitPath && (!traits[m_type].prefixmode || (m_data->m_prefix && *m_data->m_prefix == L"."))) {
 		return filename;
+	}
 
-	wxString result = GetPath();
-	if (traits[m_type].left_enclosure && traits[m_type].filename_inside_enclosure)
-		result.RemoveLast();
+	std::wstring result = GetPath();
+	if (traits[m_type].left_enclosure && traits[m_type].filename_inside_enclosure) {
+		result.pop_back();
+	}
 
-	switch (m_type)
-	{
+	switch (m_type) {
 		case VXWORKS:
-			if (!result.empty() && !wxString(traits[m_type].separators).Contains(result.Last()) && !m_data->m_segments.empty())
+			if (!result.empty() && !IsSeparator(result.back()) && !m_data->m_segments.empty()) {
 				result += traits[m_type].separators[0];
+			}
 			break;
 		case VMS:
 		case MVS:
 			break;
 		default:
-			if (!result.empty() && !wxString(traits[m_type].separators).Contains(result.Last()))
+			if (!result.empty() && !IsSeparator(result.back())) {
 				result += traits[m_type].separators[0];
+			}
 			break;
 	}
 
-	if (traits[m_type].prefixmode == 1 && !m_data->m_prefix)
-		result += _T("(") + filename + _T(")");
-	else
+	if (traits[m_type].prefixmode == 1 && !m_data->m_prefix) {
+		result += L"(" + filename + L")";
+	}
+	else {
 		result += filename;
+	}
 
-	if (traits[m_type].left_enclosure && traits[m_type].filename_inside_enclosure)
+	if (traits[m_type].left_enclosure && traits[m_type].filename_inside_enclosure) {
 		result += traits[m_type].right_enclosure;
+	}
 
 	return result;
 }
@@ -853,24 +945,27 @@ int CServerPath::CmpNoCase(const CServerPath &op) const
 	return 0;
 }
 
-bool CServerPath::AddSegment(const wxString& segment)
+bool CServerPath::AddSegment(std::wstring const& segment)
 {
-	if (empty())
+	if (empty()) {
 		return false;
+	}
 
 	// TODO: Check for invalid characters
-	m_data.get().m_segments.push_back(segment.ToStdWstring());
+	m_data.get().m_segments.push_back(segment);
 
 	return true;
 }
 
 CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 {
-	if (*this == path)
+	if (*this == path) {
 		return *this;
+	}
 
-	if (empty() || path.empty())
+	if (empty() || path.empty()) {
 		return CServerPath();
+	}
 
 	if (m_type != path.m_type ||
 		(!traits[m_type].prefixmode && m_data->m_prefix != path.m_data->m_prefix))
@@ -878,19 +973,21 @@ CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 		return CServerPath();
 	}
 
-	if (!HasParent())
-	{
-		if (path.IsSubdirOf(*this, false))
+	if (!HasParent()) {
+		if (path.IsSubdirOf(*this, false)) {
 			return *this;
-		else
+		}
+		else {
 			return CServerPath();
+		}
 	}
-	else if (!path.HasParent())
-	{
-		if (IsSubdirOf(path, false))
+	else if (!path.HasParent()) {
+		if (IsSubdirOf(path, false)) {
 			return path;
-		else
+		}
+		else {
 			return CServerPath();
+		}
 	}
 
 	CServerPath parent;
@@ -901,10 +998,12 @@ CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 	tConstSegmentIter last = m_data->m_segments.end();
 	tConstSegmentIter last2 = path.m_data->m_segments.end();
 	if (traits[m_type].prefixmode == 1) {
-		if (!m_data->m_prefix)
+		if (!m_data->m_prefix) {
 			--last;
-		if (!path.m_data->m_prefix)
+		}
+		if (!path.m_data->m_prefix) {
 			--last2;
+		}
 		parentData.m_prefix = GetParent().m_data->m_prefix;
 	}
 	else
@@ -914,10 +1013,12 @@ CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 	tConstSegmentIter iter2 = path.m_data->m_segments.begin();
 	while (iter != last && iter2 != last2) {
 		if (*iter != *iter2) {
-			if (!traits[m_type].has_root && parentData.m_segments.empty())
+			if (!traits[m_type].has_root && parentData.m_segments.empty()) {
 				return CServerPath();
-			else
+			}
+			else {
 				return parent;
+			}
 		}
 
 		parentData.m_segments.push_back(*iter);
@@ -929,12 +1030,13 @@ CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 	return parent;
 }
 
-wxString CServerPath::FormatSubdir(const wxString &subdir) const
+std::wstring CServerPath::FormatSubdir(std::wstring const& subdir) const
 {
-	if (!traits[m_type].separatorEscape)
+	if (!traits[m_type].separatorEscape) {
 		return subdir;
+	}
 
-	wxString res = subdir;
+	std::wstring res = subdir;
 	EscapeSeparators(m_type, res);
 
 	return res;
@@ -943,15 +1045,14 @@ wxString CServerPath::FormatSubdir(const wxString &subdir) const
 bool CServerPath::SegmentizeAddSegment(std::wstring & segment, tSegmentList& segments, bool& append)
 {
 	if (traits[m_type].has_dots) {
-		if (segment == _T("."))
+		if (segment == L".") {
 			return true;
-		else if (segment == _T("..")) {
-			if (segments.empty())
-				return false;
-			else {
+		}
+		else if (segment == L"..") {
+			if (!segments.empty()) {
 				segments.pop_back();
-				return true;
 			}
+			return true;
 		}
 	}
 
@@ -961,10 +1062,12 @@ bool CServerPath::SegmentizeAddSegment(std::wstring & segment, tSegmentList& seg
 		segment[segment.size() - 1] = traits[m_type].separators[0];
 	}
 
-	if (append)
+	if (append) {
 		segments.back() += segment;
-	else
+	}
+	else {
 		segments.push_back(std::move(segment));
+	}
 
 	append = append_next;
 
@@ -1008,33 +1111,46 @@ bool CServerPath::Segmentize(std::wstring const& str, tSegmentList& segments)
 	return true;
 }
 
-bool CServerPath::ExtractFile(wxString& dir, wxString& file)
+bool CServerPath::ExtractFile(std::wstring& dir, std::wstring& file)
 {
-	int pos = dir.find_last_of(traits[m_type].separators);
-	if (pos == (int)dir.Length() - 1)
+	size_t pos = dir.find_last_of(traits[m_type].separators);
+	if (pos != std::wstring::npos && pos == dir.size() - 1) {
 		return false;
+	}
 
-	if (pos == -1) {
+	if (pos == std::wstring::npos) {
 		file = dir;
 		dir.clear();
 		return true;
 	}
 
-	file = dir.Mid(pos + 1);
-	dir = dir.Left(pos + 1);
+	file = dir.substr(pos + 1);
+	dir = dir.substr(0, pos + 1);
 
 	return true;
 }
 
-void CServerPath::EscapeSeparators(ServerType type, wxString& subdir)
+void CServerPath::EscapeSeparators(ServerType type, std::wstring& subdir)
 {
 	if (traits[type].separatorEscape) {
-		for (const wxChar* p = traits[type].separators; *p; ++p)
-			subdir.Replace((wxString)*p, (wxString)traits[type].separatorEscape + traits[type].separators[0]);
+		for (wchar_t const* p = traits[type].separators; *p; ++p) {
+			fz::replace_substrings(subdir, std::wstring(1, *p), std::wstring(1, traits[type].separatorEscape) + traits[type].separators[0]);
+		}
 	}
 }
 
 size_t CServerPath::SegmentCount() const
 {
 	return empty() ? 0 : m_data->m_segments.size();
-}
\ No newline at end of file
+}
+
+bool CServerPath::IsSeparator(wchar_t c) const
+{
+	for (wchar_t const* ref = traits[m_type].separators; *ref; ++ref) {
+		if (*ref == c) {
+			return true;
+		}
+	}
+
+	return false;
+}
