@@ -110,6 +110,10 @@ void operator delete[](void* ptr, std::size_t)
 	}
 }
 
+#ifdef __GNUC__
+extern "C" void __cxa_pure_virtual() {}
+#endif
+
 //---------------------------------------------------------------------------
 class CShellExtClassFactory : public IClassFactory
 {
@@ -127,7 +131,7 @@ public:
 	STDMETHODIMP LockServer(BOOL);
 
 protected:
-	unsigned long FReferenceCounter;
+	unsigned long FReferenceCounter{};
 };
 
 //---------------------------------------------------------------------------
@@ -150,10 +154,10 @@ public:
 									 LPCTSTR SrcFile, DWORD SrcAttribs, LPCTSTR DestFile, DWORD DestAttribs);
 
 protected:
-	unsigned long FReferenceCounter;
+	unsigned long FReferenceCounter{};
 	LPDATAOBJECT FDataObj;
 	HANDLE FMutex;
-	unsigned long FLastTicks;
+	unsigned long FLastTicks{};
 };
 
 //---------------------------------------------------------------------------
@@ -600,8 +604,6 @@ CShellExtClassFactory::CShellExtClassFactory()
 {
 	DEBUG_MSG("CShellExtClassFactory");
 
-	FReferenceCounter = 0;
-
 	GRefThisDll++;
 }
 
@@ -697,11 +699,9 @@ CShellExt::CShellExt()
 {
 	DEBUG_MSG("CShellExt enter");
 
-	FReferenceCounter = 0L;
 	FDataObj = NULL;
 
 	FMutex = CreateMutex(NULL, false, DRAG_EXT_MUTEX);
-	FLastTicks = 0;
 
 	GRefThisDll++;
 
@@ -862,6 +862,8 @@ STDMETHODIMP_(UINT) CShellExt::CopyCallback(HWND Hwnd, UINT wFunc, UINT Flags,
 				DEBUG_MSG("CShellExt::CopyCallback supported structure version");
 				if (data[1] == 1) {
 					DEBUG_MSG("CShellExt::CopyCallback dragging");
+					data[DRAG_EXT_MAPPING_LENGTH - 1] = 0;
+					data[DRAG_EXT_MAPPING_LENGTH - 2] = 0;
 
 					wchar_t* file = reinterpret_cast<wchar_t *>(data + 2);
 					DEBUG_MSG("Dragged file:");
@@ -869,12 +871,12 @@ STDMETHODIMP_(UINT) CShellExt::CopyCallback(HWND Hwnd, UINT wFunc, UINT Flags,
 
 					if (_wcsicmp(file, SrcFile) == 0) {
 						data[1] = 2;
-						if (_tcslen(DestFile) > MAX_PATH) {
-							DEBUG_MSG("CShellExt::CopyCallback length of DestFile exceeding MAX_PATH");
+						if (_tcslen(DestFile) > DRAG_EXT_MAX_PATH) {
+							DEBUG_MSG("CShellExt::CopyCallback length of DestFile exceeding DRAG_EXT_MAX_PATH");
 						}
 						else {
-							wcsncpy(file, DestFile, MAX_PATH);
-							file[MAX_PATH] = 0;
+							wcsncpy(file, DestFile, DRAG_EXT_MAX_PATH);
+							file[DRAG_EXT_MAX_PATH] = 0;
 							DEBUG_MSG("CShellExt::CopyCallback destination written into buffer");
 						}
 						Result = IDNO;
