@@ -1,5 +1,5 @@
-#ifndef __NOTIFICATION_H__
-#define __NOTIFICATION_H__
+#ifndef FILEZILLA_ENGINE_NOTIFICATION_HEADER
+#define FILEZILLA_ENGINE_NOTIFICATION_HEADER
 
 // Notification overview
 // ---------------------
@@ -60,7 +60,7 @@ enum RequestId
 class CNotification
 {
 public:
-	virtual ~CNotification() {}; // TODO: One GCC >= 4.8 is in Debian Stable (Jessie by then), make default and add testcase to configure.
+	virtual ~CNotification() = default;
 	virtual NotificationId GetID() const = 0;
 
 protected:
@@ -196,8 +196,8 @@ public:
 	// Set to true if you have set a password
 	bool passwordSet{};
 
-	// Set password by calling server.SetUser
 	CServer server;
+	Credentials credentials;
 
 	std::wstring const& GetChallenge() const { return m_challenge; }
 
@@ -248,7 +248,7 @@ public:
 
 	// True on download notifications iff currentOffset != startOffset.
 	// True on FTP upload notifications iff currentOffset != startOffset
-	// AND after the first accepted data after the first wxSOCKET_WOULDBLOCK.
+	// AND after the first accepted data after the first EWOULDBLOCK.
 	// SFTP uploads: Set to true if currentOffset >= startOffset + 65536.
 	bool madeProgress{};
 
