@@ -1,8 +1,7 @@
-#ifndef __OPTION_CHANGE_EVENT_HANDLER_H__
-#define __OPTION_CHANGE_EVENT_HANDLER_H__
+#ifndef FILEZILLA_ENGINE_OPTION_CHANGE_EVENT_HANDLER_HEADER
+#define FILEZILLA_ENGINE_OPTION_CHANGE_EVENT_HANDLER_HEADER
 
 #include <bitset>
-#include <set>
 #include <vector>
 
 class COptions;
@@ -18,7 +17,7 @@ class COptionChangeEventHandler
 	friend class COptions;
 
 public:
-	COptionChangeEventHandler();
+	COptionChangeEventHandler() = default;
 	virtual ~COptionChangeEventHandler();
 
 	void RegisterOption(int option);
@@ -40,4 +39,4 @@ private:
 	static std::vector<COptionChangeEventHandler*> m_handlers;
 };
 
-#endif //__OPTION_CHANGE_EVENT_HANDLER_H__
+#endif
