@@ -4,7 +4,7 @@
 enum ServerProtocol
 {
 	// Never change any existing values or user's saved sites will become
-	// corrupted.
+	// corrupted
 	UNKNOWN = -1,
 	FTP, // FTP, attempts AUTH TLS
 	SFTP,
@@ -14,7 +14,11 @@ enum ServerProtocol
 	HTTPS,
 	INSECURE_FTP, // Insecure, as the name suggests
 
-	MAX_VALUE = INSECURE_FTP
+	S3, // Amazon S3 or compatible
+
+	STORJ,
+
+	MAX_VALUE = STORJ
 };
 
 enum ServerType
@@ -34,18 +38,6 @@ enum ServerType
 	SERVERTYPE_MAX
 };
 
-enum LogonType
-{
-	ANONYMOUS,
-	NORMAL,
-	ASK, // ASK should not be sent to the engine, it's intended to be used by the interface
-	INTERACTIVE,
-	ACCOUNT,
-	KEY,
-
-	LOGONTYPE_MAX
-};
-
 enum PasvMode
 {
 	MODE_DEFAULT,
@@ -69,15 +61,17 @@ enum CharsetEncoding
 	ENCODING_CUSTOM
 };
 
+class Credentials;
 class CServerPath;
 class CServer final
 {
 public:
 
 	// No error checking is done in the constructors
-	CServer();
+	CServer() = default;
 	CServer(ServerProtocol protocol, ServerType type, std::wstring const& host, unsigned int);
-	CServer(ServerProtocol protocol, ServerType type, std::wstring const& host, unsigned int, std::wstring const& user, std::wstring const& pass = std::wstring(), std::wstring const& account = std::wstring());
+
+	void clear();
 
 	void SetType(ServerType type);
 
@@ -85,40 +79,28 @@ public:
 	ServerType GetType() const;
 	std::wstring GetHost() const;
 	unsigned int GetPort() const;
-	LogonType GetLogonType() const;
 	std::wstring GetUser() const;
-	std::wstring GetPass() const;
-	std::wstring GetAccount() const;
-	std::wstring GetKeyFile() const;
 	int GetTimezoneOffset() const;
 	PasvMode GetPasvMode() const;
 	int MaximumMultipleConnections() const;
 	bool GetBypassProxy() const;
 
-	// Return true if URL could be parsed correctly, false otherwise.
-	// If parsing fails, pError is filled with the reason and the CServer instance may be left an undefined state.
-	bool ParseUrl(std::wstring host, unsigned int port, std::wstring user, std::wstring pass, std::wstring &error, CServerPath &path);
-	bool ParseUrl(std::wstring const& host, std::wstring const& port, std::wstring const& user, std::wstring const& pass, std::wstring &error, CServerPath &path);
-
 	void SetProtocol(ServerProtocol serverProtocol);
 	bool SetHost(std::wstring const& host, unsigned int port);
 
-	void SetLogonType(LogonType logonType);
-	bool SetUser(std::wstring const& user, std::wstring const& pass = std::wstring());
-	bool SetAccount(std::wstring const& account);
-	bool SetKeyFile(std::wstring const& keyFile);
+	void SetUser(std::wstring const& user);
 
 	CServer& operator=(const CServer &op);
 	bool operator==(const CServer &op) const;
 	bool operator<(const CServer &op) const;
 	bool operator!=(const CServer &op) const;
-	bool EqualsNoPass(const CServer &op) const;
 
 	bool SetTimezoneOffset(int minutes);
 	void SetPasvMode(PasvMode pasvMode);
 	void MaximumMultipleConnections(int maximum);
 
 	std::wstring Format(ServerFormat formatType) const;
+	std::wstring Format(ServerFormat formatType, Credentials const& credentials) const;
 
 	bool SetEncodingType(CharsetEncoding type, std::wstring const& encoding = std::wstring());
 	bool SetCustomEncoding(std::wstring const& encoding);
@@ -133,6 +115,7 @@ public:
 
 	static ServerProtocol GetProtocolFromPrefix(std::wstring const& prefix);
 	static std::wstring GetPrefixFromProtocol(ServerProtocol const protocol);
+	static std::vector<ServerProtocol> const& GetDefaultProtocols();
 
 	// Some protocol distinguish between ASCII and binary files for line-ending
 	// conversion.
@@ -153,32 +136,64 @@ public:
 	static std::wstring GetNameFromServerType(ServerType type);
 	static ServerType GetServerTypeFromName(std::wstring const& name);
 
-	static std::wstring GetNameFromLogonType(LogonType type);
-	static LogonType GetLogonTypeFromName(std::wstring const& name);
-
-	explicit operator bool() const { return !m_host.empty(); }
+	bool empty() const { return m_host.empty(); }
+	explicit operator bool() const { return !empty(); }
 
 protected:
-	void Initialize();
-
-	ServerProtocol m_protocol;
-	ServerType m_type;
+	ServerProtocol m_protocol{UNKNOWN};
+	ServerType m_type{DEFAULT};
 	std::wstring m_host;
-	unsigned int m_port;
-	LogonType m_logonType;
+	unsigned int m_port{21};
 	std::wstring m_user;
-	std::wstring m_pass;
-	std::wstring m_account;
-	std::wstring m_keyFile;
-	int m_timezoneOffset;
-	PasvMode m_pasvMode;
-	int m_maximumMultipleConnections;
-	CharsetEncoding m_encodingType;
+	int m_timezoneOffset{};
+	PasvMode m_pasvMode{MODE_DEFAULT};
+	int m_maximumMultipleConnections{};
+	CharsetEncoding m_encodingType{ENCODING_AUTO};
 	std::wstring m_customEncoding;
 	std::wstring m_name;
 
 	std::vector<std::wstring> m_postLoginCommands;
-	bool m_bypassProxy;
+	bool m_bypassProxy{};
+};
+
+
+enum class LogonType
+{
+	anonymous,
+	normal,
+	ask, // ask should not be sent to the engine, it's intended to be used by the interface
+	interactive,
+	account,
+	key,
+
+	count
+};
+std::wstring GetNameFromLogonType(LogonType type);
+LogonType GetLogonTypeFromName(std::wstring const& name);
+
+
+class Credentials
+{
+public:
+	virtual ~Credentials() = default;
+
+	bool operator==(Credentials const& rhs) const {
+		return
+			logonType_ == rhs.logonType_ &&
+			password_ == rhs.password_ &&
+			account_ == rhs.account_ &&
+			keyFile_ == rhs.keyFile_;
+	}
+
+	void SetPass(std::wstring const& password);
+	std::wstring GetPass() const;
+
+	LogonType logonType_{LogonType::anonymous};
+	std::wstring account_;
+	std::wstring keyFile_;
+
+protected:
+	std::wstring password_;
 };
 
 #endif
