@@ -7,13 +7,14 @@
 #include "Options.h"
 #include "wrapengine.h"
 #include "buildinfo.h"
-#include <wx/tokenzr.h>
 #include "cmdline.h"
 #include "welcome_dialog.h"
-#include <msgbox.h>
+#include "msgbox.h"
 
 #include <libfilezilla/local_filesys.hpp>
 
+#include <wx/evtloop.h>
+
 #include "wxfilesystem_blob_handler.h"
 #include "xrc_helper.h"
 #ifdef __WXMSW__
@@ -29,6 +30,10 @@
 
 #include "locale_initializer.h"
 
+#ifdef USE_MAC_SANDBOX
+#include "osx_sandbox_userdirs.h"
+#endif
+
 #ifdef ENABLE_BINRELOC
 	#define BR_PTHREADS 0
 	#include "prefix.h"
@@ -40,10 +45,14 @@ IMPLEMENT_APP(CFileZillaApp)
 IMPLEMENT_APP_NO_MAIN(CFileZillaApp)
 #endif //__WXGTK__
 
+#if !wxUSE_PRINTF_POS_PARAMS
+  #error Please build wxWidgets with support for positional arguments.
+#endif
+
 CFileZillaApp::CFileZillaApp()
 {
 	m_profile_start = fz::monotonic_clock::now();
-	AddStartupProfileRecord(_T("CFileZillaApp::CFileZillaApp()"));
+	AddStartupProfileRecord("CFileZillaApp::CFileZillaApp()");
 }
 
 CFileZillaApp::~CFileZillaApp()
@@ -64,14 +73,16 @@ static void SetAppId()
 	if (!dll.Load(_T("shell32.dll")))
 		return;
 
-	if (!dll.HasSymbol(_T("SetCurrentProcessExplicitAppUserModelID")))
+	if (!dll.HasSymbol(_T("SetCurrentProcessExplicitAppUserModelID"))) {
 		return;
+	}
 
 	t_SetCurrentProcessExplicitAppUserModelID pSetCurrentProcessExplicitAppUserModelID =
 		(t_SetCurrentProcessExplicitAppUserModelID)dll.GetSymbol(_T("SetCurrentProcessExplicitAppUserModelID"));
 
-	if (!pSetCurrentProcessExplicitAppUserModelID)
+	if (!pSetCurrentProcessExplicitAppUserModelID) {
 		return;
+	}
 
 	pSetCurrentProcessExplicitAppUserModelID(_T("FileZilla.Client.AppID"));
 }
@@ -110,24 +121,28 @@ void CFileZillaApp::InitLocale()
 			wxLocale *loc = wxGetLocale();
 			const wxLanguageInfo* currentInfo = loc ? loc->GetLanguageInfo(loc->GetLanguage()) : 0;
 			if (!loc || !currentInfo) {
-				if (!pInfo)
+				if (!pInfo) {
 					error.Printf(_("Failed to set language to %s, using default system language."),
 						language);
-				else
+				}
+				else {
 					error.Printf(_("Failed to set language to %s (%s), using default system language."),
 						pInfo->Description, language);
+				}
 			}
 			else {
 				wxString currentName = currentInfo->CanonicalName;
 
-				if (!pInfo)
+				if (!pInfo) {
 					error.Printf(_("Failed to set language to %s, using default system language (%s, %s)."),
 						language, loc->GetLocale(),
 						currentName);
-				else
+				}
+				else {
 					error.Printf(_("Failed to set language to %s (%s), using default system language (%s, %s)."),
 						pInfo->Description, language, loc->GetLocale(),
 						currentName);
+				}
 			}
 
 			error += _T("\n");
@@ -138,28 +153,46 @@ void CFileZillaApp::InitLocale()
 		}
 #else
 		if (!pInfo || !SetLocale(pInfo->Language)) {
-			for( language = GetFallbackLocale(language); !language.empty(); language = GetFallbackLocale(language) ) {
+			for (language = GetFallbackLocale(language); !language.empty(); language = GetFallbackLocale(language)) {
 				const wxLanguageInfo* fallbackInfo = wxLocale::FindLanguageInfo(language);
-				if( fallbackInfo && SetLocale(fallbackInfo->Language )) {
-					COptions::Get()->SetOption(OPTION_LANGUAGE, language);
+				if (fallbackInfo && SetLocale(fallbackInfo->Language)) {
+					COptions::Get()->SetOption(OPTION_LANGUAGE, language.ToStdWstring());
 					return;
 				}
 			}
-			COptions::Get()->SetOption(OPTION_LANGUAGE, wxString());
-			if (pInfo && !pInfo->Description.empty())
+			COptions::Get()->SetOption(OPTION_LANGUAGE, std::wstring());
+			if (pInfo && !pInfo->Description.empty()) {
 				wxMessageBoxEx(wxString::Format(_("Failed to set language to %s (%s), using default system language"), pInfo->Description, language), _("Failed to change language"), wxICON_EXCLAMATION);
-			else
+			}
+			else {
 				wxMessageBoxEx(wxString::Format(_("Failed to set language to %s, using default system language"), language), _("Failed to change language"), wxICON_EXCLAMATION);
+			}
 		}
 #endif
 	}
 }
 
+namespace {
+std::wstring translator(char const* const t)
+{
+	return wxGetTranslation(t).ToStdWstring();
+}
+
+std::wstring translator_pf(char const* const singular, char const* const plural, int64_t n)
+{
+	// wxGetTranslation does not support 64bit ints on 32bit systems.
+	if (n < 0) {
+		n = -n;
+	}
+	return wxGetTranslation(singular, plural, (sizeof(unsigned int) < 8 && n > 1000000000) ? (1000000000 + n % 1000000000) : n).ToStdWstring();
+}
+}
+
 bool CFileZillaApp::OnInit()
 {
-	AddStartupProfileRecord(_T("CFileZillaApp::OnInit()"));
+	AddStartupProfileRecord("CFileZillaApp::OnInit()");
 
-	srand( (unsigned)time( NULL ) );
+	fz::set_translators(translator, translator_pf);
 
 #ifdef __WXMSW__
 	if (!InitWinsock()) {
@@ -186,8 +219,9 @@ bool CFileZillaApp::OnInit()
 #endif
 
 	int cmdline_result = ProcessCommandLine();
-	if (!cmdline_result)
+	if (!cmdline_result) {
 		return false;
+	}
 
 	LoadLocales();
 
@@ -230,6 +264,9 @@ USE AT OWN RISK"), _T("Important Information"));
 	}
 
 	CheckExistsFzsftp();
+#if ENABLE_STORJ
+	CheckExistsFzstorj();
+#endif
 
 	// Turn off idle events, we don't need them
 	wxIdleEvent::SetMode(wxIDLE_PROCESS_SPECIFIED);
@@ -244,12 +281,15 @@ USE AT OWN RISK"), _T("Important Information"));
 	m_pWrapEngine = std::make_unique<CWrapEngine>();
 	m_pWrapEngine->LoadCache();
 
+#ifdef USE_MAC_SANDBOX
+	OSXSandboxUserdirs::Get().Load();
+#endif
+
 	CMainFrame *frame = new CMainFrame();
 	frame->Show(true);
 	SetTopWindow(frame);
 
-	CWelcomeDialog *welcome_dialog = new CWelcomeDialog;
-	welcome_dialog->Run(frame, false, true);
+	CWelcomeDialog::RunDelayed(frame);
 
 	frame->ProcessCommandLine();
 	frame->PostInitialize();
@@ -261,45 +301,34 @@ USE AT OWN RISK"), _T("Important Information"));
 
 int CFileZillaApp::OnExit()
 {
+	CContextManager::Get()->NotifyGlobalHandlers(STATECHANGE_QUITNOW);
+
 	COptions::Get()->SaveIfNeeded();
 
 #ifdef WITH_LIBDBUS
 	CSessionManager::Uninit();
 #endif
+
+	if (GetMainLoop() && wxEventLoopBase::GetActive() != GetMainLoop()) {
+		// There's open dialogs and such and we just have to quit _NOW_.
+		// We cannot continue as wx proceeds to destroy all windows, which interacts
+		// badly with nested event loops, virtually always causing a crash.
+		// I very much prefer clean shutdown but it's impossible in this situation. Sad.
+		_exit(0);
+	}
+
 #ifdef __WXMSW__
 	UninitWinsock();
 #endif
 	return wxApp::OnExit();
 }
 
-bool CFileZillaApp::FileExists(const wxString& file) const
+bool CFileZillaApp::FileExists(std::wstring const& file) const
 {
-	int pos = file.Find('*');
-	if (pos < 0)
-		return wxFileExists(file);
-
-	wxASSERT(pos > 0);
-	wxASSERT(file[pos - 1] == '/');
-	wxASSERT(file.size() > static_cast<size_t>(pos + 1) && file[pos + 1] == '/');
-
-	wxLogNull nullLog;
-	wxDir dir(file.Left(pos));
-	if (!dir.IsOpened())
-		return false;
-
-	wxString subDir;
-	bool found = dir.GetFirst(&subDir, _T(""), wxDIR_DIRS);
-	while (found) {
-		if (FileExists(file.Left(pos) + subDir + file.Mid(pos + 1)))
-			return true;
-
-		found = dir.GetNext(&subDir);
-	}
-
-	return false;
+	return fz::local_filesys::get_file_type(fz::to_native(file), true) == fz::local_filesys::file;
 }
 
-CLocalPath CFileZillaApp::GetDataDir(wxString fileToFind) const
+CLocalPath CFileZillaApp::GetDataDir(std::wstring fileToFind) const
 {
 	/*
 	 * Finding the resources in all cases is a difficult task,
@@ -313,9 +342,10 @@ CLocalPath CFileZillaApp::GetDataDir(wxString fileToFind) const
 	 */
 
 #ifdef __WXMAC__
-	CLocalPath path(wxStandardPaths::Get().GetDataDir());
-	if (FileExists(path.GetPath() + fileToFind))
+	CLocalPath path(wxStandardPaths::Get().GetDataDir().ToStdWstring());
+	if (FileExists(path.GetPath() + fileToFind)) {
 		return path;
+	}
 
 	return CLocalPath();
 #else
@@ -367,27 +397,33 @@ CLocalPath CFileZillaApp::GetDataDir(wxString fileToFind) const
 	// For each path, check for the resources
 	wxPathList::const_iterator node;
 	for (node = pathList.begin(); node != pathList.end(); ++node) {
-		wxString cur = CLocalPath(*node).GetPath();
-		if (FileExists(cur + fileToFind))
+		auto const cur = CLocalPath(node->ToStdWstring()).GetPath();
+		if (FileExists(cur + fileToFind)) {
 			return CLocalPath(cur);
-		if (FileExists(cur + _T("share/filezilla/") + fileToFind))
+		}
+		if (FileExists(cur + _T("share/filezilla/") + fileToFind)) {
 			return CLocalPath(cur + _T("/share/filezilla"));
-		if (FileExists(cur + _T("filezilla/") + fileToFind))
+		}
+		if (FileExists(cur + _T("filezilla/") + fileToFind)) {
 			return CLocalPath(cur + _T("filezilla"));
+		}
 	}
 
 	for (node = pathList.begin(); node != pathList.end(); ++node) {
-		wxString cur = CLocalPath(*node).GetPath();
-		if (FileExists(cur + _T("../") + fileToFind))
+		auto const cur = CLocalPath(node->ToStdWstring()).GetPath();
+		if (FileExists(cur + _T("../") + fileToFind)) {
 			return CLocalPath(cur + _T("/.."));
-		if (FileExists(cur + _T("../share/filezilla/") + fileToFind))
+		}
+		if (FileExists(cur + _T("../share/filezilla/") + fileToFind)) {
 			return CLocalPath(cur + _T("../share/filezilla"));
+		}
 	}
 
 	for (node = pathList.begin(); node != pathList.end(); ++node) {
-		wxString cur = CLocalPath(*node).GetPath();
-		if (FileExists(cur + _T("../../") + fileToFind))
+		auto const cur = CLocalPath(node->ToStdWstring()).GetPath();
+		if (FileExists(cur + _T("../../") + fileToFind)) {
 			return CLocalPath(cur + _T("../.."));
+		}
 	}
 
 	return CLocalPath();
@@ -396,8 +432,8 @@ CLocalPath CFileZillaApp::GetDataDir(wxString fileToFind) const
 
 bool CFileZillaApp::LoadResourceFiles()
 {
-	AddStartupProfileRecord(_T("CFileZillaApp::LoadResourceFiles"));
-	m_resourceDir = GetDataDir(_T("resources/theme.xml"));
+	AddStartupProfileRecord("CFileZillaApp::LoadResourceFiles");
+	m_resourceDir = GetDataDir(_T("resources/defaultfilters.xml"));
 
 	wxImage::AddHandler(new wxPNGHandler());
 
@@ -417,23 +453,17 @@ bool CFileZillaApp::LoadResourceFiles()
 
 	InitHandlers(*pResource);
 
-	if (fz::local_filesys::get_file_type(fz::to_native(m_resourceDir.GetPath() + _T("xrc/resources.xrc"))) == fz::local_filesys::file) {
-		wxFileName fn(m_resourceDir.GetPath() + _T("xrc/resources.xrc"));
-		pResource->LoadFile(fn);
-	}
-	else {
-		fz::local_filesys fs;
-		wxString dir = m_resourceDir.GetPath() + _T("xrc/");
-		bool found = fs.begin_find_files(fz::to_native(dir), false);
-		while (found) {
-			fz::native_string name;
-			found = fs.get_next_file(name);
-			if (name.size() <= 4 || name.substr(name.size() - 4) != fzT(".xrc")) {
-				continue;
-			}
-			pResource->LoadFile(dir + name);
-		}
-	}
+	fz::local_filesys fs;
+	std::wstring dir = m_resourceDir.GetPath() + L"xrc/";
+	pResource->LoadFile(wxString(dir + L"certificate.xrc"));
+	pResource->LoadFile(wxString(dir + L"dialogs.xrc"));
+	pResource->LoadFile(wxString(dir + L"inputdialog.xrc"));
+	pResource->LoadFile(wxString(dir + L"menus.xrc"));
+	pResource->LoadFile(wxString(dir + L"netconfwizard.xrc"));
+	pResource->LoadFile(wxString(dir + L"settings.xrc"));
+	pResource->LoadFile(wxString(dir + L"sitemanager.xrc"));
+	pResource->LoadFile(wxString(dir + L"update.xrc"));
+	pResource->LoadFile(wxString(dir + L"storj.xrc"));
 
 	// Useful for XRC files with embedded image data.
 	wxFileSystem::AddHandler(new wxFileSystemBlobHandler);
@@ -443,10 +473,10 @@ bool CFileZillaApp::LoadResourceFiles()
 
 bool CFileZillaApp::InitDefaultsDir()
 {
-	AddStartupProfileRecord(_T("InitDefaultsDir"));
+	AddStartupProfileRecord("InitDefaultsDir");
 #ifdef __WXGTK__
 	m_defaultsDir = COptions::GetUnadjustedSettingsDir();
-	if( m_defaultsDir.empty() || !wxFileName::FileExists(m_defaultsDir.GetPath() + _T("fzdefaults.xml"))) {
+	if (m_defaultsDir.empty() || !wxFileName::FileExists(m_defaultsDir.GetPath() + _T("fzdefaults.xml"))) {
 		if (wxFileName::FileExists(_T("/etc/filezilla/fzdefaults.xml"))) {
 			m_defaultsDir.SetPath(_T("/etc/filezilla"));
 		}
@@ -456,7 +486,7 @@ bool CFileZillaApp::InitDefaultsDir()
 	}
 
 #endif
-	if( m_defaultsDir.empty() ) {
+	if (m_defaultsDir.empty()) {
 		m_defaultsDir = GetDataDir(_T("fzdefaults.xml"));
 	}
 
@@ -465,16 +495,17 @@ bool CFileZillaApp::InitDefaultsDir()
 
 bool CFileZillaApp::LoadLocales()
 {
-	AddStartupProfileRecord(_T("CFileZillaApp::LoadLocales"));
+	AddStartupProfileRecord("CFileZillaApp::LoadLocales");
 #ifndef __WXMAC__
-	m_localesDir = GetDataDir(_T("../locale/*/filezilla.mo"));
-	if (m_localesDir.empty())
-		m_localesDir = GetDataDir(_T("../locale/*/LC_MESSAGES/filezilla.mo"));
+	m_localesDir = GetDataDir(_T("../locale/de/filezilla.mo"));
+	if (m_localesDir.empty()) {
+		m_localesDir = GetDataDir(_T("../locale/de/LC_MESSAGES/filezilla.mo"));
+	}
 	if (!m_localesDir.empty()) {
 		m_localesDir.ChangePath( _T("../locale") );
 	}
 	else {
-		m_localesDir = GetDataDir(_T("locales/*/filezilla.mo"));
+		m_localesDir = GetDataDir(_T("locales/de/filezilla.mo"));
 		if (!m_localesDir.empty()) {
 			m_localesDir.AddSegment(_T("locales"));
 		}
@@ -501,7 +532,7 @@ bool CFileZillaApp::SetLocale(int language)
 	if (!pLocale->IsOk()) {
 		return false;
 	}
-	
+
 	// Load catalog. Only fail if a non-default language has been selected
 	if (!pLocale->AddCatalog(_T("filezilla")) && language != wxLANGUAGE_DEFAULT) {
 		return false;
@@ -530,16 +561,18 @@ bool CFileZillaApp::SetLocale(int language)
 
 int CFileZillaApp::GetCurrentLanguage() const
 {
-	if (!m_pLocale)
+	if (!m_pLocale) {
 		return wxLANGUAGE_ENGLISH;
+	}
 
 	return m_pLocale->GetLanguage();
 }
 
 wxString CFileZillaApp::GetCurrentLanguageCode() const
 {
-	if (!m_pLocale)
+	if (!m_pLocale) {
 		return wxString();
+	}
 
 	return m_pLocale->GetCanonicalName();
 }
@@ -553,8 +586,9 @@ void CFileZillaApp::OnFatalException()
 void CFileZillaApp::DisplayEncodingWarning()
 {
 	static bool displayedEncodingWarning = false;
-	if (displayedEncodingWarning)
+	if (displayedEncodingWarning) {
 		return;
+	}
 
 	displayedEncodingWarning = true;
 
@@ -568,36 +602,50 @@ CWrapEngine* CFileZillaApp::GetWrapEngine()
 
 void CFileZillaApp::CheckExistsFzsftp()
 {
-	AddStartupProfileRecord(_T("CFileZillaApp::CheckExistsFzstp"));
-	// Get the correct path to the fzsftp executable
+	AddStartupProfileRecord("FileZillaApp::CheckExistsFzsftp");
+	CheckExistsTool(L"fzsftp", {L"/src/putty", L"/putty"}, L"FZ_FZSFTP", OPTION_FZSFTP_EXECUTABLE, _("SFTP support"));
+}
+
+#if ENABLE_STORJ
+void CFileZillaApp::CheckExistsFzstorj()
+{
+	AddStartupProfileRecord("FileZillaApp::CheckExistsFzstorj");
+	CheckExistsTool(L"fzstorj", {L"/src/storj", L"/storj"}, L"FZ_FZSTORJ", OPTION_FZSTORJ_EXECUTABLE, _("Storj support"));
+}
+#endif
+
+void CFileZillaApp::CheckExistsTool(std::wstring const& tool, std::vector<std::wstring> const& searchPaths, std::wstring const& env, int setting, wxString const& description)
+{
+	// Get the correct path to the specified tool
 
 #ifdef __WXMAC__
 	wxString executable = wxStandardPaths::Get().GetExecutablePath();
 	int pos = executable.Find('/', true);
-	if (pos != -1)
+	if (pos != -1) {
 		executable = executable.Left(pos);
-	executable += _T("/fzsftp");
-	if (!wxFileName::FileExists(executable))
-	{
-		wxMessageBoxEx(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPlease download FileZilla again. If this problem persists, please submit a bug report."), executable),
+	}
+	executable += _T("/") + tool;
+	if (!wxFileName::FileExists(executable.ToStdWstring())) {
+		wxMessageBoxEx(wxString::Format(_("%s could not be found. Without this component of FileZilla, %s will not work.\n\nPlease download FileZilla again. If this problem persists, please submit a bug report."), executable, description),
 			_("File not found"), wxICON_ERROR);
 		executable.clear();
 	}
 
 #else
 
-	wxString program = _T("fzsftp");
+	wxString program = tool;
 #ifdef __WXMSW__
 	program += _T(".exe");
 #endif
 
 	bool found = false;
 
-	// First check the FZ_FZSFTP environment variable
+	// First check the given environment variable
 	wxString executable;
-	if (wxGetEnv(_T("FZ_FZSFTP"), &executable)) {
-		if (wxFileName::FileExists(executable))
+	if (wxGetEnv(env, &executable)) {
+		if (wxFileName::FileExists(executable.ToStdWstring())) {
 			found = true;
+		}
 	}
 
 	if (!found) {
@@ -624,12 +672,14 @@ void CFileZillaApp::CheckExistsFzsftp()
 
 		// Add a few paths relative to the current working directory
 		pathList.Add(cwd + _T("/bin"));
-		pathList.Add(cwd + _T("/src/putty"));
-		pathList.Add(cwd + _T("/putty"));
+		for (auto const& path : searchPaths) {
+			pathList.Add(cwd + path);
+		}
 
 		executable = pathList.FindAbsoluteValidPath(program);
-		if (!executable.empty())
+		if (!executable.empty()) {
 			found = true;
+		}
 	}
 
 #ifdef __UNIX__
@@ -655,22 +705,24 @@ void CFileZillaApp::CheckExistsFzsftp()
 		wxPathList pathList;
 		pathList.AddEnvList(_T("PATH"));
 		executable = pathList.FindAbsoluteValidPath(program);
-		if (!executable.empty())
+		if (!executable.empty()) {
 			found = true;
+		}
 	}
 
 	if (!found) {
 		// Quote path if it contains spaces
-		if (executable.Find(_T(" ")) != -1 && executable[0] != '"' && executable[0] != '\'')
+		if (executable.Find(_T(" ")) != -1 && executable[0] != '"' && executable[0] != '\'') {
 			executable = _T("\"") + executable + _T("\"");
+		}
 
-		wxMessageBoxEx(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPossible solutions:\n- Make sure %s is in a directory listed in your PATH environment variable.\n- Set the full path to %s in the FZ_FZSFTP environment variable."), program, program, program),
+		wxMessageBoxEx(wxString::Format(_("%s could not be found. Without this component of FileZilla, %s will not work.\n\nPossible solutions:\n- Make sure %s is in a directory listed in your PATH environment variable.\n- Set the full path to %s in the %s environment variable."), program, description, program, program, env),
 			_("File not found"), wxICON_ERROR | wxOK);
 		executable.clear();
 	}
 #endif
 
-	COptions::Get()->SetOption(OPTION_FZSFTP_EXECUTABLE, executable);
+	COptions::Get()->SetOption(setting, executable.ToStdWstring());
 }
 
 #ifdef __WXMSW__
@@ -687,7 +739,7 @@ extern "C" BOOL CALLBACK EnumWindowCallback(HWND hwnd, LPARAM)
 
 int CFileZillaApp::ProcessCommandLine()
 {
-	AddStartupProfileRecord(_T("CFileZillaApp::ProcessCommandLine"));
+	AddStartupProfileRecord("CFileZillaApp::ProcessCommandLine");
 	m_pCommandLine = std::make_unique<CCommandLine>(argc, argv);
 	int res = m_pCommandLine->Parse() ? 1 : -1;
 
@@ -701,8 +753,9 @@ int CFileZillaApp::ProcessCommandLine()
 
 		if (m_pCommandLine->HasSwitch(CCommandLine::version)) {
 			wxString out = wxString::Format(_T("FileZilla %s"), CBuildInfo::GetVersion());
-			if (!CBuildInfo::GetBuildType().empty())
+			if (!CBuildInfo::GetBuildType().empty()) {
 				out += _T(" ") + CBuildInfo::GetBuildType() + _T(" build");
+			}
 			out += _T(", compiled on ") + CBuildInfo::GetBuildDateString();
 
 			printf("%s\n", (const char*)out.mb_str());
@@ -713,7 +766,7 @@ int CFileZillaApp::ProcessCommandLine()
 	return res;
 }
 
-void CFileZillaApp::AddStartupProfileRecord(const wxString& msg)
+void CFileZillaApp::AddStartupProfileRecord(std::string const& msg)
 {
 	if (!m_profile_start) {
 		return;
@@ -731,7 +784,7 @@ void CFileZillaApp::ShowStartupProfile()
 
 			msg += std::to_wstring(diff.get_milliseconds());
 			msg += _T(" ");
-			msg += p.second;
+			msg += fz::to_wstring(p.second);
 			msg += _T("\n");
 		}
 		wxMessageBoxEx(msg);
@@ -741,7 +794,7 @@ void CFileZillaApp::ShowStartupProfile()
 	m_startupProfile.clear();
 }
 
-wxString CFileZillaApp::GetSettingsFile(wxString const& name) const
+std::wstring CFileZillaApp::GetSettingsFile(std::wstring const& name) const
 {
 	return COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR) + name + _T(".xml");
 }
