@@ -114,7 +114,7 @@ public:
 				return wxDragNone;
 			}
 
-			if (!m_pLocalListView->m_state.GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pLocalListView->m_state.GetServer())) {
+			if (!m_pLocalListView->m_state.GetServer() || m_pRemoteDataObject->GetServer().server != m_pLocalListView->m_state.GetServer().server) {
 				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
@@ -406,12 +406,12 @@ regular_dir:
 		bool wasLink;
 		fz::native_string name;
 		while (local_filesys.get_next_file(name, wasLink, data.dir, &data.size, &data.time, &data.attributes)) {
-			if (name.empty()) {
+			data.name = fz::to_wstring(name);
+			if (name.empty() || data.name.empty()) {
 				wxGetApp().DisplayEncodingWarning();
 				continue;
 			}
 
-			data.name = fz::to_wstring(name);
 			m_fileData.push_back(data);
 			if (!filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time)) {
 				if (data.dir)
@@ -570,8 +570,8 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 		return;
 	}
 
-	const CServer* pServer = m_state.GetServer();
-	if (!pServer) {
+	ServerWithCredentials const& server = m_state.GetServer();
+	if (!server) {
 		wxBell();
 		return;
 	}
@@ -584,7 +584,7 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 
 	const bool queue_only = action == 1;
 
-	m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, path, *pServer, data->size);
+	m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, path, server, data->size);
 	m_pQueue->QueueFile_Finish(true);
 }
 
@@ -824,8 +824,8 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 
 void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 {
-	const CServer* pServer = m_state.GetServer();
-	if (!pServer) {
+	ServerWithCredentials const& server = m_state.GetServer();
+	if (!server) {
 		wxBell();
 		return;
 	}
@@ -845,17 +845,21 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 	long item = -1;
 	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (!item && m_hasParent)
+		if (!item && m_hasParent) {
 			continue;
-		if (item == -1)
+		}
+		if (item == -1) {
 			break;
+		}
 
 		const CLocalFileData *data = GetData(item);
-		if (!data)
+		if (!data) {
 			break;
+		}
 
-		if (data->comparison_flags == fill)
+		if (data->comparison_flags == fill) {
 			continue;
+		}
 
 		CServerPath remotePath = m_state.GetRemotePath();
 		if (remotePath.empty()) {
@@ -875,13 +879,14 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 			root.add_dir_to_visit(localPath, remotePath);
 		}
 		else {
-			m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, remotePath, *pServer, data->size);
+			m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, remotePath, server, data->size);
 			added = true;
 		}
 	}
 
-	if (added)
+	if (added) {
 		m_pQueue->QueueFile_Finish(!queue_only);
+	}
 
 	if (!root.empty()) {
 		recursiveOperation->AddRecursionRoot(std::move(root));
@@ -1063,29 +1068,35 @@ bool CLocalListView::OnBeginRename(const wxListEvent& event)
 bool CLocalListView::OnAcceptRename(const wxListEvent& event)
 {
 	const int index = event.GetIndex();
-	if (!index && m_hasParent)
+	if (!index && m_hasParent) {
 		return false;
+	}
 
-	if (event.GetLabel().empty())
+	if (event.GetLabel().empty()) {
 		return false;
+	}
 
-	if (!m_state.GetLocalDir().IsWriteable())
+	if (!m_state.GetLocalDir().IsWriteable()) {
 		return false;
+	}
 
 	CLocalFileData *const data = GetData(event.GetIndex());
-	if (!data || data->comparison_flags == fill)
+	if (!data || data->comparison_flags == fill) {
 		return false;
+	}
 
 	wxString newname = event.GetLabel();
 #ifdef __WXMSW__
 	newname = newname.Left(255);
 #endif
 
-	if (newname == data->name)
+	if (newname == data->name) {
 		return false;
+	}
 
-	if (!RenameFile(this, m_dir.GetPath(), data->name, newname))
+	if (!RenameFile(this, m_dir.GetPath(), data->name, newname)) {
 		return false;
+	}
 
 	data->name = newname;
 #ifdef __WXMSW__
@@ -1174,29 +1185,30 @@ std::list<wxString> CLocalListView::RememberSelectedItems(wxString& focused)
 #endif
 	{
 		int item = -1;
-		for (;;)
-		{
+		for (;;) {
 			item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-			if (item == -1)
+			if (item < 0 || static_cast<size_t>(item) >= m_indexMapping.size()) {
 				break;
+			}
 			const CLocalFileData &data = m_fileData[m_indexMapping[item]];
-			if (data.comparison_flags != fill)
-			{
-				if (data.dir)
+			if (data.comparison_flags != fill) {
+				if (data.dir) {
 					selectedNames.push_back(_T("d") + data.name);
-				else
+				}
+				else {
 					selectedNames.push_back(_T("-") + data.name);
+				}
 			}
 			SetSelection(item, false);
 		}
 	}
 
 	int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
-	if (item != -1)
-	{
+	if (item >= 0 && static_cast<size_t>(item) < m_indexMapping.size()) {
 		const CLocalFileData &data = m_fileData[m_indexMapping[item]];
-		if (data.comparison_flags != fill)
+		if (data.comparison_flags != fill) {
 			focused = data.name;
+		}
 
 		SetItemState(item, 0, wxLIST_STATE_FOCUSED);
 	}
@@ -1209,18 +1221,17 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 	// Reselect previous items if neccessary.
 	// Sorting direction did not change. We just have to scan through items once
 
-	if (selectedNames.empty())
-	{
-		if (focused.empty())
+	if (selectedNames.empty()) {
+		if (focused.empty()) {
 			return;
-		for (unsigned int i = 0; i < m_indexMapping.size(); i++)
-		{
+		}
+		for (unsigned int i = 0; i < m_indexMapping.size(); ++i) {
 			const CLocalFileData &data = m_fileData[m_indexMapping[i]];
-			if (data.name == focused)
-			{
+			if (data.name == focused) {
 				SetItemState(i, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
-				if (ensureVisible)
+				if (ensureVisible) {
 					EnsureVisible(i);
+				}
 				return;
 			}
 		}
@@ -1230,39 +1241,40 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 	int firstSelected = -1;
 
 	int i = -1;
-	for (std::list<wxString>::const_iterator iter = selectedNames.begin(); iter != selectedNames.end(); ++iter)
-	{
-		while (++i < (int)m_indexMapping.size())
-		{
+	for (auto const& selectedName : selectedNames) {
+		while (++i < (int)m_indexMapping.size()) {
 			const CLocalFileData &data = m_fileData[m_indexMapping[i]];
-			if (data.name == focused)
-			{
+			if (data.name == focused) {
 				SetItemState(i, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
-				if (ensureVisible)
+				if (ensureVisible) {
 					EnsureVisible(i);
-				focused = _T("");
+				}
+				focused.clear();
 			}
-			if (data.dir && *iter == (_T("d") + data.name))
-			{
-				if (firstSelected == -1)
+			if (data.dir && selectedName == (_T("d") + data.name)) {
+				if (firstSelected == -1) {
 					firstSelected = i;
-				if (m_pFilelistStatusBar)
+				}
+				if (m_pFilelistStatusBar) {
 					m_pFilelistStatusBar->SelectDirectory();
+				}
 				SetSelection(i, true);
 				break;
 			}
-			else if (*iter == (_T("-") + data.name))
-			{
-				if (firstSelected == -1)
+			else if (selectedName == (_T("-") + data.name)) {
+				if (firstSelected == -1) {
 					firstSelected = i;
-				if (m_pFilelistStatusBar)
+				}
+				if (m_pFilelistStatusBar) {
 					m_pFilelistStatusBar->SelectFile(data.size);
+				}
 				SetSelection(i, true);
 				break;
 			}
 		}
-		if (i == (int)m_indexMapping.size())
+		if (i == (int)m_indexMapping.size()) {
 			break;
+		}
 	}
 	if (!focused.empty()) {
 		if (firstSelected != -1) {
@@ -1276,10 +1288,12 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 
 void CLocalListView::OnStateChange(t_statechange_notifications notification, const wxString& data, const void*)
 {
-	if (notification == STATECHANGE_LOCAL_DIR)
+	if (notification == STATECHANGE_LOCAL_DIR) {
 		DisplayDir(m_state.GetLocalDir());
-	else if (notification == STATECHANGE_APPLYFILTER)
+	}
+	else if (notification == STATECHANGE_APPLYFILTER) {
 		ApplyCurrentFilter();
+	}
 	else if (notification == STATECHANGE_SERVER) {
 		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
 	}
@@ -1578,8 +1592,9 @@ bool CLocalListView::CanStartComparison()
 wxString CLocalListView::GetItemText(int item, unsigned int column)
 {
 	CLocalFileData *data = GetData(item);
-	if (!data)
+	if (!data) {
 		return wxString();
+	}
 
 	if (!column) {
 #ifdef __WXMSW__
@@ -1589,20 +1604,25 @@ wxString CLocalListView::GetItemText(int item, unsigned int column)
 #endif
 	}
 	else if (column == 1) {
-		if (data->size < 0)
+		if (data->size < 0) {
 			return wxString();
-		else
+		}
+		else {
 			return CSizeFormat::Format(data->size);
+		}
 	}
 	else if (column == 2) {
-		if (!item && m_hasParent)
+		if (!item && m_hasParent) {
 			return wxString();
+		}
 
-		if (data->comparison_flags == fill)
+		if (data->comparison_flags == fill) {
 			return wxString();
+		}
 
-		if (data->fileType.empty())
-			data->fileType = GetType(data->name, data->dir, m_dir.GetPath()).ToStdWstring();
+		if (data->fileType.empty()) {
+			data->fileType = GetType(data->name, data->dir, m_dir.GetPath());
+		}
 
 		return data->fileType;
 	}
@@ -1614,7 +1634,7 @@ wxString CLocalListView::GetItemText(int item, unsigned int column)
 
 void CLocalListView::OnMenuEdit(wxCommandEvent&)
 {
-	CServer server;
+	ServerWithCredentials server;
 	CServerPath path;
 
 	if (!m_state.GetServer()) {
@@ -1624,7 +1644,7 @@ void CLocalListView::OnMenuEdit(wxCommandEvent&)
 		}
 	}
 	else {
-		server = *m_state.GetServer();
+		server = m_state.GetServer();
 
 		path = m_state.GetRemotePath();
 		if (path.empty()) {
