@@ -1,15 +1,17 @@
 #include <filezilla.h>
-#include "LocalTreeView.h"
-#include "queue.h"
+
+#include "dndobjects.h"
+#include "dragdropmanager.h"
+#include "drop_target_ex.h"
 #include "filezillaapp.h"
 #include "filter.h"
 #include "file_utils.h"
-#include <wx/dnd.h>
-#include "dndobjects.h"
+#include "graphics.h"
 #include "inputdialog.h"
-#include "dragdropmanager.h"
-#include "drop_target_ex.h"
+#include "LocalTreeView.h"
 #include "Options.h"
+#include "queue.h"
+#include "themeprovider.h"
 
 #include <libfilezilla/local_filesys.hpp>
 
@@ -20,16 +22,20 @@
 #include "volume_enumerator.h"
 #endif
 
+#ifdef USE_MAC_SANDBOX
+#include "osx_sandbox_userdirs.h"
+#endif
+
 #include <algorithm>
 
 class CTreeItemData : public wxTreeItemData
 {
 public:
-	CTreeItemData(const wxString& known_subdir) : m_known_subdir(known_subdir) {}
-	wxString m_known_subdir;
+	CTreeItemData(std::wstring const& known_subdir) : m_known_subdir(known_subdir) {}
+	std::wstring m_known_subdir;
 };
 
-class CLocalTreeViewDropTarget : public CScrollableDropTarget<wxTreeCtrlEx>
+class CLocalTreeViewDropTarget final : public CScrollableDropTarget<wxTreeCtrlEx>
 {
 public:
 	CLocalTreeViewDropTarget(CLocalTreeView* pLocalTreeView)
@@ -58,8 +64,9 @@ public:
 		const wxString& dir = m_pLocalTreeView->GetDirFromItem(item);
 
 #ifdef __WXMSW__
-		if (dir == _T("/"))
+		if (dir == _T("/")) {
 			return wxString();
+		}
 #endif
 
 		return dir;
@@ -70,8 +77,9 @@ public:
 		int flags = 0;
 		wxTreeItemId hit = m_pLocalTreeView->HitTest(point, flags);
 
-		if (flags & (wxTREE_HITTEST_ABOVE | wxTREE_HITTEST_BELOW | wxTREE_HITTEST_NOWHERE | wxTREE_HITTEST_TOLEFT | wxTREE_HITTEST_TORIGHT))
+		if (flags & (wxTREE_HITTEST_ABOVE | wxTREE_HITTEST_BELOW | wxTREE_HITTEST_NOWHERE | wxTREE_HITTEST_TOLEFT | wxTREE_HITTEST_TORIGHT)) {
 			return wxTreeItemId();
+		}
 
 		return hit;
 	}
@@ -87,38 +95,41 @@ public:
 		}
 
 		wxTreeItemId hit = GetHit(wxPoint(x, y));
-		if (!hit)
+		if (!hit) {
 			return wxDragNone;
+		}
 
-		const CLocalPath path(GetDirFromItem(hit));
-		if (path.empty() || !path.IsWriteable())
+		const CLocalPath path(GetDirFromItem(hit).ToStdWstring());
+		if (path.empty() || !path.IsWriteable()) {
 			return wxDragNone;
+		}
 
-		if (!GetData())
+		if (!GetData()) {
 			return wxDragError;
+		}
 
 		CDragDropManager* pDragDropManager = CDragDropManager::Get();
-		if (pDragDropManager)
+		if (pDragDropManager) {
 			pDragDropManager->pDropTarget = m_pLocalTreeView;
+		}
 
-		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat())
-			m_pLocalTreeView->m_pState->HandleDroppedFiles(m_pFileDataObject, path, def == wxDragCopy);
-		else
-		{
-			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId())
-			{
+		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat()) {
+			m_pLocalTreeView->m_state.HandleDroppedFiles(m_pFileDataObject, path, def == wxDragCopy);
+		}
+		else {
+			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId()) {
 				wxMessageBoxEx(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
 				return wxDragNone;
 			}
 
-			if (!m_pLocalTreeView->m_pState->GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pLocalTreeView->m_pState->GetServer()))
-			{
+			if (!m_pLocalTreeView->m_state.GetServer() || m_pRemoteDataObject->GetServer().server != m_pLocalTreeView->m_state.GetServer().server) {
 				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
 
-			if (!m_pLocalTreeView->m_pState->DownloadDroppedFiles(m_pRemoteDataObject, path))
+			if (!m_pLocalTreeView->m_state.DownloadDroppedFiles(m_pRemoteDataObject, path)) {
 				return wxDragNone;
+			}
 		}
 
 		return def;
@@ -133,12 +144,14 @@ public:
 		ClearDropHighlight();
 
 		wxTreeItemId hit = GetHit(wxPoint(x, y));
-		if (!hit)
+		if (!hit) {
 			return false;
+		}
 
-		const wxString dir = GetDirFromItem(hit);
-		if (dir.empty() || !CLocalPath(dir).IsWriteable())
+		std::wstring const dir = GetDirFromItem(hit).ToStdWstring();
+		if (dir.empty() || !CLocalPath(dir).IsWriteable()) {
 			return false;
+		}
 
 		return true;
 	}
@@ -210,8 +223,6 @@ protected:
 	wxDataObjectComposite* m_pDataObject;
 };
 
-IMPLEMENT_CLASS(CLocalTreeView, wxTreeCtrlEx)
-
 BEGIN_EVENT_TABLE(CLocalTreeView, wxTreeCtrlEx)
 EVT_TREE_ITEM_EXPANDING(wxID_ANY, CLocalTreeView::OnItemExpanding)
 #ifdef __WXMSW__
@@ -236,19 +247,20 @@ EVT_CHAR(CLocalTreeView::OnChar)
 EVT_MENU(XRCID("ID_OPEN"), CLocalTreeView::OnMenuOpen)
 END_EVENT_TABLE()
 
-CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState *pState, CQueueView *pQueueView)
+CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState& state, CQueueView *pQueueView)
 	: wxTreeCtrlEx(parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxTR_LINES_AT_ROOT | wxTR_HAS_BUTTONS | wxNO_BORDER),
-	CSystemImageList(16),
-	CStateEventHandler(pState),
+	CSystemImageList(CThemeProvider::GetIconSize(iconSizeSmall).x),
+	CStateEventHandler(state),
 	m_pQueueView(pQueueView)
 {
-	wxGetApp().AddStartupProfileRecord(_T("CLocalTreeView::CLocalTreeView"));
+	wxGetApp().AddStartupProfileRecord("CLocalTreeView::CLocalTreeView");
 #ifdef __WXMAC__
 	SetFont(wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT));
 #endif
 
-	pState->RegisterHandler(this, STATECHANGE_LOCAL_DIR);
-	pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
+	state.RegisterHandler(this, STATECHANGE_LOCAL_DIR);
+	state.RegisterHandler(this, STATECHANGE_APPLYFILTER);
+	state.RegisterHandler(this, STATECHANGE_SERVER);
 
 	SetImageList(GetSystemImageList());
 
@@ -266,10 +278,19 @@ CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState *pState,
 	SetItemImage(root, GetIconIndex(iconType::dir), wxTreeItemIcon_Expanded);
 	SetItemImage(root, GetIconIndex(iconType::opened_dir), wxTreeItemIcon_SelectedExpanded);
 
+#if USE_MAC_SANDBOX
+	auto userDirs = OSXSandboxUserdirs::Get().GetDirs();
+	for (auto const& dir : userDirs) {
+		SetDir(dir);
+	}
+#endif
+
 	SetDir(_T("/"));
 #endif
 
 	SetDropTarget(new CLocalTreeViewDropTarget(this));
+
+	m_windowTinter = std::make_unique<CWindowTinter>(*this);
 }
 
 CLocalTreeView::~CLocalTreeView()
@@ -279,16 +300,14 @@ CLocalTreeView::~CLocalTreeView()
 #endif
 }
 
-void CLocalTreeView::SetDir(wxString localDir)
+void CLocalTreeView::SetDir(wxString const& localDir)
 {
-	if (m_currentDir == localDir)
-	{
+	if (m_currentDir == localDir) {
 		RefreshListing();
 		return;
 	}
 
-	if (localDir.Left(2) == _T("\\\\"))
-	{
+	if (localDir.Left(2) == _T("\\\\")) {
 		// TODO: UNC path, don't display it yet
 		m_currentDir = _T("");
 		SafeSelectItem(wxTreeItemId());
@@ -297,8 +316,7 @@ void CLocalTreeView::SetDir(wxString localDir)
 	m_currentDir = localDir;
 
 #ifdef __WXMSW__
-	if (localDir == _T("\\"))
-	{
+	if (localDir == _T("\\")) {
 		SafeSelectItem(m_drives);
 		return;
 	}
@@ -306,20 +324,19 @@ void CLocalTreeView::SetDir(wxString localDir)
 
 	wxString subDirs = localDir;
 	wxTreeItemId parent = GetNearestParent(subDirs);
-	if (!parent)
-	{
+	if (!parent) {
 		SafeSelectItem(wxTreeItemId());
 		return;
 	}
 
-	if (subDirs.empty())
-	{
+	if (subDirs.empty()) {
 		SafeSelectItem(parent);
 		return;
 	}
 	wxTreeItemId item = MakeSubdirs(parent, localDir.Left(localDir.Length() - subDirs.Length()), subDirs);
-	if (!item)
+	if (!item) {
 		return;
+	}
 
 	SafeSelectItem(item);
 }
@@ -398,9 +415,9 @@ wxTreeItemId CLocalTreeView::GetSubdir(wxTreeItemId parent, const wxString& subD
 
 bool CLocalTreeView::DisplayDrives(wxTreeItemId parent)
 {
-	wxGetApp().AddStartupProfileRecord(_T("CLocalTreeView::DisplayDrives"));
+	wxGetApp().AddStartupProfileRecord("CLocalTreeView::DisplayDrives");
 
-	std::list<wxString> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
+	std::list<std::wstring> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
 
 	m_pVolumeEnumeratorThread = new CVolumeDescriptionEnumeratorThread(this);
 	if (m_pVolumeEnumeratorThread->Failed()) {
@@ -408,25 +425,24 @@ bool CLocalTreeView::DisplayDrives(wxTreeItemId parent)
 		m_pVolumeEnumeratorThread = 0;
 	}
 
-	for( std::list<wxString>::const_iterator it = drives.begin(); it != drives.end(); ++it ) {
+	for (auto it = drives.begin(); it != drives.end(); ++it) {
 		wxString drive = *it;
 		if (drive.Right(1) == _T("\\"))
 			drive = drive.RemoveLast();
 
-		wxGetApp().AddStartupProfileRecord(wxString::Format(_T("CLocalTreeView::DisplayDrives adding drive %s"), drive.c_str()));
-		wxTreeItemId item = AppendItem(parent, drive, GetIconIndex(iconType::dir, drive));
+		wxTreeItemId item = AppendItem(parent, drive, GetIconIndex(iconType::dir, _T(""), false));
 		AppendItem(item, _T(""));
-		SortChildren(parent);
 	}
+	SortChildren(parent);
 
-	wxGetApp().AddStartupProfileRecord(_T("CLocalTreeView::DisplayDrives adding drives done"));
+	wxGetApp().AddStartupProfileRecord("CLocalTreeView::DisplayDrives adding drives done");
 
 	return true;
 }
 
 #endif
 
-void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, const wxString& knownSubdir /*=_T("")*/)
+void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, std::wstring const& knownSubdir)
 {
 	fz::local_filesys local_filesys;
 
@@ -435,8 +451,9 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 		if (!local_filesys.begin_find_files(fz::to_native(dirname), true)) {
 			if (!knownSubdir.empty()) {
 				wxTreeItemId item = GetSubdir(parent, knownSubdir);
-				if (item != wxTreeItemId())
+				if (item != wxTreeItemId()) {
 					return;
+				}
 
 				const wxString fullName = dirname + knownSubdir;
 				item = AppendItem(parent, knownSubdir, GetIconIndex(iconType::dir, fullName),
@@ -448,20 +465,19 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 					);
 				CheckSubdirStatus(item, fullName);
 			}
-			else
-			{
-				m_setSelection = true;
+			else {
+				++m_setSelection;
 				DeleteChildren(parent);
-				m_setSelection = false;
+				--m_setSelection;
 			}
 			return;
 		}
 	}
 
 	wxASSERT(parent);
-	m_setSelection = true;
+	++m_setSelection;
 	DeleteChildren(parent);
-	m_setSelection = false;
+	--m_setSelection;
 
 	CFilterManager filter;
 
@@ -475,26 +491,28 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 	fz::datetime date;
 	while (local_filesys.get_next_file(file, wasLink, is_dir, 0, &date, &attributes)) {
 		wxASSERT(is_dir);
-		if (file.empty()) {
+		std::wstring wfile = fz::to_wstring(file);
+		if (file.empty() || wfile.empty()) {
 			wxGetApp().DisplayEncodingWarning();
 			continue;
 		}
-		wxString file_wx = file;
 
-		wxString fullName = dirname + file_wx;
+		std::wstring fullName = dirname.ToStdWstring() + wfile;
 #ifdef __WXMSW__
-		if (file_wx.CmpNoCase(knownSubdir))
+		if (fz::stricmp(wfile, knownSubdir))
 #else
-		if (file_wx != knownSubdir)
+		if (wfile != knownSubdir)
 #endif
 		{
-			if (filter.FilenameFiltered(file_wx, dirname, true, size, true, attributes, date))
+			if (filter.FilenameFiltered(wfile, dirname, true, size, true, attributes, date)) {
 				continue;
+			}
 		}
-		else
+		else {
 			matchedKnown = true;
+		}
 
-		wxTreeItemId item = AppendItem(parent, file_wx, GetIconIndex(iconType::dir, fullName),
+		wxTreeItemId item = AppendItem(parent, wfile, GetIconIndex(iconType::dir, fullName),
 #ifdef __WXMSW__
 				-1
 #else
@@ -528,8 +546,9 @@ wxString CLocalTreeView::HasSubdir(const wxString& dirname)
 	CFilterManager filter;
 
 	fz::local_filesys local_filesys;
-	if (!local_filesys.begin_find_files(fz::to_native(dirname), true))
+	if (!local_filesys.begin_find_files(fz::to_native(dirname), true)) {
 		return wxString();
+	}
 
 	fz::native_string file;
 	bool wasLink;
@@ -539,16 +558,17 @@ wxString CLocalTreeView::HasSubdir(const wxString& dirname)
 	fz::datetime date;
 	while (local_filesys.get_next_file(file, wasLink, is_dir, 0, &date, &attributes)) {
 		wxASSERT(is_dir);
-		if (file.empty()) {
+		std::wstring wfile = fz::to_wstring(file);
+		if (file.empty() || wfile.empty()) {
 			wxGetApp().DisplayEncodingWarning();
 			continue;
 		}
 
-		wxString file_wx = file;
-		if (filter.FilenameFiltered(file_wx, dirname, true, size, true, attributes, date))
+		if (filter.FilenameFiltered(wfile, dirname, true, size, true, attributes, date)) {
 			continue;
+		}
 
-		return file_wx;
+		return wfile;
 	}
 
 	return wxString();
@@ -558,10 +578,14 @@ wxTreeItemId CLocalTreeView::MakeSubdirs(wxTreeItemId parent, wxString dirname,
 {
 	const wxString& separator = wxFileName::GetPathSeparator();
 
+	wxString segment;
 	while (!subDir.empty()) {
 		int pos = subDir.Find(separator);
-		wxString segment;
-		if (pos == -1) {
+		if (!pos) {
+			subDir = subDir.Mid(1);
+			continue;
+		}
+		else if (pos == -1) {
 			segment = subDir;
 			subDir = _T("");
 		}
@@ -570,7 +594,7 @@ wxTreeItemId CLocalTreeView::MakeSubdirs(wxTreeItemId parent, wxString dirname,
 			subDir = subDir.Mid(pos + 1);
 		}
 
-		DisplayDir(parent, dirname, segment);
+		DisplayDir(parent, dirname, segment.ToStdWstring());
 
 		wxTreeItemId item = GetSubdir(parent, segment);
 		if (!item)
@@ -591,8 +615,10 @@ void CLocalTreeView::OnItemExpanding(wxTreeEvent& event)
 
 	wxTreeItemIdValue value;
 	wxTreeItemId child = GetFirstChild(item, value);
-	if (child && GetItemText(child).empty())
+	if (child && GetItemText(child).empty()) {
+		wxCHECK_RET(!m_setSelection, "OnItemExpanding called on an item with empty child during item selection of one of its children.");
 		DisplayDir(item, GetDirFromItem(item));
+	}
 }
 
 wxString CLocalTreeView::GetDirFromItem(wxTreeItemId item)
@@ -698,10 +724,10 @@ void CLocalTreeView::RefreshListing()
 		}
 	}
 #else
-	t_dir dir;
-	dir.dir = separator;
-	dir.item = GetRootItem();
-	dirsToCheck.push_back(dir);
+	t_dir root_dir;
+	root_dir.dir = separator;
+	root_dir.item = GetRootItem();
+	dirsToCheck.push_back(root_dir);
 #endif
 
 	CFilterManager filter;
@@ -726,7 +752,7 @@ void CLocalTreeView::RefreshListing()
 		}
 
 		// Step 2: Enumerate subdirectories on disk and sort them
-		std::vector<wxString> dirs;
+		std::vector<std::wstring> dirs;
 
 		fz::native_string file;
 		static int64_t const size(-1);
@@ -735,16 +761,17 @@ void CLocalTreeView::RefreshListing()
 		int attributes;
 		fz::datetime date;
 		while (local_filesys.get_next_file(file, was_link, is_dir, 0, &date, &attributes)) {
-			if (file.empty()) {
+			std::wstring wfile = fz::to_wstring(file);
+			if (file.empty() || wfile.empty()) {
 				wxGetApp().DisplayEncodingWarning();
 				continue;
 			}
 
-			wxString file_wx = file;
-			if (filter.FilenameFiltered(file_wx, dir.dir, true, size, true, attributes, date))
+			if (filter.FilenameFiltered(wfile, dir.dir, true, size, true, attributes, date)) {
 				continue;
+			}
 
-			dirs.push_back(file_wx);
+			dirs.emplace_back(std::move(wfile));
 		}
 		auto const& sortFunc = CFileListCtrlSortBase::GetCmpFunction(m_nameSortMode);
 		std::sort(dirs.begin(), dirs.end(), [&](auto const& lhs, auto const& rhs) { return sortFunc(lhs, rhs) < 0; });
@@ -832,27 +859,34 @@ void CLocalTreeView::OnSelectionChanged(wxTreeEvent& event)
 	}
 
 	wxTreeItemId item = event.GetItem();
-	if (!item)
+	if (!item) {
 		return;
+	}
 
-	wxString dir = GetDirFromItem(item);
+	std::wstring dir = GetDirFromItem(item).ToStdWstring();
 
-	wxString error;
-	if (!m_pState->SetLocalDir(dir, &error)) {
-		if (!error.empty())
+	std::wstring error;
+	if (!m_state.SetLocalDir(dir, &error)) {
+		if (!error.empty()) {
 			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
-		else
+		}
+		else {
 			wxBell();
-		m_setSelection = true;
+		}
+		++m_setSelection;
 		SelectItem(event.GetOldItem());
-		m_setSelection = false;
+		--m_setSelection;
 	}
 }
 
-void CLocalTreeView::OnStateChange(CState*, enum t_statechange_notifications notification, const wxString&, const void*)
+void CLocalTreeView::OnStateChange(t_statechange_notifications notification, const wxString&, const void*)
 {
-	if (notification == STATECHANGE_LOCAL_DIR)
-		SetDir(m_pState->GetLocalDir().GetPath());
+	if (notification == STATECHANGE_LOCAL_DIR) {
+		SetDir(m_state.GetLocalDir().GetPath());
+	}
+	else if (notification == STATECHANGE_SERVER) {
+		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+	}
 	else {
 		wxASSERT(notification == STATECHANGE_APPLYFILTER);
 		RefreshListing();
@@ -861,6 +895,10 @@ void CLocalTreeView::OnStateChange(CState*, enum t_statechange_notifications not
 
 void CLocalTreeView::OnBeginDrag(wxTreeEvent& event)
 {
+	if (COptions::Get()->GetOptionVal(OPTION_DND_DISABLED) != 0) {
+		return;
+	}
+
 	wxTreeItemId item = event.GetItem();
 	if (!item)
 		return;
@@ -905,7 +943,7 @@ void CLocalTreeView::OnBeginDrag(wxTreeEvent& event)
 	{
 		// We only need to refresh local side if the operation got handled
 		// externally, the internal handlers do this for us already
-		m_pState->RefreshLocal();
+		m_state.RefreshLocal();
 	}
 }
 
@@ -913,35 +951,28 @@ void CLocalTreeView::OnBeginDrag(wxTreeEvent& event)
 
 wxString CLocalTreeView::GetSpecialFolder(int folder, int &iconIndex, int &openIconIndex)
 {
-	LPITEMIDLIST list;
-	if (SHGetSpecialFolderLocation((HWND)GetHandle(), folder, &list) != S_OK)
-		return wxString();
+	wxString name;
 
-	SHFILEINFO shFinfo;
-	if (!SHGetFileInfo((LPCTSTR)list, 0, &shFinfo, sizeof(shFinfo), SHGFI_PIDL | SHGFI_ICON | SHGFI_SMALLICON))
-		return wxString();
+	LPITEMIDLIST list{};
+	if (SHGetSpecialFolderLocation((HWND)GetHandle(), folder, &list) == S_OK) {
+		SHFILEINFO shFinfo{};
+		if (SHGetFileInfo((LPCTSTR)list, 0, &shFinfo, sizeof(shFinfo), SHGFI_PIDL | SHGFI_ICON | SHGFI_SMALLICON) != 0) {
+			if (shFinfo.hIcon) {
+				DestroyIcon(shFinfo.hIcon);
+			}
+			iconIndex = shFinfo.iIcon;
+		}
 
-	DestroyIcon(shFinfo.hIcon);
-	iconIndex = shFinfo.iIcon;
+		if (SHGetFileInfo((LPCTSTR)list, 0, &shFinfo, sizeof(shFinfo), SHGFI_PIDL | SHGFI_ICON | SHGFI_SMALLICON | SHGFI_OPENICON | SHGFI_DISPLAYNAME) != 0) {
+			if (shFinfo.hIcon) {
+				DestroyIcon(shFinfo.hIcon);
+			}
+			openIconIndex = shFinfo.iIcon;
+			name = shFinfo.szDisplayName;
+		}
 
-	if (!SHGetFileInfo((LPCTSTR)list, 0, &shFinfo, sizeof(shFinfo), SHGFI_PIDL | SHGFI_ICON | SHGFI_SMALLICON | SHGFI_OPENICON | SHGFI_DISPLAYNAME))
-		return wxString();
-
-	DestroyIcon(shFinfo.hIcon);
-	openIconIndex = shFinfo.iIcon;
-
-	wxString name = shFinfo.szDisplayName;
-
-	LPMALLOC pMalloc;
-	SHGetMalloc(&pMalloc);
-
-	if (pMalloc)
-	{
-		pMalloc->Free(list);
-		pMalloc->Release();
+		CoTaskMemFree(list);
 	}
-	else
-		wxLogLastError(wxT("SHGetMalloc"));
 
 	return name;
 }
@@ -986,30 +1017,33 @@ bool CLocalTreeView::CreateRoot()
 
 void CLocalTreeView::OnVolumesEnumerated(wxCommandEvent& event)
 {
-	if (!m_pVolumeEnumeratorThread)
+	if (!m_pVolumeEnumeratorThread) {
 		return;
+	}
 
 	std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo> volumeInfo;
 	volumeInfo = m_pVolumeEnumeratorThread->GetVolumes();
 
-	if (event.GetEventType() == fzEVT_VOLUMESENUMERATED)
-	{
+	if (event.GetEventType() == fzEVT_VOLUMESENUMERATED) {
 		delete m_pVolumeEnumeratorThread;
 		m_pVolumeEnumeratorThread = 0;
 	}
 
-	for (std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo>::const_iterator iter = volumeInfo.begin(); iter != volumeInfo.end(); ++iter)
-	{
+	for (std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo>::const_iterator iter = volumeInfo.begin(); iter != volumeInfo.end(); ++iter) {
 		wxString drive = iter->volume;
 
 		wxTreeItemIdValue tmp;
 		wxTreeItemId item = GetFirstChild(m_drives, tmp);
-		while (item)
-		{
+		while (item) {
 			wxString name = GetItemText(item);
-			if (name == drive || name.Left(drive.Len() + 1) == drive + _T(" "))
-			{
-				SetItemText(item, drive + _T(" (") + iter->volumeName + _T(")"));
+			if (name == drive || name.Left(drive.Len() + 1) == drive + _T(" ")) {
+
+				if (!iter->volumeName.empty()) {
+					SetItemText(item, drive + _T(" (") + iter->volumeName + _T(")"));
+				}
+				if (iter->icon != -1) {
+					SetItemImage(item, iter->icon);
+				}
 				break;
 			}
 			item = GetNextChild(m_drives, tmp);
@@ -1022,22 +1056,26 @@ void CLocalTreeView::OnVolumesEnumerated(wxCommandEvent& event)
 void CLocalTreeView::OnContextMenu(wxTreeEvent& event)
 {
 	m_contextMenuItem = event.GetItem();
-	if (!m_contextMenuItem.IsOk())
+	if (!m_contextMenuItem.IsOk()) {
 		return;
+	}
 
 	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_LOCALTREE"));
-	if (!pMenu)
+	if (!pMenu) {
 		return;
+	}
 
-	const CLocalPath path(GetDirFromItem(m_contextMenuItem));
+	CLocalPath const path(GetDirFromItem(m_contextMenuItem).ToStdWstring());
 
 	const bool hasParent = path.HasParent();
 	const bool writeable = path.IsWriteable();
 
-	const bool remoteConnected = m_pState->IsRemoteConnected() && !m_pState->GetRemotePath().empty();
+	const bool remoteConnected = m_state.IsRemoteConnected() && !m_state.GetRemotePath().empty();
 
-	pMenu->Enable(XRCID("ID_UPLOAD"), hasParent && remoteConnected);
-	pMenu->Enable(XRCID("ID_ADDTOQUEUE"), hasParent && remoteConnected);
+	bool const idle = m_state.GetLocalRecursiveOperation() && !m_state.GetLocalRecursiveOperation()->IsActive();
+
+	pMenu->Enable(XRCID("ID_UPLOAD"), hasParent && remoteConnected && idle);
+	pMenu->Enable(XRCID("ID_ADDTOQUEUE"), hasParent && remoteConnected && idle);
 	pMenu->Enable(XRCID("ID_MKDIR"), writeable);
 	pMenu->Enable(XRCID("ID_DELETE"), writeable && hasParent);
 	pMenu->Enable(XRCID("ID_RENAME"), writeable && hasParent);
@@ -1048,26 +1086,47 @@ void CLocalTreeView::OnContextMenu(wxTreeEvent& event)
 
 void CLocalTreeView::OnMenuUpload(wxCommandEvent& event)
 {
-	if (!m_contextMenuItem.IsOk())
+	auto recursiveOperation = m_state.GetLocalRecursiveOperation();
+	if (recursiveOperation->IsActive()) {
 		return;
+	}
 
-	CLocalPath path(GetDirFromItem(m_contextMenuItem));
-
-	if (!path.HasParent())
+	if (!m_contextMenuItem.IsOk()) {
 		return;
+	}
 
-	if (!m_pState->IsRemoteConnected())
+	CLocalPath path(GetDirFromItem(m_contextMenuItem).ToStdWstring());
+
+	if (!path.HasParent()) {
 		return;
+	}
 
-	const CServer server = *m_pState->GetServer();
-	CServerPath remotePath = m_pState->GetRemotePath();
-	if (remotePath.empty())
+	if (!m_state.IsRemoteConnected()) {
 		return;
+	}
 
-	if (!remotePath.ChangePath(GetItemText(m_contextMenuItem)))
+	ServerWithCredentials const& server = m_state.GetServer();
+	if (!server) {
 		return;
+	}
 
-	m_pQueueView->QueueFolder(event.GetId() == XRCID("ID_ADDTOQUEUE"), false, path, remotePath, server);
+	CServerPath remotePath = m_state.GetRemotePath();
+	if (remotePath.empty()) {
+		return;
+	}
+
+	if (!remotePath.ChangePath(GetItemText(m_contextMenuItem).ToStdWstring())) {
+		return;
+	}
+
+	local_recursion_root root;
+	root.add_dir_to_visit(path, remotePath);
+	recursiveOperation->AddRecursionRoot(std::move(root));
+
+	bool const queue_only = event.GetId() == XRCID("ID_ADDTOQUEUE");
+
+	CFilterManager filter;
+	recursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), !queue_only);
 }
 
 // Create a new Directory
@@ -1076,26 +1135,27 @@ void CLocalTreeView::OnMenuMkdir(wxCommandEvent&)
 	wxString newdir = MenuMkdir();
 	if (!newdir.empty()) {
 		RefreshListing();
-		m_pState->RefreshLocal();
+		m_state.RefreshLocal();
 	}
 }
 
 // Create a new Directory and enter the new Directory
 void CLocalTreeView::OnMenuMkdirChgDir(wxCommandEvent&)
 {
-	wxString newdir = MenuMkdir();
+	std::wstring newdir = MenuMkdir().ToStdWstring();
 	if (newdir.empty()) {
 		return;
 	}
 
 	// OnMenuEnter
-	wxString error;
-	if (!m_pState->SetLocalDir(newdir, &error))
-	{
-		if (!error.empty())
+	std::wstring error;
+	if (!m_state.SetLocalDir(newdir, &error)) {
+		if (!error.empty()) {
 			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
-		else
+		}
+		else {
 			wxBell();
+		}
 	}
 }
 
@@ -1103,33 +1163,35 @@ void CLocalTreeView::OnMenuMkdirChgDir(wxCommandEvent&)
 // Returns the name of the new directory
 wxString CLocalTreeView::MenuMkdir()
 {
-	if (!m_contextMenuItem.IsOk())
+	if (!m_contextMenuItem.IsOk()) {
 		return wxString();
+	}
 
-	wxString path = GetDirFromItem(m_contextMenuItem);
-	if (path.Last() != wxFileName::GetPathSeparator())
+	std::wstring path = GetDirFromItem(m_contextMenuItem).ToStdWstring();
+	if (!path.empty() && path.back() != wxFileName::GetPathSeparator()) {
 		path += wxFileName::GetPathSeparator();
+	}
 
-	if (!CLocalPath(path).IsWriteable())
-	{
+	if (!CLocalPath(path).IsWriteable()) {
 		wxBell();
 		return wxString();
 	}
 
 	CInputDialog dlg;
-	if (!dlg.Create(this, _("Create directory"), _("Please enter the name of the directory which should be created:")))
+	if (!dlg.Create(this, _("Create directory"), _("Please enter the name of the directory which should be created:"))) {
 		return wxString();
+	}
 
 	wxString newName = _("New directory");
 	dlg.SetValue(path + newName);
-	dlg.SelectText(path.Len(), path.Len() + newName.Len());
+	dlg.SelectText(path.size(), path.size() + newName.size());
 
-	if (dlg.ShowModal() != wxID_OK)
+	if (dlg.ShowModal() != wxID_OK) {
 		return wxString();
+	}
 
 	wxFileName fn(dlg.GetValue(), _T(""));
-	if (!fn.Normalize(wxPATH_NORM_ALL, path))
-	{
+	if (!fn.Normalize(wxPATH_NORM_ALL, path)) {
 		wxBell();
 		return wxString();
 	}
@@ -1160,7 +1222,7 @@ void CLocalTreeView::OnMenuRename(wxCommandEvent&)
 	}
 #endif
 
-	CLocalPath path(GetDirFromItem(m_contextMenuItem));
+	CLocalPath path(GetDirFromItem(m_contextMenuItem).ToStdWstring());
 	if (!path.HasParent() || !path.IsWriteable()) {
 		wxBell();
 		return;
@@ -1171,39 +1233,47 @@ void CLocalTreeView::OnMenuRename(wxCommandEvent&)
 
 void CLocalTreeView::OnMenuDelete(wxCommandEvent&)
 {
-	if (!m_contextMenuItem.IsOk())
+	if (!m_contextMenuItem.IsOk()) {
 		return;
+	}
 
-	wxString path = GetDirFromItem(m_contextMenuItem);
+	std::wstring path = GetDirFromItem(m_contextMenuItem).ToStdWstring();
 
 	CLocalPath local_path(path);
-	if (!local_path.HasParent() || !local_path.IsWriteable())
+	if (!local_path.HasParent() || !local_path.IsWriteable()) {
 		return;
+	}
 
 	gui_recursive_remove rmd(this);
 	rmd.remove(fz::to_native(path));
 
 	wxTreeItemId item = GetSelection();
-	while (item && item != m_contextMenuItem)
+	while (item && item != m_contextMenuItem) {
 		item = GetItemParent(item);
+	}
 
 	if (!item) {
-		if (GetItemParent(m_contextMenuItem) == GetSelection())
-			m_pState->RefreshLocal();
-		else
+		if (GetItemParent(m_contextMenuItem) == GetSelection()) {
+			m_state.RefreshLocal();
+		}
+		else {
 			RefreshListing();
+		}
 		return;
 	}
 
-	if (!path.empty() && path.Last() == wxFileName::GetPathSeparator())
-		path.RemoveLast();
-	int pos = path.Find(wxFileName::GetPathSeparator(), true);
-	if (pos < 1)
+	if (!path.empty() && path.back() == wxFileName::GetPathSeparator()) {
+		path.pop_back();
+	}
+	size_t pos = path.rfind(wxFileName::GetPathSeparator());
+	if (pos == std::wstring::npos || !pos) {
 		path = _T("/");
-	else
-		path = path.Left(pos);
+	}
+	else {
+		path = path.substr(0, pos);
+	}
 
-	m_pState->SetLocalDir(path);
+	m_state.SetLocalDir(path);
 	RefreshListing();
 }
 
@@ -1220,7 +1290,7 @@ void CLocalTreeView::OnBeginLabelEdit(wxTreeEvent& event)
 	}
 #endif
 
-	CLocalPath path(GetDirFromItem(item));
+	CLocalPath path(GetDirFromItem(item).ToStdWstring());
 
 	if (!path.HasParent() || !path.IsWriteable())
 	{
@@ -1247,7 +1317,7 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	}
 #endif
 
-	wxString path = GetDirFromItem(item);
+	std::wstring path = GetDirFromItem(item).ToStdWstring();
 
 	CLocalPath local_path(path);
 	if (!local_path.HasParent() || !local_path.IsWriteable()) {
@@ -1256,16 +1326,17 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 		return;
 	}
 
-	if (!path.empty() && path.Last() == wxFileName::GetPathSeparator())
-		path.RemoveLast();
+	if (!path.empty() && path.back() == wxFileName::GetPathSeparator()) {
+		path.pop_back();
+	}
 
-	int pos = path.Find(wxFileName::GetPathSeparator(), true);
-	wxASSERT(pos != -1);
+	size_t pos = path.rfind(wxFileName::GetPathSeparator());
+	wxASSERT(pos != std::wstring::npos);
 
-	wxString parent = path.Left(pos + 1);
+	std::wstring parent = path.substr(0, pos + 1);
 
 	const wxString& oldName = GetItemText(item);
-	const wxString& newName = event.GetLabel();
+	std::wstring const newName = event.GetLabel().ToStdWstring();
 	if (newName.empty()) {
 		wxBell();
 		event.Veto();
@@ -1274,8 +1345,9 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 
 	wxASSERT(parent + oldName == path);
 
-	if (oldName == newName)
+	if (oldName == newName) {
 		return;
+	}
 
 	if (!RenameFile(this, parent, oldName, newName)) {
 		event.Veto();
@@ -1295,15 +1367,15 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	}
 
 	if (item == currentSel) {
-		m_pState->SetLocalDir(parent + newName);
+		m_state.SetLocalDir(parent + newName);
 		return;
 	}
 
-	wxString sub;
+	std::wstring sub;
 
 	wxTreeItemId tmp = currentSel;
 	while (tmp != GetRootItem() && tmp != item) {
-		sub = wxFileName::GetPathSeparator() + GetItemText(tmp) + sub;
+		sub = (wxFileName::GetPathSeparator() + GetItemText(tmp) + sub).ToStdWstring();
 		tmp = GetItemParent(tmp);
 	}
 
@@ -1313,7 +1385,7 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	}
 
 	// Current selection below renamed item
-	m_pState->SetLocalDir(parent + newName + sub);
+	m_state.SetLocalDir(parent + newName + sub);
 }
 
 void CLocalTreeView::OnChar(wxKeyEvent& event)
@@ -1354,19 +1426,22 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 #endif
 
 	if (child) {
-		if (!GetItemText(child).empty())
+		if (!GetItemText(child).empty()) {
 			return false;
+		}
 
 		CTreeItemData* pData = (CTreeItemData*)GetItemData(child);
 		if (pData) {
 			bool wasLink;
 			int attributes;
-			enum fz::local_filesys::type type;
+			fz::local_filesys::type type;
 			fz::datetime date;
-			if (!path.empty() && path.Last() == fz::local_filesys::path_separator)
+			if (!path.empty() && path.Last() == fz::local_filesys::path_separator) {
 				type = fz::local_filesys::get_file_info(fz::to_native(path + pData->m_known_subdir), wasLink, 0, &date, &attributes);
-			else
+			}
+			else {
 				type = fz::local_filesys::get_file_info(fz::to_native(path + fz::local_filesys::path_separator + pData->m_known_subdir), wasLink, 0, &date, &attributes);
+			}
 			if (type == fz::local_filesys::dir) {
 				CFilterManager filter;
 				if (!filter.FilenameFiltered(pData->m_known_subdir, path, true, size, true, attributes, date))
@@ -1378,7 +1453,7 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 	wxString sub = HasSubdir(path);
 	if (!sub.empty()) {
 		wxTreeItemId subItem = AppendItem(item, _T(""));
-		SetItemData(subItem, new CTreeItemData(sub));
+		SetItemData(subItem, new CTreeItemData(sub.ToStdWstring()));
 	}
 	else if (child)
 		Delete(child);
@@ -1419,8 +1494,8 @@ void CLocalTreeView::OnDevicechange(WPARAM wParam, LPARAM lParam)
 			mask <<= 1;
 		}
 
-		if (GetSelection() == m_drives && m_pState)
-			m_pState->RefreshLocal();
+		if (GetSelection() == m_drives)
+			m_state.RefreshLocal();
 	}
 }
 
