@@ -14,6 +14,7 @@
 #include "context_control.h"
 #include "defaultfileexistsdlg.h"
 #include "edithandler.h"
+#include "encoding_converter.h"
 #include "export.h"
 #include "filelist_statusbar.h"
 #include "filezillaapp.h"
@@ -201,17 +202,19 @@ protected:
 	{
 		if (notification == STATECHANGE_CHANGEDCONTEXT) {
 			// Update window title
-			const CServer* pServer = pState ? pState->GetServer() : 0;
-			if (!pServer)
+			if (!pState || !pState->GetServer()) {
 				m_pMainFrame->SetTitle(_T("FileZilla"));
-			else
+			}
+			else {
 				m_pMainFrame->SetTitle(pState->GetTitle() + _T(" - FileZilla"));
+			}
 
 			return;
 		}
 
-		if (!pState)
+		if (!pState) {
 			return;
+		}
 
 		if (!m_pMainFrame->m_pContextControl) {
 			return;
@@ -223,17 +226,18 @@ protected:
 		}
 
 		if (!controls->used()) {
-			if (notification == STATECHANGE_REMOTE_IDLE || notification == STATECHANGE_SERVER)
+			if (notification == STATECHANGE_REMOTE_IDLE || notification == STATECHANGE_SERVER) {
 				pState->Disconnect();
+			}
 
 			return;
 		}
 
 		if (notification == STATECHANGE_SERVER) {
-			const CServer* pServer = pState->GetServer();
+			ServerWithCredentials const& server = pState->GetServer();
 
 			if (pState == CContextManager::Get()->GetCurrentContext()) {
-				if (!pServer) {
+				if (!server) {
 					m_pMainFrame->SetTitle(_T("FileZilla"));
 				}
 				else {
@@ -248,8 +252,51 @@ protected:
 	CMainFrame* m_pMainFrame;
 };
 
+
+/*#include "overlay.h"
+#include <wx/hyperlink.h>
+
+namespace {
+void ShowStorjOverlay(wxWindow* parent, wxWindow* anchor, wxPoint const& offset)
+{
+	auto p = new OverlayWindow(parent);
+
+	auto box = new wxBoxSizer(wxVERTICAL);
+	auto sizer = new wxFlexGridSizer(1, wxSize(5, 5));
+	box->Add(sizer, 0, wxALL, 7);
+
+	p->SetSizer(box);
+
+	auto title = new wxStaticText(p, -1, _("Did you know about this new FileZilla feature?"));
+
+	wxFont f = title->GetFont();
+	f.SetWeight(wxBOLD);
+	title->SetFont(f);
+	sizer->Add(title);
+
+	sizer->AddSpacer(0);
+
+	sizer->Add(new wxStaticText(p, -1, _("You can use FileZilla to store your files securely in the Storj decentralized cloud.")));
+	sizer->Add(new wxHyperlinkCtrl(p, -1, _("Click to learn more"), L"https://wiki.filezilla-project.org/Storj"));
+
+	sizer->AddSpacer(10);
+
+	//sizer->Add(new wxCheckBox(p, -1, L"Don't show this again"), 0, wxALIGN_RIGHT);
+
+	auto dismiss = new wxButton(p, -1, L"Dismiss");
+	dismiss->Bind(wxEVT_BUTTON, [p](wxCommandEvent&) { p->Destroy(); });
+	sizer->Add(dismiss, 0, wxALIGN_RIGHT);
+
+	wxGetApp().GetWrapEngine()->WrapRecursive(p, 3);
+
+	box->Fit(p);
+
+	p->SetAnchor(anchor, offset);
+}
+}*/
+
 CMainFrame::CMainFrame()
-	: m_engineContext(*COptions::Get())
+	: m_engineContext(*COptions::Get(), CustomEncodingConverter::Get())
 	, m_comparisonToggleAcceleratorId(wxNewId())
 {
 	m_pActivityLed[0] = m_pActivityLed[1] = 0;
@@ -338,10 +385,12 @@ CMainFrame::CMainFrame()
 	m_pQueueLogSplitter->SetSashGravity(0.5);
 	m_pQueuePane = new CQueue(m_pQueueLogSplitter, this, m_pAsyncRequestQueue);
 
-	if (message_log_position == 1)
+	if (message_log_position == 1) {
 		m_pStatusView = new CStatusView(m_pQueueLogSplitter, -1);
-	else
+	}
+	else {
 		m_pStatusView = new CStatusView(m_pTopSplitter, -1);
+	}
 
 	m_pQueueView = m_pQueuePane->GetQueueView();
 
@@ -554,23 +603,28 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		OpenSiteManager();
 	}
 	else if (event.GetId() == XRCID("ID_MENU_FILE_COPYSITEMANAGER")) {
+		ServerWithCredentials server;
 		CState* pState = CContextManager::Get()->GetCurrentContext();
-		const CServer* pServer = pState ? pState->GetServer() : 0;
-		if (!pServer) {
+		if (pState) {
+			server = pState->GetServer();
+		}
+		if (!server) {
 			wxMessageBoxEx(_("Not connected to any server."), _("Cannot add server to Site Manager"), wxICON_EXCLAMATION);
 			return;
 		}
-		OpenSiteManager(pServer);
+		OpenSiteManager(&server);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_SERVER_CMD")) {
 		CState* pState = CContextManager::Get()->GetCurrentContext();
-		if (!pState || !pState->m_pCommandQueue || !pState->IsRemoteConnected() || !pState->IsRemoteIdle())
+		if (!pState || !pState->m_pCommandQueue || !pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
 			return;
+		}
 
 		CInputDialog dlg;
 		dlg.Create(this, _("Enter custom command"), _("Please enter raw FTP command.\nUsing raw ftp commands will clear the directory cache."));
-		if (dlg.ShowModal() != wxID_OK)
+		if (dlg.ShowModal() != wxID_OK) {
 			return;
+		}
 
 		pState = CContextManager::Get()->GetCurrentContext();
 		if (!pState || !pState->m_pCommandQueue || !pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
@@ -793,11 +847,11 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		// controls->last_bookmark_path can get modified if it's empty now
 		int res;
 		if (event.GetId() == XRCID("ID_BOOKMARK_ADD")) {
-			CNewBookmarkDialog dlg(this, sitePath, old_site.m_server ? &old_site.m_server : 0);
+			CNewBookmarkDialog dlg(this, sitePath, old_site.server_ ? &old_site.server_ : 0);
 			res = dlg.Run(pState->GetLocalDir().GetPath(), pState->GetRemotePath());
 		}
 		else {
-			CBookmarksDialog dlg(this, sitePath, old_site.m_server ? &old_site.m_server : 0);
+			CBookmarksDialog dlg(this, sitePath, old_site.server_ ? &old_site.server_ : 0);
 			res = dlg.Run();
 		}
 		if (res == wxID_OK) {
@@ -887,26 +941,33 @@ void CMainFrame::DoOnEngineEvent(CFileZillaEngine* engine)
 	const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
 	CState* pState = 0;
 	for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); ++iter) {
-		if ((*iter)->m_pEngine != engine)
+		if ((*iter)->m_pEngine != engine) {
 			continue;
+		}
 
 		pState = *iter;
 		break;
 	}
-	if (!pState)
+	if (!pState) {
 		return;
+	}
 
 	std::unique_ptr<CNotification> pNotification = pState->m_pEngine->GetNextNotification();
 	while (pNotification) {
 		switch (pNotification->GetID())
 		{
 		case nId_logmsg:
-			m_pStatusView->AddToLog(static_cast<CLogmsgNotification&>(*pNotification.get()));
-			if (COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION) == 2)
+			if (m_pStatusView) {
+				m_pStatusView->AddToLog(static_cast<CLogmsgNotification&>(*pNotification.get()));
+			}
+			if (COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION) == 2 && m_pQueuePane) {
 				m_pQueuePane->Highlight(3);
+			}
 			break;
 		case nId_operation:
-			pState->m_pCommandQueue->Finish(unique_static_cast<COperationNotification>(std::move(pNotification)));
+			if (pState->m_pCommandQueue) {
+				pState->m_pCommandQueue->Finish(unique_static_cast<COperationNotification>(std::move(pNotification)));
+			}
 			if (m_bQuit) {
 				Close();
 				return;
@@ -915,18 +976,26 @@ void CMainFrame::DoOnEngineEvent(CFileZillaEngine* engine)
 		case nId_listing:
 			{
 				auto const& listingNotification = static_cast<CDirectoryListingNotification const&>(*pNotification.get());
-				pState->m_pCommandQueue->ProcessDirectoryListing(listingNotification);
+				if (pState->m_pCommandQueue) {
+					pState->m_pCommandQueue->ProcessDirectoryListing(listingNotification);
+				}
 			}
 			break;
 		case nId_asyncrequest:
 			{
 				auto pAsyncRequest = unique_static_cast<CAsyncRequestNotification>(std::move(pNotification));
-				if (pAsyncRequest->GetRequestID() == reqId_fileexists)
-					m_pQueueView->ProcessNotification(pState->m_pEngine, std::move(pAsyncRequest));
+				if (pAsyncRequest->GetRequestID() == reqId_fileexists) {
+					if (m_pQueueView) {
+						m_pQueueView->ProcessNotification(pState->m_pEngine, std::move(pAsyncRequest));
+					}
+				}
 				else {
-					if (pAsyncRequest->GetRequestID() == reqId_certificate)
+					if (pAsyncRequest->GetRequestID() == reqId_certificate) {
 						pState->SetSecurityInfo(static_cast<CCertificateNotification&>(*pAsyncRequest));
-					m_pAsyncRequestQueue->AddRequest(pState->m_pEngine, std::move(pAsyncRequest));
+					}
+					if (m_pAsyncRequestQueue) {
+						m_pAsyncRequestQueue->AddRequest(pState->m_pEngine, std::move(pAsyncRequest));
+					}
 				}
 			}
 			break;
@@ -968,8 +1037,9 @@ void CMainFrame::OnUpdateLedTooltip(wxCommandEvent&)
 	wxFileOffset uploadSpeed = m_pQueueView ? m_pQueueView->GetCurrentUploadSpeed() : 0;
 
 	CSizeFormat::_format format = static_cast<CSizeFormat::_format>(COptions::Get()->GetOptionVal(OPTION_SIZE_FORMAT));
-	if (format == CSizeFormat::bytes)
+	if (format == CSizeFormat::bytes) {
 		format = CSizeFormat::iec;
+	}
 
 	const wxString downloadSpeedStr = CSizeFormat::Format(downloadSpeed, true, format,
 														  COptions::Get()->GetOptionVal(OPTION_SIZE_USETHOUSANDSEP) != 0,
@@ -986,11 +1056,11 @@ void CMainFrame::OnUpdateLedTooltip(wxCommandEvent&)
 bool CMainFrame::CreateMainToolBar()
 {
 	wxGetApp().AddStartupProfileRecord("CMainFrame::CreateMainToolBar");
-	if (m_pToolBar)
-	{
+	if (m_pToolBar) {
 #ifdef __WXMAC__
-		if (m_pToolBar)
+		if (m_pToolBar) {
 			COptions::Get()->SetOption(OPTION_TOOLBAR_HIDDEN, m_pToolBar->IsShown() ? 0 : 1);
+		}
 #endif
 		SetToolBar(0);
 		delete m_pToolBar;
@@ -998,20 +1068,28 @@ bool CMainFrame::CreateMainToolBar()
 	}
 
 #ifndef __WXMAC__
-	if (COptions::Get()->GetOptionVal(OPTION_TOOLBAR_HIDDEN) != 0)
+	if (COptions::Get()->GetOptionVal(OPTION_TOOLBAR_HIDDEN) != 0) {
 		return true;
+	}
 #endif
 
 	m_pToolBar = CToolBar::Load(this);
-	if (!m_pToolBar)
-	{
+	if (!m_pToolBar) {
 		wxLogError(_("Cannot load toolbar from resource file"));
 		return false;
 	}
 	SetToolBar(m_pToolBar);
 
-	if (m_pQuickconnectBar)
+#ifdef __WXMAC__
+	if (COptions::Get()->GetOptionVal(OPTION_TOOLBAR_HIDDEN) != 0) {
+		m_pToolBar->Show(false);
+	}
+#endif
+
+
+	if (m_pQuickconnectBar) {
 		m_pQuickconnectBar->Refresh();
+	}
 
 	return true;
 }
@@ -1063,8 +1141,7 @@ BOOL CALLBACK FzEnumThreadWndProc(HWND hwnd, LPARAM lParam)
 bool CMainFrame::CloseDialogsAndQuit(wxCloseEvent &event)
 {
 #ifndef __WXMAC__
-	if (m_taskBarIcon)
-	{
+	if (m_taskBarIcon) {
 		delete m_taskBarIcon;
 		m_taskBarIcon = 0;
 		m_closeEvent = event.GetEventType();
@@ -1083,16 +1160,27 @@ bool CMainFrame::CloseDialogsAndQuit(wxCloseEvent &event)
 		wxWindowList::reverse_iterator iter = wxTopLevelWindows.rbegin();
 		wxTopLevelWindow* pTop = (wxTopLevelWindow*)(*iter);
 		while (pTop != this && (size != prev_size || pLast != pTop)) {
+			if (!pTop) {
+				++iter;
+				if (iter == wxTopLevelWindows.rend()) {
+					break;
+				}
+				pTop = (wxTopLevelWindow*)(*iter);
+				continue;
+			}
+
 			wxDialog* pDialog = dynamic_cast<wxDialog*>(pTop);
-			if (pDialog)
+			if (pDialog) {
 				pDialog->EndModal(wxID_CANCEL);
+			}
 			else {
 				wxWindow* pParent = pTop->GetParent();
 				if (m_pQueuePane && pParent == m_pQueuePane) {
 					// It's the AUI frame manager hint window. Ignore it
 					++iter;
-					if (iter == wxTopLevelWindows.rend())
+					if (iter == wxTopLevelWindows.rend()) {
 						break;
+					}
 					pTop = (wxTopLevelWindow*)(*iter);
 					continue;
 				}
@@ -1114,8 +1202,7 @@ bool CMainFrame::CloseDialogsAndQuit(wxCloseEvent &event)
 	// wxMessageBoxEx does not use wxTopLevelWindow, close it too
 	bool dialog = false;
 	EnumThreadWindows(GetCurrentThreadId(), FzEnumThreadWndProc, (LPARAM)&dialog);
-	if (dialog)
-	{
+	if (dialog) {
 		m_closeEvent = event.GetEventType();
 		m_closeEventTimer.Start(1, true);
 
@@ -1189,19 +1276,22 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 		RememberSplitterPositions();
 
 #ifdef __WXMAC__
-		if (m_pToolBar)
+		if (m_pToolBar) {
 			COptions::Get()->SetOption(OPTION_TOOLBAR_HIDDEN, m_pToolBar->IsShown() ? 0 : 1);
+		}
 #endif
 		m_bQuit = true;
 	}
 
 	Show(false);
-	if (!CloseDialogsAndQuit(event))
+	if (!CloseDialogsAndQuit(event)) {
 		return;
+	}
 
 	// Getting deleted by wxWidgets
-	for (int i = 0; i < 2; i++)
+	for (int i = 0; i < 2; ++i) {
 		m_pActivityLed[i] = 0;
+	}
 	m_pStatusBar = 0;
 	m_pMenuBar = 0;
 	m_pToolBar = 0;
@@ -1211,7 +1301,7 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 	m_pStateEventHandler = 0;
 
 	if (m_pQueueView && !m_pQueueView->Quit()) {
-		if( event.CanVeto() ) {
+		if (event.CanVeto()) {
 			event.Veto();
 		}
 		return;
@@ -1234,13 +1324,14 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 		}
 
 		if (pState->m_pCommandQueue) {
-			if (!pState->m_pCommandQueue->Quit())
+			if (!pState->m_pCommandQueue->Quit()) {
 				res = false;
+			}
 		}
 	}
 
 	if (!res) {
-		if( event.CanVeto() ) {
+		if (event.CanVeto()) {
 			event.Veto();
 		}
 		return;
@@ -1249,7 +1340,7 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 	CContextControl::_context_controls* controls = m_pContextControl ? m_pContextControl->GetCurrentControls() : 0;
 	if (controls) {
 		Site site = controls->pState->GetLastSite();
-		COptions::Get()->SetLastServer(site.m_server);
+		COptions::Get()->SetLastServer(site.server_);
 		COptions::Get()->SetOption(OPTION_LASTSERVERPATH, controls->pState->GetLastServerPath().GetSafePath());
 		COptions::Get()->SetOption(OPTION_LAST_CONNECTED_SITE, site.m_path);
 	}
@@ -1319,7 +1410,7 @@ void CMainFrame::OnTimer(wxTimerEvent& event)
 #endif
 }
 
-void CMainFrame::OpenSiteManager(CServer const* pServer)
+void CMainFrame::OpenSiteManager(ServerWithCredentials const* pServer)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
 	if (!pState) {
@@ -1354,7 +1445,7 @@ void CMainFrame::OpenSiteManager(CServer const* pServer)
 
 		CSiteManagerDialog::_connected_site connected_site;
 		connected_site.old_path = path;
-		connected_site.server = site.m_server;
+		connected_site.server = site.server_;
 		connected_sites.push_back(connected_site);
 		handled_paths.insert(path);
 	}
@@ -1423,8 +1514,9 @@ void CMainFrame::OnProcessQueue(wxCommandEvent& event)
 void CMainFrame::OnMenuEditSettings(wxCommandEvent&)
 {
 	CSettingsDialog dlg(m_engineContext);
-	if (!dlg.Create(this))
+	if (!dlg.Create(this)) {
 		return;
+	}
 
 	COptions* pOptions = COptions::Get();
 
@@ -1453,8 +1545,9 @@ void CMainFrame::OnMenuEditSettings(wxCommandEvent&)
 
 void CMainFrame::OnToggleLogView(wxCommandEvent&)
 {
-	if (!m_pTopSplitter)
+	if (!m_pTopSplitter) {
 		return;
+	}
 
 	bool shown;
 
@@ -1896,12 +1989,8 @@ void CMainFrame::OnSitemanagerDropdown(wxCommandEvent& event)
 bool CMainFrame::ConnectToSite(Site & data, Bookmark const& bookmark)
 {
 	// First check if we need to ask user for a password
-	if (data.m_server.GetLogonType() == ASK ||
-		(data.m_server.GetLogonType() == INTERACTIVE && data.m_server.GetUser().empty()))
-	{
-		if (!CLoginManager::Get().GetPassword(data.m_server, false, data.m_server.GetName())) {
-			return false;
-		}
+	if (!CLoginManager::Get().GetPassword(data.server_, false, data.server_.server.GetName())) {
+		return false;
 	}
 
 	// Check if current state is already connected and if needed ask whether to open in new tab
@@ -2366,14 +2455,14 @@ void CMainFrame::ProcessCommandLine()
 	if (!param.empty()) {
 		std::wstring error;
 
-		CServer server;
+		ServerWithCredentials server;
 
 		wxString logontype = pCommandLine->GetOption(CCommandLine::logontype);
 		if (logontype == _T("ask")) {
-			server.SetLogonType(ASK);
+			server.SetLogonType(LogonType::ask);
 		}
 		else if (logontype == _T("interactive")) {
-			server.SetLogonType(INTERACTIVE);
+			server.SetLogonType(LogonType::interactive);
 		}
 
 		CServerPath path;
@@ -2383,20 +2472,16 @@ void CMainFrame::ProcessCommandLine()
 			wxMessageBoxEx(error, _("Syntax error in command line"));
 		}
 
-		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) && server.GetLogonType() == NORMAL) {
-			server.SetLogonType(ASK);
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) && server.credentials.logonType_ == LogonType::normal) {
+			server.SetLogonType(LogonType::ask);
 			CLoginManager::Get().RememberPassword(server);
 		}
-		else if (server.GetLogonType() == ASK ||
-			(server.GetLogonType() == INTERACTIVE && server.GetUser().empty()))
-		{
-			if (!CLoginManager::Get().GetPassword(server, false)) {
-				return;
-			}
+		else if (!CLoginManager::Get().GetPassword(server, false)) {
+			return;
 		}
 
 		Site site;
-		site.m_server = server;
+		site.server_ = server;
 		Bookmark bm;
 		bm.m_remoteDir = path;
 		ConnectToSite(site, bm);
@@ -2568,15 +2653,14 @@ void CMainFrame::PostInitialize()
 
 #if FZ_MANUALUPDATECHECK
 	// Need to do this after welcome screen to avoid simultaneous display of multiple dialogs
-	if( !m_pUpdater ) {
+	if (!m_pUpdater) {
 		update_dialog_timer_.SetOwner(this);
 		m_pUpdater = new CUpdater(*this, m_engineContext);
 		m_pUpdater->Init();
 	}
 #endif
 
-	if (COptions::Get()->GetOptionVal(OPTION_INTERFACE_SITEMANAGER_ON_STARTUP) != 0)
-	{
+	if (COptions::Get()->GetOptionVal(OPTION_INTERFACE_SITEMANAGER_ON_STARTUP) != 0) {
 		Show();
 		OpenSiteManager();
 	}
@@ -2584,14 +2668,16 @@ void CMainFrame::PostInitialize()
 
 void CMainFrame::OnMenuNewTab(wxCommandEvent&)
 {
-	if (m_pContextControl)
+	if (m_pContextControl) {
 		m_pContextControl->CreateTab();
+	}
 }
 
 void CMainFrame::OnMenuCloseTab(wxCommandEvent&)
 {
-	if (!m_pContextControl)
+	if (!m_pContextControl) {
 		return;
+	}
 
 	m_pContextControl->CloseTab(m_pContextControl->GetCurrentTab());
 }
@@ -2600,12 +2686,14 @@ void CMainFrame::OnToggleToolBar(wxCommandEvent& event)
 {
 	COptions::Get()->SetOption(OPTION_TOOLBAR_HIDDEN, event.IsChecked() ? 0 : 1);
 #ifdef __WXMAC__
-	if (m_pToolBar)
-		m_pToolBar->Show( event.IsChecked() );
+	if (m_pToolBar) {
+		m_pToolBar->Show(event.IsChecked());
+	}
 #else
 	CreateMainToolBar();
-	if (m_pToolBar)
+	if (m_pToolBar) {
 		m_pToolBar->UpdateToolbarState();
+	}
 	HandleResize();
 #endif
 }
