@@ -14,7 +14,7 @@
 #include "remote_recursive_operation.h"
 #include "themeprovider.h"
 
-#include "uri.h"
+#include <libfilezilla/uri.hpp>
 
 #include <wx/clipbrd.h>
 
@@ -57,8 +57,9 @@ public:
 		int flags = 0;
 		wxTreeItemId hit = m_pRemoteTreeView->HitTest(point, flags);
 
-		if (flags & (wxTREE_HITTEST_ABOVE | wxTREE_HITTEST_BELOW | wxTREE_HITTEST_NOWHERE | wxTREE_HITTEST_TOLEFT | wxTREE_HITTEST_TORIGHT))
+		if (flags & (wxTREE_HITTEST_ABOVE | wxTREE_HITTEST_BELOW | wxTREE_HITTEST_NOWHERE | wxTREE_HITTEST_TOLEFT | wxTREE_HITTEST_TORIGHT)) {
 			return wxTreeItemId();
+		}
 
 		return hit;
 	}
@@ -71,32 +72,34 @@ public:
 			return def;
 
 		wxTreeItemId hit = GetHit(wxPoint(x, y));
-		if (!hit)
+		if (!hit) {
 			return wxDragNone;
+		}
 
 		CServerPath path = m_pRemoteTreeView->GetPathFromItem(hit);
-		if (path.empty())
+		if (path.empty()) {
 			return wxDragNone;
+		}
 
-		if (!GetData())
+		if (!GetData()) {
 			return wxDragError;
+		}
 
 		CDragDropManager* pDragDropManager = CDragDropManager::Get();
-		if (pDragDropManager)
+		if (pDragDropManager) {
 			pDragDropManager->pDropTarget = m_pRemoteTreeView;
+		}
 
-		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat())
+		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat()) {
 			m_pRemoteTreeView->m_state.UploadDroppedFiles(m_pFileDataObject, path, false);
-		else
-		{
-			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId())
-			{
+		}
+		else {
+			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId()) {
 				wxMessageBoxEx(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
 				return wxDragNone;
 			}
 
-			if (!m_pRemoteTreeView->m_state.GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pRemoteTreeView->m_state.GetServer()))
-			{
+			if (!m_pRemoteTreeView->m_state.GetServer() || m_pRemoteDataObject->GetServer().server != m_pRemoteTreeView->m_state.GetServer().server) {
 				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
@@ -219,8 +222,6 @@ protected:
 	wxDataObjectComposite* m_pDataObject;
 };
 
-IMPLEMENT_CLASS(CRemoteTreeView, wxTreeCtrlEx)
-
 BEGIN_EVENT_TABLE(CRemoteTreeView, wxTreeCtrlEx)
 EVT_TREE_ITEM_EXPANDING(wxID_ANY, CRemoteTreeView::OnItemExpanding)
 EVT_TREE_SEL_CHANGED(wxID_ANY, CRemoteTreeView::OnSelectionChanged)
@@ -258,7 +259,6 @@ CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState& state,
 	UpdateSortMode();
 	RegisterOption(OPTION_FILELIST_NAMESORT);
 
-	m_busy = false;
 	m_pQueue = pQueue;
 	AddRoot(_T(""));
 	m_ExpandAfterList = wxTreeItemId();
@@ -324,8 +324,7 @@ void CRemoteTreeView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 	Freeze();
 #endif
 	wxTreeItemId parent = MakeParent(pListing->path, !modified);
-	if (!parent)
-	{
+	if (!parent) {
 		m_busy = false;
 #ifndef __WXMSW__
 		Thaw();
@@ -444,11 +443,12 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 
 wxBitmap CRemoteTreeView::CreateIcon(int index, const wxString& overlay)
 {
+	auto * imageList = GetSystemImageList();
 #ifdef __WXMSW__
 	// Need to use wxImageList::GetIcon, wxImageList::GetBitmap kills the alpha channel on MSW...
-	wxBitmap bmp = GetSystemImageList()->GetIcon(index);
+	wxBitmap bmp = imageList ? imageList->GetIcon(index) : wxBitmap();
 #else
-	wxBitmap bmp = GetSystemImageList()->GetBitmap(index);
+	wxBitmap bmp = imageList ? imageList->GetBitmap(index) : wxBitmap();
 #endif
 	if (!overlay.empty()) {
 		wxBitmap unknown = CThemeProvider::Get()->CreateBitmap(overlay, wxART_OTHER, bmp.GetScaledSize());
@@ -464,12 +464,23 @@ wxBitmap CRemoteTreeView::CreateIcon(int index, const wxString& overlay)
 
 void CRemoteTreeView::CreateImageList()
 {
-	wxSize s = CThemeProvider::GetIconSize(iconSizeSmall);
-	m_pImageList = new wxImageList(s.x, s.y, true, 4);
+	if (!GetSystemImageList()) {
+		return;
+	}
 
 	// Normal directory
 	int index = GetIconIndex(iconType::dir, _T("{78013B9C-3532-4fe1-A418-5CD1955127CC}"), false);
-	m_pImageList->Add(CreateIcon(index));
+	wxBitmap dirIcon = CreateIcon(index);
+
+	wxSize s = dirIcon.GetScaledSize();
+
+	// Must create image list only after we know actual icon size,
+	// wxSystemSettings cannot return the correct size due to a bug in
+	// Windows. On Vista and possibly 7, the documentation does not
+	// match Windows' actual behavior.
+	m_pImageList = new wxImageList(s.x, s.y, true, 4);
+
+	m_pImageList->Add(dirIcon);
 	m_pImageList->Add(CreateIcon(index, _T("ART_UNKNOWN")));
 
 	// Opened directory
@@ -670,15 +681,16 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 
 void CRemoteTreeView::OnItemExpanding(wxTreeEvent& event)
 {
-	if (m_busy)
+	if (m_busy) {
 		return;
+	}
 
 	wxTreeItemId item = event.GetItem();
-	if (!item)
+	if (!item) {
 		return;
+	}
 
-	if (!ListExpand(item))
-	{
+	if (!ListExpand(item)) {
 		event.Veto();
 		return;
 	}
@@ -711,10 +723,12 @@ void CRemoteTreeView::SetItemImages(wxTreeItemId item, bool unknown)
 
 void CRemoteTreeView::OnSelectionChanged(wxTreeEvent& event)
 {
-	if (event.GetItem() != m_ExpandAfterList)
+	if (event.GetItem() != m_ExpandAfterList) {
 		m_ExpandAfterList = wxTreeItemId();
-	if (m_busy)
+	}
+	if (m_busy) {
 		return;
+	}
 
 	if (!m_state.IsRemoteIdle(true)) {
 		wxBell();
@@ -722,13 +736,15 @@ void CRemoteTreeView::OnSelectionChanged(wxTreeEvent& event)
 	}
 
 	wxTreeItemId item = event.GetItem();
-	if (!item)
+	if (!item) {
 		return;
+	}
 
 	const CServerPath path = GetPathFromItem(item);
 	wxASSERT(!path.empty());
-	if (path.empty())
+	if (path.empty()) {
 		return;
+	}
 
 	m_state.ChangeRemoteDir(path);
 }
@@ -776,27 +792,29 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 	}
 
 	const wxTreeItemId& item = event.GetItem();
-	if (!item)
+	if (!item) {
 		return;
+	}
 
 	CServerPath path = GetPathFromItem(item);
-	if (path.empty() || !path.HasParent())
+	if (path.empty() || !path.HasParent()) {
 		return;
+	}
 
 	const CServerPath& parent = path.GetParent();
 	const wxString& lastSegment = path.GetLastSegment();
-	if (lastSegment.empty())
+	if (lastSegment.empty()) {
 		return;
+	}
 
 	wxDataObjectComposite object;
 
-	CServer const* pServer = m_state.GetServer();
-	if (!pServer)
+	ServerWithCredentials server = m_state.GetServer(); // Make a copy as DoDragDrop later runs the event loop
+	if (!server) {
 		return;
-	CServer const server = *pServer;
+	}
 
-
-	CRemoteDataObject *pRemoteDataObject = new CRemoteDataObject(*pServer, parent);
+	CRemoteDataObject *pRemoteDataObject = new CRemoteDataObject(server, parent);
 	pRemoteDataObject->AddFile(lastSegment, true, -1, false);
 
 	pRemoteDataObject->Finalize();
@@ -819,7 +837,7 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 
 	CDragDropManager* pDragDropManager = CDragDropManager::Init();
 	pDragDropManager->pDragSource = this;
-	pDragDropManager->server = *pServer;
+	pDragDropManager->server = server;
 	pDragDropManager->remoteParent = parent;
 
 	wxDropSource source(this);
@@ -836,8 +854,8 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 #if FZ3_USESHELLEXT
 	if (ext) {
 		if (!pRemoteDataObject->DidSendData()) {
-			pServer = m_state.GetServer();
-			if (!pServer || !m_state.IsRemoteIdle() || *pServer != server) {
+			server = m_state.GetServer();
+			if (!server || !m_state.IsRemoteIdle() || server != server) {
 				wxBell();
 				return;
 			}
@@ -859,8 +877,9 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 {
 	m_contextMenuItem = event.GetItem();
 	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_REMOTETREE"));
-	if (!pMenu)
+	if (!pMenu) {
 		return;
+	}
 
 	const CServerPath& path = m_contextMenuItem ? GetPathFromItem(m_contextMenuItem) : CServerPath();
 	if (!m_state.IsRemoteIdle() || path.empty()) {
@@ -1377,14 +1396,14 @@ void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
 		return;
 	}
 
-	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
+	CServerPath const& path = GetPathFromItem(m_contextMenuItem);
 	if (path.empty()) {
 		wxBell();
 		return;
 	}
 
-	const CServer *pServer = m_state.GetServer();
-	if (!pServer) {
+	ServerWithCredentials const& server = m_state.GetServer();
+	if (!server) {
 		wxBell();
 		return;
 	}
@@ -1394,7 +1413,7 @@ void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
 		return;
 	}
 
-	wxString url = pServer->Format((event.GetId() == XRCID("ID_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
+	wxString url = server.Format((event.GetId() == XRCID("ID_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
 
 	std::wstring const pathPart = fz::percent_encode_w(path.GetPath(), true);
 	if (!pathPart.empty() && pathPart[0] != '/') {
