@@ -12,69 +12,78 @@ CShellExtensionInterface::CShellExtensionInterface()
 {
 	m_shellExtension = 0;
 
-	CoCreateInstance(CLSID_ShellExtension, NULL,
+	int res = CoCreateInstance(CLSID_ShellExtension, NULL,
 	  CLSCTX_INPROC_SERVER | CLSCTX_LOCAL_SERVER, IID_IUnknown,
 	  &m_shellExtension);
+	if (res != S_OK) {
+		m_shellExtension = 0;
+	}
 
-	if (m_shellExtension)
+	if (m_shellExtension) {
 		m_hMutex = CreateMutex(0, false, _T("FileZilla3DragDropExtLogMutex"));
-	else
+	}
+	else {
 		m_hMutex = 0;
+	}
 
 	m_hMapping = 0;
 }
 
 CShellExtensionInterface::~CShellExtensionInterface()
 {
-	if (m_shellExtension)
-	{
+	if (m_shellExtension) {
 		((IUnknown*)m_shellExtension)->Release();
 		CoFreeUnusedLibraries();
 	}
 
-	if (m_hMapping)
+	if (m_hMapping) {
 		CloseHandle(m_hMapping);
+	}
 
-	if (m_hMutex)
+	if (m_hMutex) {
 		CloseHandle(m_hMutex);
+	}
 
-	if (!m_dragDirectory.empty())
-		RemoveDirectory(m_dragDirectory.wc_str());
+	if (!m_dragDirectory.empty()) {
+		RemoveDirectory(m_dragDirectory.c_str());
+	}
 }
 
 wxString CShellExtensionInterface::InitDrag()
 {
-	if (!m_shellExtension)
+	if (!m_shellExtension) {
 		return wxString();
+	}
 
-	if (!m_hMutex)
+	if (!m_hMutex) {
 		return wxString();
+	}
 
-	if (!CreateDragDirectory())
+	if (!CreateDragDirectory()) {
 		return wxString();
+	}
 
 	m_hMapping = CreateFileMapping(INVALID_HANDLE_VALUE, 0, PAGE_READWRITE, 0, DRAG_EXT_MAPPING_LENGTH, DRAG_EXT_MAPPING);
-	if (!m_hMapping)
+	if (!m_hMapping) {
 		return wxString();
+	}
 
 	char* data = (char*)MapViewOfFile(m_hMapping, FILE_MAP_READ | FILE_MAP_WRITE, 0, 0, DRAG_EXT_MAPPING_LENGTH);
-	if (!data)
-	{
+	if (!data) {
 		CloseHandle(m_hMapping);
 		m_hMapping = 0;
 		return wxString();
 	}
 
 	DWORD result = WaitForSingleObject(m_hMutex, 250);
-	if (result != WAIT_OBJECT_0)
-	{
+	if (result != WAIT_OBJECT_0) {
 		UnmapViewOfFile(data);
 		return wxString();
 	}
 
 	*data = DRAG_EXT_VERSION;
 	data[1] = 1;
-	wcscpy((wchar_t*)(data + 2), m_dragDirectory.wc_str(wxConvLocal));
+	wcscpy((wchar_t*)(data + 2), m_dragDirectory.c_str());
 
 	ReleaseMutex(m_hMutex);
 
@@ -85,35 +94,36 @@ wxString CShellExtensionInterface::InitDrag()
 
 wxString CShellExtensionInterface::GetTarget()
 {
-	if (!m_shellExtension)
+	if (!m_shellExtension) {
 		return wxString();
+	}
 
-	if (!m_hMutex)
+	if (!m_hMutex) {
 		return wxString();
+	}
 
-	if (!m_hMapping)
+	if (!m_hMapping) {
 		return wxString();
+	}
 
 	char* data = (char*)MapViewOfFile(m_hMapping, FILE_MAP_READ | FILE_MAP_WRITE, 0, 0, DRAG_EXT_MAPPING_LENGTH);
-	if (!data)
-	{
+	if (!data) {
 		CloseHandle(m_hMapping);
 		m_hMapping = 0;
 		return wxString();
 	}
 
 	DWORD result = WaitForSingleObject(m_hMutex, 250);
-	if (result != WAIT_OBJECT_0)
-	{
+	if (result != WAIT_OBJECT_0) {
 		UnmapViewOfFile(data);
 		return wxString();
 	}
 
 	wxString target;
 	const char reply = data[1];
-	if (reply == 2)
-	{
+	if (reply == 2) {
 		data[DRAG_EXT_MAPPING_LENGTH - 1] = 0;
+		data[DRAG_EXT_MAPPING_LENGTH - 2] = 0;
 		target = (wchar_t*)(data + 2);
 	}
 
@@ -121,14 +131,17 @@ wxString CShellExtensionInterface::GetTarget()
 
 	UnmapViewOfFile(data);
 
-	if (target.empty())
+	if (target.empty()) {
 		return target;
+	}
 
-	if (target.Last() == '\\')
+	if (target.Last() == '\\') {
 		target.RemoveLast();
+	}
 	int pos = target.Find('\\', true);
-	if (pos < 1)
+	if (pos < 1) {
 		return wxString();
+	}
 	target = target.Left(pos + 1);
 
 	return target;
@@ -146,12 +159,13 @@ bool CShellExtensionInterface::CreateDragDirectory()
 
 		wxFileName dirname(wxStandardPaths::Get().GetTempDir(), DRAG_EXT_DUMMY_DIR_PREFIX + std::to_wstring(value));
 		dirname.Normalize();
-		wxString dir = dirname.GetFullPath();
+		std::wstring dir = dirname.GetFullPath().ToStdWstring();
 
-		if (dir.Len() > MAX_PATH)
+		if (dir.size() > DRAG_EXT_MAX_PATH) {
 			return false;
+		}
 
-		if (CreateDirectory(dir.wc_str(), 0)) {
+		if (CreateDirectory(dir.c_str(), 0)) {
 			m_dragDirectory = dir;
 			return true;
 		}
@@ -170,22 +184,18 @@ std::unique_ptr<CShellExtensionInterface> CShellExtensionInterface::CreateInitia
 	return ret;
 }
 
-//{7BB79969-2C7E-4107-996C-36DB90890AB2}
+#endif
 
-#endif //__WXMSW__
-
-CRemoteDataObject::CRemoteDataObject(const CServer& server, const CServerPath& path)
+CRemoteDataObject::CRemoteDataObject(ServerWithCredentials const& server, const CServerPath& path)
 	: wxDataObjectSimple(wxDataFormat(_T("FileZilla3RemoteDataObject")))
-	, m_server(server)
+	, server_(server)
 	, m_path(path)
-	, m_didSendData()
 	, m_processId(wxGetProcessId())
 {
 }
 
 CRemoteDataObject::CRemoteDataObject()
 	: wxDataObjectSimple(wxDataFormat(_T("FileZilla3RemoteDataObject")))
-	, m_didSendData()
 	, m_processId(wxGetProcessId())
 {
 }
@@ -225,7 +235,7 @@ void CRemoteDataObject::Finalize()
 	AddTextElement(element, "ProcessId", m_processId);
 
 	auto xServer = element.append_child("Server");
-	SetServer(xServer, m_server);
+	SetServer(xServer, server_);
 
 	AddTextElement(element, "Path", m_path.GetSafePath());
 
@@ -242,47 +252,57 @@ void CRemoteDataObject::Finalize()
 bool CRemoteDataObject::SetData(size_t len, const void* buf)
 {
 	char* data = (char*)buf;
-	if (!len || data[len - 1] != 0)
+	if (!len || data[len - 1] != 0) {
 		return false;
+	}
 
-	if (!m_xmlFile.ParseData(data))
+	if (!m_xmlFile.ParseData(data)) {
 		return false;
+	}
 
 	auto element = m_xmlFile.GetElement();
-	if (!element || !(element = element.child("RemoteDataObject")))
+	if (!element || !(element = element.child("RemoteDataObject"))) {
 		return false;
+	}
 
 	m_processId = GetTextElementInt(element, "ProcessId", -1);
-	if (m_processId == -1)
+	if (m_processId == -1) {
 		return false;
+	}
 
-	auto server = element.child("Server");
-	if (!server || !::GetServer(server, m_server))
+	auto serverElement = element.child("Server");
+	if (!serverElement || !::GetServer(serverElement, server_)) {
 		return false;
+	}
 
-	wxString path = GetTextElement(element, "Path");
-	if (path.empty() || !m_path.SetSafePath(path))
+	std::wstring path = GetTextElement(element, "Path");
+	if (path.empty() || !m_path.SetSafePath(path)) {
 		return false;
+	}
 
 	m_fileList.clear();
 	auto files = element.child("Files");
-	if (!files)
+	if (!files) {
 		return false;
+	}
 
 	for (auto file = files.child("File"); file; file = file.next_sibling("File")) {
 		t_fileInfo info;
 		info.name = GetTextElement(file, "Name");
-		if (info.name.empty())
+		if (info.name.empty()) {
 			return false;
+		}
 
 		const int dir = GetTextElementInt(file, "Dir", -1);
-		if (dir == -1)
+		if (dir == -1) {
 			return false;
+		}
 		info.dir = dir == 1;
 
 		info.size = GetTextElementInt(file, "Size", -2);
-		if (info.size <= -2)
+		if (info.size <= -2) {
 			return false;
+		}
 
 		info.link = GetTextElementBool(file, "Link", false);
 
