@@ -47,7 +47,7 @@ void CFileExistsDlg::DisplayFile(bool left, wxString name, int64_t size, fz::dat
 	}
 
 	wxString timeStr = _("Date/time unknown");
-	if (time.empty())
+	if (!time.empty())
 		timeStr = CTimeFormat::Format(time);
 
 	xrc_call(*this, left ? "ID_FILE1_NAME" : "ID_FILE2_NAME", &wxStaticText::SetLabel, name);
@@ -77,13 +77,14 @@ bool CFileExistsDlg::CreateControls()
 void CFileExistsDlg::LoadIcon(int id, const wxString &file)
 {
 	wxStaticBitmap *pStatBmp = static_cast<wxStaticBitmap *>(FindWindow(id));
-	if (!pStatBmp)
+	if (!pStatBmp) {
 		return;
+	}
 
 	wxSize size = CThemeProvider::GetIconSize(iconSizeNormal);
 	pStatBmp->SetInitialSize(size);
 	pStatBmp->InvalidateBestSize();
-	pStatBmp->SetBitmap(CThemeProvider::GetBitmap(_T("ART_FILE"), wxART_OTHER, size));
+	pStatBmp->SetBitmap(CThemeProvider::Get()->CreateBitmap(_T("ART_FILE"), wxART_OTHER, size));
 
 #ifdef __WXMSW__
 	SHFILEINFO fileinfo;
@@ -188,7 +189,7 @@ void CFileExistsDlg::OnOK(wxCommandEvent&)
 	EndModal(wxID_OK);
 }
 
-enum CFileExistsNotification::OverwriteAction CFileExistsDlg::GetAction() const
+CFileExistsNotification::OverwriteAction CFileExistsDlg::GetAction() const
 {
 	return m_action;
 }
