@@ -1,9 +1,9 @@
-#ifndef __FILEEXISTSDLG_H__
-#define __FILEEXISTSDLG_H__
+#ifndef FILEZILLA_INTERFACE_FILEEXISTSDLG_HEADER
+#define FILEZILLA_INTERFACE_FILEEXISTSDLG_HEADER
 
 #include "dialogex.h"
 
-class CFileExistsDlg: public wxDialogEx
+class CFileExistsDlg final : public wxDialogEx
 {
 	DECLARE_EVENT_TABLE()
 
@@ -14,7 +14,7 @@ public:
 	/// Creation
 	bool Create(wxWindow* parent);
 
-	enum CFileExistsNotification::OverwriteAction GetAction() const;
+	CFileExistsNotification::OverwriteAction GetAction() const;
 	bool Always(bool &directionOnly, bool &queueOnly) const;
 
 protected:
@@ -31,10 +31,10 @@ protected:
 	wxString GetPathEllipsis(wxString path, wxWindow *window);
 
 	CFileExistsNotification *m_pNotification;
-	enum CFileExistsNotification::OverwriteAction m_action;
+	CFileExistsNotification::OverwriteAction m_action;
 	bool m_always{};
 	bool m_directionOnly{};
 	bool m_queueOnly{};
 };
 
-#endif //__FILEEXISTSDLG_H__
+#endif
