@@ -8,6 +8,7 @@
 #include "conditionaldialog.h"
 #include <algorithm>
 #include "filelist_statusbar.h"
+#include "themeprovider.h"
 #if defined(__WXGTK__) && !defined(__WXGTK3__)
 #include <gtk/gtk.h>
 #endif
@@ -230,16 +231,13 @@ static gboolean gtk_button_release_event(GtkWidget*, void *gdk_event, CGtkEventC
 }
 #endif
 
-template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pParent, CState*, CQueueView* pQueue, bool border /*=false*/)
+template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pParent, CQueueView* pQueue, bool border /*=false*/)
 : wxListCtrlEx(pParent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxLC_VIRTUAL | wxLC_REPORT | wxLC_EDIT_LABELS | (border ? wxBORDER_SUNKEN : wxNO_BORDER)),
 	CComparableListing(this)
 {
-	CreateSystemImageList(16);
+	CreateSystemImageList(CThemeProvider::GetIconSize(iconSizeSmall).x);
 	m_pQueue = pQueue;
 
-	m_sortColumn = 0;
-	m_sortDirection = 0;
-
 	m_hasParent = true;
 
 	m_comparisonIndex = -1;
@@ -267,8 +265,8 @@ template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pPar
 	g_signal_connect(widget, "button_release_event", G_CALLBACK(gtk_button_release_event), m_gtkEventCallbackProxy.get());
 #endif
 
-	m_genericTypes[genericTypes::file] = _("File");
-	m_genericTypes[genericTypes::directory] = _("Directory");
+	m_genericTypes[genericTypes::file] = _("File").ToStdWstring();
+	m_genericTypes[genericTypes::directory] = _("Directory").ToStdWstring();
 
 	SetBackgroundStyle(wxBG_STYLE_SYSTEM);
 #ifndef __WXMSW__
@@ -285,26 +283,33 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 	CancelLabelEdit();
 
 	if (column != -1) {
-		if (column != m_sortColumn)
-		{
+		if (column != m_sortColumn) {
 			const int oldVisibleColumn = GetColumnVisibleIndex(m_sortColumn);
 			if (oldVisibleColumn != -1)
 				SetHeaderSortIconIndex(oldVisibleColumn, -1);
 		}
 	}
-	else
-		column = m_sortColumn;
+	else {
+		if (m_sortColumn != -1) {
+			column = m_sortColumn;
+		}
+		else {
+			column = 0;
+		}
+	}
 
 	if (direction == -1)
 		direction = m_sortDirection;
 
-	int newVisibleColumn = GetColumnVisibleIndex(column);
-	if (newVisibleColumn == -1) {
-		newVisibleColumn = 0;
-		column = 0;
-	}
 
-	SetHeaderSortIconIndex(newVisibleColumn, direction);
+	if (column != m_sortColumn || direction != m_sortDirection) {
+		int newVisibleColumn = GetColumnVisibleIndex(column);
+		if (newVisibleColumn == -1) {
+			newVisibleColumn = 0;
+			column = 0;
+		}
+		SetHeaderSortIconIndex(newVisibleColumn, direction);
+	}
 
 	// Remember which files are selected
 	bool *selected = 0;
@@ -321,7 +326,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 			if (item != -1) {
 				selected = new bool[m_fileData.size()];
 				memset(selected, 0, sizeof(bool) * m_fileData.size());
-			
+
 				do {
 					selected[m_indexMapping[item]] = 1;
 					item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
@@ -389,7 +394,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList_UpdateSelectio
 			}
 		}
 	}
-	
+
 	if (selections) {
 		for (unsigned int i = m_hasParent ? 1 : 0; i < m_indexMapping.size(); ++i) {
 			const int state = GetItemState(i, wxLIST_STATE_SELECTED);
@@ -406,7 +411,7 @@ template<class CFileData> CFileListCtrlSortBase::DirSortMode CFileListCtrl<CFile
 {
 	const int dirSortOption = COptions::Get()->GetOptionVal(OPTION_FILELIST_DIRSORT);
 
-	enum CFileListCtrlSortBase::DirSortMode dirSortMode;
+	CFileListCtrlSortBase::DirSortMode dirSortMode;
 	switch (dirSortOption)
 	{
 	case 0:
@@ -431,7 +436,7 @@ template<class CFileData> CFileListCtrlSortBase::NameSortMode CFileListCtrl<CFil
 {
 	const int nameSortOption = COptions::Get()->GetOptionVal(OPTION_FILELIST_NAMESORT);
 
-	enum CFileListCtrlSortBase::NameSortMode nameSortMode;
+	CFileListCtrlSortBase::NameSortMode nameSortMode;
 	switch (nameSortOption)
 	{
 	case 0:
@@ -494,104 +499,115 @@ wxString GetExt(const wxString& file)
 }
 #endif
 
-template<class CFileData> wxString CFileListCtrl<CFileData>::GetType(wxString name, bool dir, const wxString& path /*=_T("")*/)
+template<class CFileData> std::wstring CFileListCtrl<CFileData>::GetType(std::wstring name, bool dir, std::wstring const& path)
 {
 #ifdef __WXMSW__
-	wxString ext = GetExt(name);
-	ext.MakeLower();
-	std::map<wxString, wxString>::iterator typeIter = m_fileTypeMap.find(ext);
-	if (typeIter != m_fileTypeMap.end())
+	wxString ext;
+	if (dir) {
+		if (!path.empty()) {
+			ext = '/';
+		}
+	}
+	else {
+		ext = GetExt(name);
+		ext.MakeLower();
+	}
+	auto typeIter = m_fileTypeMap.find(ext);
+	if (typeIter != m_fileTypeMap.end()) {
 		return typeIter->second;
+	}
 
 	wxString type;
 	int flags = SHGFI_TYPENAME;
-	if (path.empty())
+	if (path.empty()) {
 		flags |= SHGFI_USEFILEATTRIBUTES;
-	else if (path == _T("\\"))
+	}
+	else if (path == _T("\\")) {
 		name += _T("\\");
-	else
+	}
+	else {
 		name = path + name;
+	}
 
 	SHFILEINFO shFinfo;
 	memset(&shFinfo, 0, sizeof(SHFILEINFO));
-	if (SHGetFileInfo(name.wc_str(),
+	if (SHGetFileInfo(name.c_str(),
 		dir ? FILE_ATTRIBUTE_DIRECTORY : FILE_ATTRIBUTE_NORMAL,
 		&shFinfo,
 		sizeof(shFinfo),
 		flags))
 	{
-		if (!*shFinfo.szTypeName)
-		{
-			if (!ext.empty())
-			{
+		if (!*shFinfo.szTypeName) {
+			if (!ext.empty()) {
 				type = ext;
 				type.MakeUpper();
 				type += _T("-");
 				type += _("file");
 			}
-			else
+			else {
 				type = m_genericTypes[genericTypes::file];
+			}
 		}
-		else
-		{
+		else {
 			type = shFinfo.szTypeName;
-			if (!dir && !ext.empty())
-				m_fileTypeMap[ext.MakeLower()] = type;
+			if (!ext.empty()) {
+				m_fileTypeMap[ext.MakeLower()] = type.ToStdWstring();
+			}
 		}
 	}
-	else
-	{
-		if (!ext.empty())
-		{
+	else {
+		if (!ext.empty()) {
 			type = ext;
 			type.MakeUpper();
 			type += _T("-");
 			type += _("file");
 		}
-		else
+		else {
 			type = m_genericTypes[genericTypes::file];
+		}
 	}
-	return type;
+	return type.ToStdWstring();
 #else
 	(void)path;
 
-	if (dir)
+	if (dir) {
 		return m_genericTypes[genericTypes::directory];
+	}
 
-	int pos = name.Find('.', true);
-	if (pos < 1 || !name[pos + 1]) // Starts or ends with dot
+	size_t pos = name.rfind('.');
+	if (pos == std::wstring::npos || pos < 1 || !name[pos + 1]) { // No dot or starts or ends with dot
 		return m_genericTypes[genericTypes::file];
-	wxString ext = name.Mid(pos + 1);
+	}
+	wxString ext = name.substr(pos + 1);
 	wxString lower_ext = ext.Lower();
 
-	std::map<wxString, wxString>::iterator typeIter = m_fileTypeMap.find(lower_ext);
-	if (typeIter != m_fileTypeMap.end())
+	auto typeIter = m_fileTypeMap.find(lower_ext);
+	if (typeIter != m_fileTypeMap.end()) {
 		return typeIter->second;
+	}
 
 	wxFileType *pType = wxTheMimeTypesManager->GetFileTypeFromExtension(ext);
-	if (!pType)
-	{
+	if (!pType) {
 		wxString desc = ext;
 		desc += _T("-");
 		desc += _("file");
-		m_fileTypeMap[ext] = desc;
-		return desc;
+		m_fileTypeMap[ext] = desc.ToStdWstring();
+		return desc.ToStdWstring();
 	}
 
 	wxString desc;
-	if (pType->GetDescription(&desc) && !desc.empty())
-	{
+	if (pType->GetDescription(&desc) && !desc.empty()) {
 		delete pType;
-		m_fileTypeMap[ext] = desc;
-		return desc;
+		m_fileTypeMap[ext] = desc.ToStdWstring();
+		return desc.ToStdWstring();
 	}
 	delete pType;
 
 	desc = ext;
 	desc += _T("-");
 	desc += _("file");
-	m_fileTypeMap[lower_ext] = desc;
-	return desc;
+	m_fileTypeMap[lower_ext] = desc.ToStdWstring();
+	return desc.ToStdWstring();
 #endif
 }
 
@@ -602,12 +618,14 @@ template<class CFileData> void CFileListCtrl<CFileData>::ScrollTopItem(int item)
 
 template<class CFileData> void CFileListCtrl<CFileData>::OnPostScroll()
 {
-	if (!IsComparing())
+	if (!IsComparing()) {
 		return;
+	}
 
 	CComparableListing* pOther = GetOther();
-	if (!pOther)
+	if (!pOther) {
 		return;
+	}
 
 	pOther->ScrollTopItem(GetTopItem());
 }
@@ -623,8 +641,9 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnExitComparisonMode()
 	m_indexMapping.clear();
 	m_indexMapping.swap(m_originalIndexMapping);
 
-	for (unsigned int i = 0; i < m_fileData.size() - 1; i++)
+	for (unsigned int i = 0; i < m_fileData.size() - 1; ++i) {
 		m_fileData[i].comparison_flags = normal;
+	}
 
 	SetItemCount(m_indexMapping.size());
 
@@ -650,17 +669,20 @@ template<class CFileData> void CFileListCtrl<CFileData>::ComparisonRememberSelec
 {
 	m_comparisonSelections.clear();
 
-	if (GetItemCount() != (int)m_indexMapping.size())
+	if (GetItemCount() != (int)m_indexMapping.size()) {
 		return;
+	}
 
 	int focus = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
 	if (focus != -1) {
 		SetItemState(focus, 0, wxLIST_STATE_FOCUSED);
 		int index = m_indexMapping[focus];
-		if (m_fileData[index].comparison_flags == fill)
+		if (m_fileData[index].comparison_flags == fill) {
 			focus = -1;
-		else
+		}
+		else {
 			focus = index;
+		}
 	}
 	m_comparisonSelections.push_back(focus);
 
@@ -732,7 +754,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnColumnRightClicked(wx
 template<class CFileData> void CFileListCtrl<CFileData>::InitSort(int optionID)
 {
 	wxString sortInfo = COptions::Get()->GetOption(optionID);
-	if( !sortInfo.empty() ) {
+	if (!sortInfo.empty()) {
 		m_sortDirection = sortInfo[0] - '0';
 	}
 	else {
@@ -741,88 +763,111 @@ template<class CFileData> void CFileListCtrl<CFileData>::InitSort(int optionID)
 	if (m_sortDirection < 0 || m_sortDirection > 1)
 		m_sortDirection = 0;
 
-	if (sortInfo.Len() == 3)
-	{
+	if (sortInfo.Len() == 3) {
 		m_sortColumn = sortInfo[2] - '0';
-		if (GetColumnVisibleIndex(m_sortColumn) == -1)
+		if (GetColumnVisibleIndex(m_sortColumn) == -1) {
 			m_sortColumn = 0;
+		}
 	}
 	else
 		m_sortColumn = 0;
+
+	SetHeaderSortIconIndex(GetColumnVisibleIndex(m_sortColumn), m_sortDirection);
 }
 
 template<class CFileData> void CFileListCtrl<CFileData>::OnItemSelected(wxListEvent& event)
 {
 #ifndef __WXMSW__
 	// On MSW this is done in the subclassed window proc
-	if (m_insideSetSelection)
+	if (m_insideSetSelection) {
 		return;
-	if (m_pending_focus_processing)
+	}
+	if (m_pending_focus_processing) {
 		return;
+	}
 #endif
 
 	const int item = event.GetIndex();
+	if (item < 0 || item >= (int)m_indexMapping.size()) {
+		return;
+	}
 
 #ifndef __WXMSW__
-	if (m_selections[item])
+	if (m_selections[item]) {
+		// Need to re-do all selections
+		// Unfortunately the focus change event comes before the selection change event
+		// in the generic list ctrl if multiple items are already selected
+		// and selecting one of those a second time, causing all others to be deselected
+		UpdateSelections(0, GetItemCount() - 1);
 		return;
+	}
 	m_selections[item] = true;
 #endif
 
-	if (!m_pFilelistStatusBar)
+	if (!m_pFilelistStatusBar) {
 		return;
+	}
 
-	if (item < 0 || item >= (int)m_indexMapping.size())
-		return;
 
-	if (m_hasParent && !item)
+	if (m_hasParent && !item) {
 		return;
+	}
 
 	const int index = m_indexMapping[item];
 	const CFileData& data = m_fileData[index];
-	if (data.comparison_flags == fill)
+	if (data.comparison_flags == fill) {
 		return;
+	}
 
-	if (ItemIsDir(index))
+	if (ItemIsDir(index)) {
 		m_pFilelistStatusBar->SelectDirectory();
-	else
+	}
+	else {
 		m_pFilelistStatusBar->SelectFile(ItemGetSize(index));
+	}
 }
 
 template<class CFileData> void CFileListCtrl<CFileData>::OnItemDeselected(wxListEvent& event)
 {
 #ifndef __WXMSW__
 	// On MSW this is done in the subclassed window proc
-	if (m_insideSetSelection)
+	if (m_insideSetSelection) {
 		return;
+	}
 #endif
 
 	const int item = event.GetIndex();
+	if (item < 0 || item >= (int)m_indexMapping.size()) {
+		return;
+	}
 
 #ifndef __WXMSW__
-	if (!m_selections[item])
+	if (!m_selections[item]) {
 		return;
+	}
 	m_selections[item] = false;
 #endif
 
-	if (!m_pFilelistStatusBar)
+	if (!m_pFilelistStatusBar) {
 		return;
+	}
 
-	if (item < 0 || item >= (int)m_indexMapping.size())
-		return;
-
-	if (m_hasParent && !item)
+	if (m_hasParent && !item) {
 		return;
+	}
 
 	const int index = m_indexMapping[item];
 	const CFileData& data = m_fileData[index];
-	if (data.comparison_flags == fill)
+	if (data.comparison_flags == fill) {
 		return;
+	}
 
-	if (ItemIsDir(index))
+	if (ItemIsDir(index)) {
 		m_pFilelistStatusBar->UnselectDirectory();
-	else
+	}
+	else {
 		m_pFilelistStatusBar->UnselectFile(ItemGetSize(index));
+	}
 }
 
 template<class CFileData> void CFileListCtrl<CFileData>::SetSelection(int item, bool select)
@@ -854,8 +899,9 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnFocusChanged(wxListEv
 template<class CFileData> void CFileListCtrl<CFileData>::SetItemCount(int count)
 {
 	m_selections.resize(count, false);
-	if (m_focusItem >= count)
+	if (m_focusItem >= count) {
 		m_focusItem = -1;
+	}
 	wxListCtrlEx::SetItemCount(count);
 }
 
@@ -865,74 +911,86 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnProcessFocusChange(wx
 	int old_focus = event.GetInt();
 	int new_focus = (int)event.GetExtraLong();
 
-	if (old_focus >= GetItemCount())
+	if (old_focus >= GetItemCount()) {
 		return;
+	}
 
-	if (old_focus != -1)
-	{
+	if (old_focus != -1) {
 		bool selected = GetItemState(old_focus, wxLIST_STATE_SELECTED) == wxLIST_STATE_SELECTED;
-		if (!selected && m_selections[old_focus])
-		{
+		if (!selected && m_selections[old_focus]) {
 			// Need to deselect all
-			if (m_pFilelistStatusBar)
+			if (m_pFilelistStatusBar) {
 				m_pFilelistStatusBar->UnselectAll();
-			for (unsigned int i = 0; i < m_selections.size(); i++)
+			}
+			for (unsigned int i = 0; i < m_selections.size(); ++i) {
 				m_selections[i] = 0;
+			}
 		}
 	}
 
 	int min;
 	int max;
-	if (new_focus > old_focus)
-	{
+	if (new_focus > old_focus) {
 		min = old_focus;
 		max = new_focus;
 	}
-	else
-	{
+	else {
 		min = new_focus;
 		max = old_focus;
 	}
-	if (min == -1)
+	if (min == -1) {
 		min++;
-	if (max == -1)
+	}
+	if (max == -1) {
 		return;
+	}
 
-	if (max >= GetItemCount())
+	if (max >= GetItemCount()) {
 		return;
+	}
 
-	for (int i = min; i <= max; i++)
-	{
+	UpdateSelections(min, max);
+}
+
+template <class CFileData> void CFileListCtrl<CFileData>::UpdateSelections(int min, int max)
+{
+	for (int i = min; i <= max; ++i) {
 		bool selected = GetItemState(i, wxLIST_STATE_SELECTED) == wxLIST_STATE_SELECTED;
-		if (selected == m_selections[i])
+		if (selected == m_selections[i]) {
 			continue;
+		}
 
 		m_selections[i] = selected;
 
-		if (!m_pFilelistStatusBar)
+		if (!m_pFilelistStatusBar) {
 			continue;
+		}
 
-		if (m_hasParent && !i)
+		if (m_hasParent && !i) {
 			continue;
+		}
 
 		const int index = m_indexMapping[i];
 		const CFileData& data = m_fileData[index];
-		if (data.comparison_flags == fill)
+		if (data.comparison_flags == fill) {
 			continue;
-
-		if (selected)
-		{
-			if (ItemIsDir(index))
-				m_pFilelistStatusBar->SelectDirectory();
-			else
-				m_pFilelistStatusBar->SelectFile(ItemGetSize(index));
 		}
-		else
-		{
-			if (ItemIsDir(index))
+
+		if (selected) {
+			if (ItemIsDir(index)) {
+				m_pFilelistStatusBar->SelectDirectory();
+			}
+			else {
+				m_pFilelistStatusBar->SelectFile(ItemGetSize(index));
+			}
+		}
+		else {
+			if (ItemIsDir(index)) {
 				m_pFilelistStatusBar->UnselectDirectory();
-			else
+			}
+			else {
 				m_pFilelistStatusBar->UnselectFile(ItemGetSize(index));
+			}
 		}
 	}
 }
@@ -949,22 +1007,26 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnLeftDown(wxMouseEvent
 
 template<class CFileData> void CFileListCtrl<CFileData>::OnProcessMouseEvent(wxCommandEvent&)
 {
-	if (m_pending_focus_processing)
+	if (m_pending_focus_processing) {
 		return;
+	}
 
-	if (m_focusItem >= GetItemCount())
+	if (m_focusItem >= GetItemCount()) {
 		return;
-	if (m_focusItem == -1)
+	}
+	if (m_focusItem == -1) {
 		return;
+	}
 
 	bool selected = GetItemState(m_focusItem, wxLIST_STATE_SELECTED) == wxLIST_STATE_SELECTED;
-	if (!selected && m_selections[m_focusItem])
-	{
+	if (!selected && m_selections[m_focusItem]) {
 		// Need to deselect all
-		if (m_pFilelistStatusBar)
+		if (m_pFilelistStatusBar) {
 			m_pFilelistStatusBar->UnselectAll();
-		for (unsigned int i = 0; i < m_selections.size(); i++)
+		}
+		for (unsigned int i = 0; i < m_selections.size(); ++i) {
 			m_selections[i] = 0;
+		}
 	}
 }
 #endif
@@ -999,20 +1061,22 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnKeyDown(wxKeyEvent& e
 
 	const int code = event.GetKeyCode();
 	const int mods = event.GetModifiers();
-	if (code == 'A' && (mods == wxMOD_CMD || mods == (wxMOD_CONTROL | wxMOD_META)))
-	{
-		for (unsigned int i = m_hasParent ? 1 : 0; i < m_indexMapping.size(); i++)
-		{
+	if (code == 'A' && (mods == wxMOD_CMD || mods == (wxMOD_CONTROL | wxMOD_META))) {
+		for (unsigned int i = m_hasParent ? 1 : 0; i < m_indexMapping.size(); ++i) {
 			const CFileData& data = m_fileData[m_indexMapping[i]];
-			if (data.comparison_flags != fill)
+			if (data.comparison_flags != fill) {
 				SetSelection(i, true);
-			else
+			}
+			else {
 				SetSelection(i, false);
+			}
 		}
-		if (m_hasParent)
+		if (m_hasParent && GetItemCount()) {
 			SetSelection(0, false);
-		if (m_pFilelistStatusBar)
+		}
+		if (m_pFilelistStatusBar) {
 			m_pFilelistStatusBar->SelectAll();
+		}
 	}
 	else if (code == WXK_BACK ||
 			(code == WXK_UP && event.GetModifiers() == CursorModifierKey) ||
@@ -1021,6 +1085,32 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnKeyDown(wxKeyEvent& e
 	{
 		OnNavigationEvent(false);
 	}
-	else
+	else {
 		event.Skip();
+	}
+}
+
+template<class CFileData> void CFileListCtrl<CFileData>::UpdateSelections_ItemsAdded(std::vector<int> const& added_indexes)
+{
+	if (added_indexes.empty()) {
+		return;
+	}
+
+	auto added_index = added_indexes.cbegin();
+	std::deque<bool> selected;
+	// This is O(n) in number of items. I think it's possible to make it O(n) in number of selections
+	for (unsigned int i = *added_index; i < m_indexMapping.size(); ++i) {
+		if (added_index != added_indexes.end() && i == static_cast<unsigned int>(*added_index)) {
+			selected.push_front(false);
+			++added_index;
+		}
+		bool is_selected = GetItemState(i, wxLIST_STATE_SELECTED) != 0;
+		selected.push_back(is_selected);
+
+		bool should_selected = selected.front();
+		selected.pop_front();
+		if (is_selected != should_selected) {
+			SetSelection(i, should_selected);
+		}
+	}
 }
