@@ -1,14 +1,10 @@
-#ifndef __FILEZILLAAPP_H__
-#define __FILEZILLAAPP_H__
-
-#if wxUSE_DEBUGREPORT && wxUSE_ON_FATAL_EXCEPTION
-#include <wx/debugrpt.h>
-#endif
-
-#include <list>
+#ifndef FILEZILLA_INTERFACE_FILEZILLAAPP_HEADER
+#define FILEZILLA_INTERFACE_FILEZILLAAPP_HEADER
 
 #include "local_path.h"
 
+#include <vector>
+
 class CWrapEngine;
 class CCommandLine;
 class CFileZillaApp : public wxApp
@@ -25,9 +21,12 @@ public:
 	CLocalPath GetDefaultsDir() const { return m_defaultsDir; }
 	CLocalPath GetLocalesDir() const { return m_localesDir; }
 
-	wxString GetSettingsFile(wxString const& name) const;
+	std::wstring GetSettingsFile(std::wstring const& name) const;
 
 	void CheckExistsFzsftp();
+#if ENABLE_STORJ
+	void CheckExistsFzstorj();
+#endif
 
 	void InitLocale();
 	bool SetLocale(int language);
@@ -38,27 +37,20 @@ public:
 
 	CWrapEngine* GetWrapEngine();
 
-	const CCommandLine* GetCommandLine() const { return m_pCommandLine; }
-
-#if 0 // Disabled for now due to some wx controls expecting wxYield to process all events.
-#if wxMAJOR_VERSION == 2 && wxMINOR_VERSION == 8
-	#define USE_CHUNKED_PROCESS_PENDING_EVENTS 1
-#endif
-#endif
-#if USE_CHUNKED_PROCESS_PENDING_EVENTS
-	virtual void ProcessPendingEvents();
-#endif
+	const CCommandLine* GetCommandLine() const { return m_pCommandLine.get(); }
 
 	void ShowStartupProfile();
-	void AddStartupProfileRecord(const wxString& msg);
+	void AddStartupProfileRecord(std::string const& msg);
 
 protected:
+	void CheckExistsTool(std::wstring const& tool, std::vector<std::wstring> const& searchPaths, std::wstring const& env, int setting, wxString const& description);
+
 	bool InitDefaultsDir();
 	bool LoadResourceFiles();
 	bool LoadLocales();
 	int ProcessCommandLine();
 
-	wxLocale* m_pLocale;
+	std::unique_ptr<wxLocale> m_pLocale;
 
 	CLocalPath m_resourceDir;
 	CLocalPath m_defaultsDir;
@@ -68,22 +60,18 @@ protected:
 	virtual void OnFatalException();
 #endif
 
-	CLocalPath GetDataDir(wxString fileToFind) const;
+	CLocalPath GetDataDir(std::wstring fileToFind) const;
 
-	// FileExists accepts full paths as parameter,
-	// with the addition that path segments may be obmitted
-	// with a wildcard (*). A matching directory will then be searched.
-	// Example: FileExists(_T("/home/*/.filezilla/filezilla.xml"));
-	bool FileExists(const wxString& file) const;
+	bool FileExists(std::wstring const& file) const;
 
-	CWrapEngine* m_pWrapEngine;
+	std::unique_ptr<CWrapEngine> m_pWrapEngine;
 
-	CCommandLine* m_pCommandLine;
+	std::unique_ptr<CCommandLine> m_pCommandLine;
 
-	bool m_profilingActive;
-	std::list<std::pair<wxDateTime, wxString> > m_startupProfile;
+	fz::monotonic_clock m_profile_start;
+	std::vector<std::pair<fz::monotonic_clock, std::string>> m_startupProfile;
 };
 
 DECLARE_APP(CFileZillaApp)
 
-#endif //__FILEZILLAAPP_H__
+#endif
