@@ -5,6 +5,7 @@
 #include "ipcmutex.h"
 #include "Options.h"
 #include "queue.h"
+#include "xrc_helper.h"
 
 CImportDialog::CImportDialog(wxWindow* parent, CQueueView* pQueueView)
 	: m_parent(parent), m_pQueueView(pQueueView)
@@ -29,24 +30,31 @@ void CImportDialog::Run()
 		return;
 	}
 
-	CXmlFile fz3(dlg.GetPath());
+	CXmlFile fz3(dlg.GetPath().ToStdWstring());
 	auto fz3Root = fz3.Load();
 	if (fz3Root) {
 		bool settings = fz3Root.child("Settings") != 0;
 		bool queue = fz3Root.child("Queue") != 0;
 		bool sites = fz3Root.child("Servers") != 0;
+		bool filters = fz3Root.child("Filters") != 0;
 
-		if (settings || queue || sites) {
+		if (settings || queue || sites || filters) {
 			if (!Load(m_parent, _T("ID_IMPORT"))) {
 				wxBell();
 				return;
 			}
-			if (!queue)
-				XRCCTRL(*this, "ID_QUEUE", wxCheckBox)->Hide();
-			if (!sites)
-				XRCCTRL(*this, "ID_SITEMANAGER", wxCheckBox)->Hide();
-			if (!settings)
-				XRCCTRL(*this, "ID_SETTINGS", wxCheckBox)->Hide();
+			if (!queue) {
+				xrc_call(*this, "ID_QUEUE", &wxCheckBox::Hide);
+			}
+			if (!sites) {
+				xrc_call(*this, "ID_SITEMANAGER", &wxCheckBox::Hide);
+			}
+			if (!settings) {
+				xrc_call(*this, "ID_SETTINGS", &wxCheckBox::Hide);
+			}
+			if (!filters) {
+				xrc_call(*this, "ID_FILTERS", &wxCheckBox::Hide);
+			}
 			GetSizer()->Fit(this);
 
 			if (ShowModal() != wxID_OK) {
@@ -60,25 +68,29 @@ void CImportDialog::Run()
 				}
 			}
 
-			if (queue && XRCCTRL(*this, "ID_QUEUE", wxCheckBox)->IsChecked()) {
+			if (queue && xrc_call(*this, "ID_QUEUE", &wxCheckBox::IsChecked)) {
 				m_pQueueView->ImportQueue(fz3Root.child("Queue"), true);
 			}
 
-			if (sites && XRCCTRL(*this, "ID_SITEMANAGER", wxCheckBox)->IsChecked()) {
+			if (sites && xrc_call(*this, "ID_SITEMANAGER", &wxCheckBox::IsChecked)) {
 				ImportSites(fz3Root.child("Servers"));
 			}
 
-			if (settings && XRCCTRL(*this, "ID_SETTINGS", wxCheckBox)->IsChecked()) {
+			if (settings && xrc_call(*this, "ID_SETTINGS", &wxCheckBox::IsChecked)) {
 				COptions::Get()->Import(fz3Root.child("Settings"));
 				wxMessageBoxEx(_("The settings have been imported. You have to restart FileZilla for all settings to have effect."), _("Import successful"), wxOK, this);
 			}
 
+			if (filters && xrc_call(*this, "ID_FILTERS", &wxCheckBox::IsChecked)) {
+				CFilterManager::Import(fz3Root);
+			}
+
 			wxMessageBoxEx(_("The selected categories have been imported."), _("Import successful"), wxOK, this);
 			return;
 		}
 	}
 
-	CXmlFile fz2(dlg.GetPath(), _T("FileZilla"));
+	CXmlFile fz2(dlg.GetPath().ToStdWstring(), "FileZilla");
 	auto fz2Root = fz2.Load();
 	if (fz2Root) {
 		auto sites_fz2 = fz2Root.child("Sites");
@@ -118,25 +130,25 @@ bool CImportDialog::ImportLegacySites(pugi::xml_node sites)
 	return file.Save(true);
 }
 
-wxString CImportDialog::DecodeLegacyPassword(wxString pass)
+std::wstring CImportDialog::DecodeLegacyPassword(std::wstring const& pass)
 {
-	if( pass.size() % 3 ) {
-		return wxString();
+	if (pass.size() % 3) {
+		return std::wstring();
 	}
 
-	wxString output;
+	std::wstring output;
 	const char* key = "FILEZILLA1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZ";
 
-	int pos = (pass.Length() / 3) % strlen(key);
-	for (unsigned int i = 0; i < pass.Length(); i += 3) {
+	int pos = (pass.size() / 3) % strlen(key);
+	for (size_t i = 0; i < pass.size(); i += 3) {
 		if (pass[i] < '0' || pass[i] > '9' ||
 			pass[i + 1] < '0' || pass[i + 1] > '9' ||
 			pass[i + 2] < '0' || pass[i + 2] > '9')
-			return wxString();
+			return std::wstring();
 		int number = (pass[i] - '0') * 100 +
 						(pass[i + 1] - '0') * 10 +
 						(pass[i + 2] - '0');
-		wxChar c = number ^ key[(i / 3 + pos) % strlen(key)];
+		wchar_t c = number ^ key[(i / 3 + pos) % strlen(key)];
 		output += c;
 	}
 
@@ -146,36 +158,41 @@ wxString CImportDialog::DecodeLegacyPassword(wxString pass)
 bool CImportDialog::ImportLegacySites(pugi::xml_node sitesToImport, pugi::xml_node existingSites)
 {
 	for (auto importFolder = sitesToImport.child("Folder"); importFolder; importFolder = importFolder.next_sibling("Folder")) {
-		wxString name = GetTextAttribute(importFolder, "Name");
-		if (name.empty())
+		std::wstring name = GetTextAttribute(importFolder, "Name");
+		if (name.empty()) {
 			continue;
+		}
 
-		wxString newName = name;
+		std::wstring newName = name;
 		int i = 2;
 		pugi::xml_node folder;
 		while (!(folder = GetFolderWithName(existingSites, newName))) {
-			newName = wxString::Format(_T("%s %d"), name, i++);
+			newName = fz::sprintf(L"%s %d", name, i++);
 		}
 
 		ImportLegacySites(importFolder, folder);
 	}
 
 	for (auto importSite = sitesToImport.child("Site"); importSite; importSite = importSite.next_sibling("Site")) {
-		wxString name = GetTextAttribute(importSite, "Name");
-		if (name.empty())
+		std::wstring name = GetTextAttribute(importSite, "Name");
+		if (name.empty()) {
 			continue;
+		}
 
-		wxString host = GetTextAttribute(importSite, "Host");
-		if (host.empty())
+		std::wstring host = GetTextAttribute(importSite, "Host");
+		if (host.empty()) {
 			continue;
+		}
 
 		int port = GetAttributeInt(importSite, "Port");
-		if (port < 1 || port > 65535)
+		if (port < 1 || port > 65535) {
 			continue;
+		}
 
 		int serverType = GetAttributeInt(importSite, "ServerType");
-		if (serverType < 0 || serverType > 4)
+		if (serverType < 0 || serverType > 4) {
 			continue;
+		}
 
 		int protocol;
 		switch (serverType)
@@ -199,24 +216,28 @@ bool CImportDialog::ImportLegacySites(pugi::xml_node sitesToImport, pugi::xml_no
 		bool dontSavePass = GetAttributeInt(importSite, "DontSavePass") == 1;
 
 		int logontype = GetAttributeInt(importSite, "Logontype");
-		if (logontype < 0 || logontype > 2)
+		if (logontype < 0 || logontype > 2) {
 			continue;
-		if (logontype == 2)
+		}
+		if (logontype == 2) {
 			logontype = 4;
-		if (logontype == 1 && dontSavePass)
+		}
+		if (logontype == 1 && dontSavePass) {
 			logontype = 2;
+		}
 
-		wxString user = GetTextAttribute(importSite, "User");
-		wxString pass = DecodeLegacyPassword(GetTextAttribute(importSite, "Pass"));
-		wxString account = GetTextAttribute(importSite, "Account");
-		if (logontype && user.empty())
+		std::wstring user = GetTextAttribute(importSite, "User");
+		std::wstring pass = DecodeLegacyPassword(GetTextAttribute(importSite, "Pass"));
+		std::wstring account = GetTextAttribute(importSite, "Account");
+		if (logontype && user.empty()) {
 			continue;
+		}
 
 		// Find free name
-		wxString newName = name;
+		std::wstring newName = name;
 		int i = 2;
 		while (HasEntryWithName(existingSites, newName)) {
-			newName = wxString::Format(_T("%s %d"), name, i++);
+			newName = fz::sprintf(L"%s %d", name, i++);
 		}
 
 		auto server = existingSites.append_child("Server");
@@ -234,46 +255,41 @@ bool CImportDialog::ImportLegacySites(pugi::xml_node sitesToImport, pugi::xml_no
 	return true;
 }
 
-bool CImportDialog::HasEntryWithName(pugi::xml_node element, const wxString& name)
+bool CImportDialog::HasEntryWithName(pugi::xml_node element, std::wstring const& name)
 {
 	pugi::xml_node child;
 	for (child = element.child("Server"); child; child = child.next_sibling("Server")) {
-		wxString childName = GetTextElement(child);
-		childName.Trim(true);
-		childName.Trim(false);
-		if (!name.CmpNoCase(childName))
+		std::wstring childName = GetTextElement_Trimmed(child);
+		if (!fz::stricmp(name, childName)) {
 			return true;
+		}
 	}
 	for (child = element.child("Folder"); child; child = child.next_sibling("Folder")) {
-		wxString childName = GetTextElement(child);
-		childName.Trim(true);
-		childName.Trim(false);
-		if (!name.CmpNoCase(childName))
+		std::wstring childName = GetTextElement_Trimmed(child);
+		if (!fz::stricmp(name, childName)) {
 			return true;
+		}
 	}
 
 	return false;
 }
 
-pugi::xml_node CImportDialog::GetFolderWithName(pugi::xml_node element, const wxString& name)
+pugi::xml_node CImportDialog::GetFolderWithName(pugi::xml_node element, std::wstring const& name)
 {
 	pugi::xml_node child;
 	for (child = element.child("Server"); child; child = child.next_sibling("Server")) {
-		wxString childName = GetTextElement(child);
-		childName.Trim(true);
-		childName.Trim(false);
-		if (!name.CmpNoCase(childName)) {
+		std::wstring childName = GetTextElement_Trimmed(child);
+		if (!fz::stricmp(name, childName)) {
 			// We do not allow servers and directories to share the same name
 			return pugi::xml_node();
 		}
 	}
 
 	for (child = element.child("Folder"); child; child = child.next_sibling("Folder")) {
-		wxString childName = GetTextElement(child);
-		childName.Trim(true);
-		childName.Trim(false);
-		if (!name.CmpNoCase(childName))
+		std::wstring childName = GetTextElement_Trimmed(child);
+		if (!fz::stricmp(name, childName)) {
 			return child;
+		}
 	}
 
 	child = element.append_child("Folder");
@@ -296,11 +312,13 @@ bool CImportDialog::ImportSites(pugi::xml_node sites)
 	}
 
 	auto currentSites = element.child("Servers");
-	if (!currentSites)
+	if (!currentSites) {
 		currentSites = element.append_child("Servers");
+	}
 
-	if (!ImportSites(sites, currentSites))
+	if (!ImportSites(sites, currentSites)) {
 		return false;
+	}
 
 	return file.Save(true);
 }
@@ -308,34 +326,38 @@ bool CImportDialog::ImportSites(pugi::xml_node sites)
 bool CImportDialog::ImportSites(pugi::xml_node sitesToImport, pugi::xml_node existingSites)
 {
 	for (auto importFolder = sitesToImport.child("Folder"); importFolder; importFolder = importFolder.next_sibling("Folder")) {
-		wxString name = GetTextElement_Trimmed(importFolder, "Name");
-		if (name.empty())
+		std::wstring name = GetTextElement_Trimmed(importFolder, "Name");
+		if (name.empty()) {
 			name = GetTextElement_Trimmed(importFolder);
-		if (name.empty())
+		}
+		if (name.empty()) {
 			continue;
+		}
 
-		wxString newName = name;
+		std::wstring newName = name;
 		int i = 2;
 		pugi::xml_node folder;
 		while (!(folder = GetFolderWithName(existingSites, newName))) {
-			newName = wxString::Format(_T("%s %d"), name, i++);
+			newName = fz::sprintf(L"%s %d", name, i++);
 		}
 
 		ImportSites(importFolder, folder);
 	}
 
 	for (auto importSite = sitesToImport.child("Server"); importSite; importSite = importSite.next_sibling("Server")) {
-		wxString name = GetTextElement_Trimmed(importSite, "Name");
-		if (name.empty())
+		std::wstring name = GetTextElement_Trimmed(importSite, "Name");
+		if (name.empty()) {
 			name = GetTextElement_Trimmed(importSite);
-		if (name.empty())
+		}
+		if (name.empty()) {
 			continue;
+		}
 
 		// Find free name
-		wxString newName = name;
+		std::wstring newName = name;
 		int i = 2;
 		while (HasEntryWithName(existingSites, newName)) {
-			newName = wxString::Format(_T("%s %d"), name, i++);
+			newName = fz::sprintf(L"%s %d", name, i++);
 		}
 
 		auto server = existingSites.append_copy(importSite);
