@@ -1,9 +1,9 @@
-#ifndef __INPUTDIALOG_H__
-#define __INPUTDIALOG_H__
+#ifndef FILEZILLA_INTERFACE_INPUTDIALOG_HEADER
+#define FILEZILLA_INTERFACE_INPUTDIALOG_HEADER
 
 #include "dialogex.h"
 
-class CInputDialog : public wxDialogEx
+class CInputDialog final : public wxDialogEx
 {
 public:
 	CInputDialog();
@@ -29,4 +29,4 @@ protected:
 	wxTextCtrl* m_pTextCtrl;
 };
 
-#endif //__INPUTDIALOG_H__
+#endif
