@@ -500,27 +500,27 @@ void wxListCtrlEx::SaveColumnSettings(int widthsOptionId, int visibilityOptionId
 	}
 
 	if (visibilityOptionId != -1) {
-		wxString visibleColumns;
+		std::wstring visibleColumns;
 		for (unsigned int i = 0; i < m_columnInfo.size(); ++i) {
 			if (m_columnInfo[i].shown) {
-				visibleColumns += _T("1");
+				visibleColumns += L"1";
 			}
 			else {
-				visibleColumns += _T("0");
+				visibleColumns += L"0";
 			}
 		}
-		COptions::Get()->SetOption(visibilityOptionId, visibleColumns.ToStdWstring());
+		COptions::Get()->SetOption(visibilityOptionId, visibleColumns);
 	}
 
 	if (sortOptionId != -1) {
-		wxString order;
+		std::wstring order;
 		for (unsigned int i = 0; i < m_columnInfo.size(); ++i) {
 			if (i) {
-				order += _T(",");
+				order += L",";
 			}
-			order += wxString::Format(_T("%d"), m_columnInfo[i].order);
+			order += fz::to_wstring(m_columnInfo[i].order);
 		}
-		COptions::Get()->SetOption(sortOptionId, order.ToStdWstring());
+		COptions::Get()->SetOption(sortOptionId, order);
 	}
 }
 
@@ -766,40 +766,38 @@ void wxListCtrlEx::ShowColumnEditor()
 	wxCheckListBox* pListBox = XRCCTRL(dlg, "ID_ACTIVE", wxCheckListBox);
 
 	dlg.m_order = new int[m_columnInfo.size()];
-	for (unsigned int j = 0; j < m_columnInfo.size(); j++)
-	{
-		for (unsigned int i = 0; i < m_columnInfo.size(); i++)
-		{
-			if (m_columnInfo[i].order != j)
+	for (unsigned int j = 0; j < m_columnInfo.size(); ++j) {
+		for (unsigned int i = 0; i < m_columnInfo.size(); ++i) {
+			if (m_columnInfo[i].order != j) {
 				continue;
+			}
 			dlg.m_order[j] = i;
 			pListBox->Append(m_columnInfo[i].name);
-			if (m_columnInfo[i].shown)
+			if (m_columnInfo[i].shown) {
 				pListBox->Check(j);
+			}
 		}
 	}
 	wxASSERT(pListBox->GetCount() == m_columnInfo.size());
 
 	dlg.GetSizer()->Fit(&dlg);
 
-	if (dlg.ShowModal() != wxID_OK)
-	{
+	if (dlg.ShowModal() != wxID_OK) {
 		delete [] dlg.m_order;
 		return;
 	}
 
-	for (unsigned int i = 0; i < m_columnInfo.size(); i++)
-	{
+	for (unsigned int i = 0; i < m_columnInfo.size(); ++i) {
 		int col = dlg.m_order[i];
 		bool isChecked = pListBox->IsChecked(i);
-		if (!isChecked && !col)
-		{
+		if (!isChecked && !col) {
 			isChecked = true;
 			wxMessageBoxEx(_("The filename column cannot be hidden."));
 		}
 		MoveColumn(col, i);
-		if (m_columnInfo[col].shown != isChecked)
+		if (m_columnInfo[col].shown != isChecked) {
 			ShowColumn(col, isChecked);
+		}
 	}
 
 	delete [] dlg.m_order;
@@ -810,12 +808,14 @@ void wxListCtrlEx::ShowColumnEditor()
 
 int wxListCtrlEx::GetColumnVisibleIndex(int col)
 {
-	if (!m_pVisibleColumnMapping)
+	if (!m_pVisibleColumnMapping) {
 		return -1;
+	}
 
 	for (int i = 0; i < GetColumnCount(); ++i) {
-		if (m_pVisibleColumnMapping[i] == (unsigned int)col)
+		if (m_pVisibleColumnMapping[i] == (unsigned int)col) {
 			return i;
+		}
 	}
 
 	return -1;
@@ -823,8 +823,9 @@ int wxListCtrlEx::GetColumnVisibleIndex(int col)
 
 int wxListCtrlEx::GetHeaderSortIconIndex(int col)
 {
-	if (col < 0 || col >= GetColumnCount())
+	if (col < 0 || col >= GetColumnCount()) {
 		return -1;
+	}
 
 #ifdef __WXMSW__
 	HWND hWnd = (HWND)GetHandle();
@@ -834,14 +835,16 @@ int wxListCtrlEx::GetHeaderSortIconIndex(int col)
 	item.mask = HDI_IMAGE | HDI_FORMAT;
 	SendMessage(header, HDM_GETITEM, col, (LPARAM)&item);
 
-	if (!(item.fmt & HDF_IMAGE))
+	if (!(item.fmt & HDF_IMAGE)) {
 		return -1;
+	}
 
 	return item.iImage;
 #else
 	wxListItem item;
-	if (!GetColumn(col, item))
+	if (!GetColumn(col, item)) {
 		return -1;
+	}
 
 	return item.GetImage();
 #endif
@@ -853,24 +856,30 @@ void wxListCtrlEx::InitHeaderSortImageList()
 	wxColour colour = wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE);
 
 	wxString lightness;
-	if (colour.Red() + colour.Green() + colour.Blue() > 3 * 128)
+	if (colour.Red() + colour.Green() + colour.Blue() > 3 * 128) {
 		lightness = _T("DARK");
-	else
+	}
+	else {
 		lightness = _T("LIGHT");
+	}
 
-	wxBitmap bmp;
+	auto * imageList = GetSystemImageList();
+	if (imageList) {
+		wxBitmap bmp;
 
-	bmp = wxArtProvider::GetBitmap(_T("ART_SORT_UP_") + lightness, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
-	m_header_icon_index.up = GetSystemImageList()->Add(bmp);
-	bmp = wxArtProvider::GetBitmap(_T("ART_SORT_DOWN_") + lightness, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
-	m_header_icon_index.down = GetSystemImageList()->Add(bmp);
+		bmp = wxArtProvider::GetBitmap(_T("ART_SORT_UP_") + lightness, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
+		m_header_icon_index.up = imageList->Add(bmp);
+		bmp = wxArtProvider::GetBitmap(_T("ART_SORT_DOWN_") + lightness, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
+		m_header_icon_index.down = imageList->Add(bmp);
+	}
 #endif
 }
 
 void wxListCtrlEx::SetHeaderSortIconIndex(int col, int icon)
 {
-	if (col < 0 || col >= GetColumnCount())
+	if (col < 0 || col >= GetColumnCount()) {
 		return;
+	}
 
 #ifdef __WXMSW__
 	HWND hWnd = (HWND)GetHandle();
@@ -882,10 +891,12 @@ void wxListCtrlEx::SetHeaderSortIconIndex(int col, int icon)
 	if (icon != -1) {
 		item.fmt &= ~(HDF_IMAGE | HDF_BITMAP_ON_RIGHT | HDF_SORTUP | HDF_SORTDOWN);
 		item.iImage = -1;
-		if (icon)
+		if (icon) {
 			item.fmt |= HDF_SORTDOWN;
-		else
+		}
+		else {
 			item.fmt |= HDF_SORTUP;
+		}
 	}
 	else {
 		item.fmt &= ~(HDF_IMAGE | HDF_BITMAP_ON_RIGHT | HDF_SORTUP | HDF_SORTDOWN);
@@ -934,23 +945,23 @@ void wxListCtrlEx::CancelLabelEdit()
 void wxListCtrlEx::OnBeginLabelEdit(wxListEvent& event)
 {
 #ifndef __WXMSW__
-	if (m_editing)
-	{
+	if (m_editing) {
 		event.Veto();
 		return;
 	}
 #endif
-	if (m_blockedLabelEditing)
-	{
+	if (m_blockedLabelEditing) {
 		event.Veto();
 		return;
 	}
 
-	if (!OnBeginRename(event))
+	if (!OnBeginRename(event)) {
 		event.Veto();
+	}
 #ifndef __WXMSW__
-	else
+	else {
 		m_editing = true;
+	}
 #endif
 }
 
@@ -958,35 +969,36 @@ void wxListCtrlEx::OnEndLabelEdit(wxListEvent& event)
 {
 #ifdef __WXMAC__
 	int item = event.GetIndex();
-	if (item != -1)
-	{
+	if (item != -1) {
 		int to = item + 1;
-		if (to < GetItemCount())
-		{
+		if (to < GetItemCount()) {
 			int from = item;
-			if (from)
+			if (from) {
 				from--;
+			}
 			RefreshItems(from, to);
 		}
-		else
+		else {
 			RefreshListOnly();
+		}
 	}
 #endif
 
 #ifndef __WXMSW__
-	if (!m_editing)
-	{
+	if (!m_editing) {
 		event.Veto();
 		return;
 	}
 	m_editing = false;
 #endif
 
-	if (event.IsEditCancelled())
+	if (event.IsEditCancelled()) {
 		return;
+	}
 
-	if (!OnAcceptRename(event))
+	if (!OnAcceptRename(event)) {
 		event.Veto();
+	}
 }
 
 bool wxListCtrlEx::OnBeginRename(const wxListEvent&)
@@ -1001,16 +1013,15 @@ bool wxListCtrlEx::OnAcceptRename(const wxListEvent&)
 
 void wxListCtrlEx::SetLabelEditBlock(bool block)
 {
-	if (block)
-	{
+	if (block) {
 		CancelLabelEdit();
 		++m_blockedLabelEditing;
 	}
-	else
-	{
+	else {
 		wxASSERT(m_blockedLabelEditing);
-		if (m_blockedLabelEditing > 0)
-			m_blockedLabelEditing--;
+		if (m_blockedLabelEditing > 0) {
+			--m_blockedLabelEditing;
+		}
 	}
 }
 
@@ -1027,8 +1038,9 @@ CLabelEditBlocker::~CLabelEditBlocker()
 
 void wxListCtrlEx::OnColumnDragging(wxListEvent& event)
 {
-	if (event.GetItem().GetWidth() < MIN_COLUMN_WIDTH)
+	if (event.GetItem().GetWidth() < MIN_COLUMN_WIDTH) {
 		event.Veto();
+	}
 }
 
 #ifdef __WXMSW__
@@ -1039,8 +1051,9 @@ bool wxListCtrlEx::MSWOnNotify(int idCtrl, WXLPARAM lParam, WXLPARAM *result)
 	NMHDR *nmhdr = (NMHDR *)lParam;
 	HWND hwndHdr = ListView_GetHeader((HWND)GetHandle());
 
-	if (nmhdr->hwndFrom != hwndHdr)
+	if (nmhdr->hwndFrom != hwndHdr) {
 		return wxListCtrl::MSWOnNotify(idCtrl, lParam, result);
+	}
 
 	HD_NOTIFY *nmHDR = (HD_NOTIFY *)nmhdr;
 
