@@ -47,17 +47,17 @@ bool CLocalRecursiveOperation::DoStartRecursiveOperation(OperationMode mode, Act
 	if (!m_pQueue) {
 		return false;
 	}
-	
-	auto const server = m_state.GetServer();
+
+	ServerWithCredentials const& server = m_state.GetServer();
 	if (server) {
-		server_ = *server;
+		server_ = server;
 	}
 	else {
 		if (mode != OperationMode::recursive_list) {
 			return false;
 		}
 
-		server_ = CServer();
+		server_ = ServerWithCredentials();
 	}
 
 	{
@@ -121,7 +121,7 @@ void CLocalRecursiveOperation::StopRecursiveOperation()
 	m_actionAfterBlocker.reset();
 }
 
-void CLocalRecursiveOperation::OnStateChange(t_statechange_notifications notification, const wxString&, const void* data2)
+void CLocalRecursiveOperation::OnStateChange(t_statechange_notifications, const wxString&, const void*)
 {
 }
 
@@ -275,7 +275,7 @@ void CLocalRecursiveOperation::OnListedDirectory()
 		}
 
 		if (d.localPath.empty()) {
-			StopRecursiveOperation();
+			stop = true;
 		}
 		else {
 			if (queue) {
@@ -290,7 +290,7 @@ void CLocalRecursiveOperation::OnListedDirectory()
 	if (queue) {
 		m_pQueue->QueueFile_Finish(m_immediate);
 	}
-	
+
 	m_processedFiles += processed;
 	if (stop) {
 		StopRecursiveOperation();
