@@ -27,23 +27,16 @@ END_EVENT_TABLE()
 
 CManualTransfer::CManualTransfer(CQueueView* pQueueView)
 	: m_local_file_exists()
-	, m_pServer()
-	, m_pLastSite()
 	, m_pState()
 	, m_pQueueView(pQueueView)
 {
 }
 
-CManualTransfer::~CManualTransfer()
-{
-	delete m_pServer;
-	delete m_pLastSite;
-}
-
 void CManualTransfer::Run(wxWindow* pParent, CState* pState)
 {
-	if (!Load(pParent, _T("ID_MANUALTRANSFER")))
+	if (!Load(pParent, _T("ID_MANUALTRANSFER"))) {
 		return;
+	}
 
 	m_pState = pState;
 
@@ -58,11 +51,12 @@ void CManualTransfer::Run(wxWindow* pParent, CState* pState)
 
 	wxChoice* pChoice = XRCCTRL(*this, "ID_LOGONTYPE", wxChoice);
 	wxASSERT(pChoice);
-	for (int i = 0; i < LOGONTYPE_MAX; ++i)
-		pChoice->Append(CServer::GetNameFromLogonType((enum LogonType)i));
+	for (int i = 0; i < static_cast<int>(LogonType::count); ++i) {
+		pChoice->Append(GetNameFromLogonType(static_cast<LogonType>(i)));
+	}
 
-	if (m_pState->GetServer()) {
-		m_pServer = new CServer(*m_pState->GetServer());
+	server_ = m_pState->GetServer();
+	if (server_) {
 		XRCCTRL(*this, "ID_SERVER_CURRENT", wxRadioButton)->SetValue(true);
 		DisplayServer();
 	}
@@ -114,7 +108,7 @@ void CManualTransfer::SetAutoAsciiState()
 			XRCCTRL(*this, "ID_TYPE_AUTO_ASCII", wxStaticText)->Hide();
 			XRCCTRL(*this, "ID_TYPE_AUTO_BINARY", wxStaticText)->Hide();
 		}
-		else if (CAutoAsciiFiles::TransferLocalAsAscii(remote_file, m_pServer ? m_pServer->GetType() : UNIX)) {
+		else if (CAutoAsciiFiles::TransferLocalAsAscii(remote_file, server_.server.GetType())) {
 			XRCCTRL(*this, "ID_TYPE_AUTO_ASCII", wxStaticText)->Show();
 			XRCCTRL(*this, "ID_TYPE_AUTO_BINARY", wxStaticText)->Hide();
 		}
@@ -129,7 +123,7 @@ void CManualTransfer::SetAutoAsciiState()
 			XRCCTRL(*this, "ID_TYPE_AUTO_ASCII", wxStaticText)->Hide();
 			XRCCTRL(*this, "ID_TYPE_AUTO_BINARY", wxStaticText)->Hide();
 		}
-		else if (CAutoAsciiFiles::TransferLocalAsAscii(local_file, m_pServer ? m_pServer->GetType() : UNIX)) {
+		else if (CAutoAsciiFiles::TransferLocalAsAscii(local_file, server_.server.GetType())) {
 			XRCCTRL(*this, "ID_TYPE_AUTO_ASCII", wxStaticText)->Show();
 			XRCCTRL(*this, "ID_TYPE_AUTO_BINARY", wxStaticText)->Hide();
 		}
@@ -157,34 +151,37 @@ void CManualTransfer::SetServerState()
 
 void CManualTransfer::DisplayServer()
 {
-	if (m_pServer)
-	{
-		XRCCTRL(*this, "ID_HOST", wxTextCtrl)->ChangeValue(m_pServer->FormatHost(true));
-		unsigned int port = m_pServer->GetPort();
+	if (server_) {
+		XRCCTRL(*this, "ID_HOST", wxTextCtrl)->ChangeValue(server_.Format(ServerFormat::host_only));
+		unsigned int port = server_.server.GetPort();
 
-		if (port != CServer::GetDefaultPort(m_pServer->GetProtocol()))
+		if (port != CServer::GetDefaultPort(server_.server.GetProtocol())) {
 			XRCCTRL(*this, "ID_PORT", wxTextCtrl)->ChangeValue(wxString::Format(_T("%d"), port));
-		else
+		}
+		else {
 			XRCCTRL(*this, "ID_PORT", wxTextCtrl)->ChangeValue(_T(""));
+		}
 
-		const wxString& protocolName = CServer::GetProtocolName(m_pServer->GetProtocol());
-		if (!protocolName.empty())
+		const wxString& protocolName = CServer::GetProtocolName(server_.server.GetProtocol());
+		if (!protocolName.empty()) {
 			XRCCTRL(*this, "ID_PROTOCOL", wxChoice)->SetStringSelection(protocolName);
-		else
+		}
+		else {
 			XRCCTRL(*this, "ID_PROTOCOL", wxChoice)->SetStringSelection(CServer::GetProtocolName(FTP));
+		}
 
-		switch (m_pServer->GetLogonType())
+		switch (server_.credentials.logonType_)
 		{
-		case NORMAL:
+		case LogonType::normal:
 			XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->SetStringSelection(_("Normal"));
 			break;
-		case ASK:
+		case LogonType::ask:
 			XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->SetStringSelection(_("Ask for password"));
 			break;
-		case INTERACTIVE:
+		case LogonType::interactive:
 			XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->SetStringSelection(_("Interactive"));
 			break;
-		case ACCOUNT:
+		case LogonType::account:
 			XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->SetStringSelection(_("Account"));
 			break;
 		default:
@@ -192,12 +189,11 @@ void CManualTransfer::DisplayServer()
 			break;
 		}
 
-		XRCCTRL(*this, "ID_USER", wxTextCtrl)->ChangeValue(m_pServer->GetUser());
-		XRCCTRL(*this, "ID_ACCOUNT", wxTextCtrl)->ChangeValue(m_pServer->GetAccount());
-		XRCCTRL(*this, "ID_PASS", wxTextCtrl)->ChangeValue(m_pServer->GetPass());
+		XRCCTRL(*this, "ID_USER", wxTextCtrl)->ChangeValue(server_.server.GetUser());
+		XRCCTRL(*this, "ID_ACCOUNT", wxTextCtrl)->ChangeValue(server_.credentials.account_);
+		XRCCTRL(*this, "ID_PASS", wxTextCtrl)->ChangeValue(server_.credentials.GetPass());
 	}
-	else
-	{
+	else {
 		XRCCTRL(*this, "ID_HOST", wxTextCtrl)->ChangeValue(_T(""));
 		XRCCTRL(*this, "ID_PORT", wxTextCtrl)->ChangeValue(_T(""));
 		XRCCTRL(*this, "ID_PROTOCOL", wxChoice)->SetStringSelection(CServer::GetProtocolName(FTP));
@@ -214,8 +210,9 @@ void CManualTransfer::DisplayServer()
 
 void CManualTransfer::OnLocalChanged(wxCommandEvent&)
 {
-	if (XRCCTRL(*this, "ID_DOWNLOAD", wxRadioButton)->GetValue())
+	if (XRCCTRL(*this, "ID_DOWNLOAD", wxRadioButton)->GetValue()) {
 		return;
+	}
 
 	wxString file = XRCCTRL(*this, "ID_LOCALFILE", wxTextCtrl)->GetValue();
 
@@ -245,8 +242,9 @@ void CManualTransfer::OnLocalBrowse(wxCommandEvent&)
 	wxFileDialog dlg(this, title, _T(""), _T(""), _T("*.*"), flags);
 	int res = dlg.ShowModal();
 
-	if (res != wxID_OK)
+	if (res != wxID_OK) {
 		return;
+	}
 
 	// SetValue on purpose
 	xrc_call(*this, "ID_LOCALFILE", &wxTextCtrl::SetValue, dlg.GetPath());
@@ -254,8 +252,9 @@ void CManualTransfer::OnLocalBrowse(wxCommandEvent&)
 
 void CManualTransfer::OnDirection(wxCommandEvent& event)
 {
-	if (xrc_call(*this, "ID_DOWNLOAD", &wxRadioButton::GetValue))
+	if (xrc_call(*this, "ID_DOWNLOAD", &wxRadioButton::GetValue)) {
 		SetAutoAsciiState();
+	}
 	else {
 		// Need to check for file existence
 		OnLocalChanged(event);
@@ -265,19 +264,10 @@ void CManualTransfer::OnDirection(wxCommandEvent& event)
 void CManualTransfer::OnServerTypeChanged(wxCommandEvent& event)
 {
 	if (event.GetId() == XRCID("ID_SERVER_CURRENT")) {
-		delete m_pServer;
-		if (m_pState->GetServer())
-			m_pServer = new CServer(*m_pState->GetServer());
-		else
-			m_pServer = 0;
+		server_ = m_pState->GetServer();
 	}
 	else if (event.GetId() == XRCID("ID_SERVER_SITE")) {
-		delete m_pServer;
-		if (m_pLastSite)
-			m_pServer = new CServer(*m_pLastSite);
-		else
-			m_pServer = 0;
-
+		server_ = lastSite_;
 	}
 	xrc_call(*this, "ID_SERVER_SITE_SELECT", &wxButton::Enable, event.GetId() == XRCID("ID_SERVER_SITE"));
 	DisplayServer();
@@ -286,19 +276,20 @@ void CManualTransfer::OnServerTypeChanged(wxCommandEvent& event)
 
 void CManualTransfer::OnOK(wxCommandEvent&)
 {
-	if (!UpdateServer())
+	if (!UpdateServer()) {
 		return;
+	}
 
 	bool download = xrc_call(*this, "ID_DOWNLOAD", &wxRadioButton::GetValue);
 
 	bool start = xrc_call(*this, "ID_START", &wxCheckBox::GetValue);
 
-	if (!m_pServer) {
+	if (!server_) {
 		wxMessageBoxEx(_("You need to specify a server."), _("Manual transfer"), wxICON_EXCLAMATION);
 		return;
 	}
 
-	wxString local_file = xrc_call(*this, "ID_LOCALFILE", &wxTextCtrl::GetValue);
+	std::wstring local_file = xrc_call(*this, "ID_LOCALFILE", &wxTextCtrl::GetValue).ToStdWstring();
 	if (local_file.empty()) {
 		wxMessageBoxEx(_("You need to specify a local file."), _("Manual transfer"), wxICON_EXCLAMATION);
 		return;
@@ -314,20 +305,20 @@ void CManualTransfer::OnOK(wxCommandEvent&)
 		return;
 	}
 
-	wxString remote_file = xrc_call(*this, "ID_REMOTEFILE", &wxTextCtrl::GetValue);
+	std::wstring remote_file = xrc_call(*this, "ID_REMOTEFILE", &wxTextCtrl::GetValue).ToStdWstring();
 
 	if (remote_file.empty()) {
 		wxMessageBoxEx(_("You need to specify a remote file."), _("Manual transfer"), wxICON_EXCLAMATION);
 		return;
 	}
 
-	wxString remote_path_str = xrc_call(*this, "ID_REMOTEPATH", &wxTextCtrl::GetValue);
+	std::wstring remote_path_str = xrc_call(*this, "ID_REMOTEPATH", &wxTextCtrl::GetValue).ToStdWstring();
 	if (remote_path_str.empty()) {
 		wxMessageBoxEx(_("You need to specify a remote path."), _("Manual transfer"), wxICON_EXCLAMATION);
 		return;
 	}
 
-	CServerPath path(remote_path_str, m_pServer->GetType());
+	CServerPath path(remote_path_str, server_.server.GetType());
 	if (path.empty()) {
 		wxMessageBoxEx(_("Remote path could not be parsed."), _("Manual transfer"), wxICON_EXCLAMATION);
 		return;
@@ -336,14 +327,17 @@ void CManualTransfer::OnOK(wxCommandEvent&)
 	int old_data_type = COptions::Get()->GetOptionVal(OPTION_ASCIIBINARY);
 
 	// Set data type for the file to add
-	if (xrc_call(*this, "ID_TYPE_ASCII", &wxRadioButton::GetValue))
+	if (xrc_call(*this, "ID_TYPE_ASCII", &wxRadioButton::GetValue)) {
 		COptions::Get()->SetOption(OPTION_ASCIIBINARY, 1);
-	else if (xrc_call(*this, "ID_TYPE_BINARY", &wxRadioButton::GetValue))
+	}
+	else if (xrc_call(*this, "ID_TYPE_BINARY", &wxRadioButton::GetValue)) {
 		COptions::Get()->SetOption(OPTION_ASCIIBINARY, 2);
-	else
+	}
+	else {
 		COptions::Get()->SetOption(OPTION_ASCIIBINARY, 0);
+	}
 
-	wxString name;
+	std::wstring name;
 	CLocalPath localPath(local_file, &name);
 
 	if (name.empty()) {
@@ -353,8 +347,8 @@ void CManualTransfer::OnOK(wxCommandEvent&)
 
 	m_pQueueView->QueueFile(!start, download,
 		download ? remote_file : name,
-		(remote_file != name) ? (download ? name : remote_file) : wxString(),
-		localPath, path, *m_pServer, -1);
+		(remote_file != name) ? (download ? name : remote_file) : std::wstring(),
+		localPath, path, server_, -1);
 
 	// Restore old data type
 	COptions::Get()->SetOption(OPTION_ASCIIBINARY, old_data_type);
@@ -366,62 +360,60 @@ void CManualTransfer::OnOK(wxCommandEvent&)
 
 bool CManualTransfer::UpdateServer()
 {
-	if (!xrc_call(*this, "ID_SERVER_CUSTOM", &wxRadioButton::GetValue))
+	if (!xrc_call(*this, "ID_SERVER_CUSTOM", &wxRadioButton::GetValue)) {
 		return true;
+	}
 
-	if (!VerifyServer())
+	if (!VerifyServer()) {
 		return false;
-
-	CServer server;
+	}
 
 	unsigned long port;
 	if (!xrc_call(*this, "ID_PORT", &wxTextCtrl::GetValue).ToULong(&port)) {
 		return false;
 	}
 
-	wxString host = xrc_call(*this, "ID_HOST", &wxTextCtrl::GetValue);
+	server_ = ServerWithCredentials();
+
+	std::wstring host = xrc_call(*this, "ID_HOST", &wxTextCtrl::GetValue).ToStdWstring();
 	// SetHost does not accept URL syntax
 	if (!host.empty() && host[0] == '[') {
-		host.RemoveLast();
-		host = host.Mid(1);
+		host = host.substr(1, host.size() - 2);
 	}
-	server.SetHost(host, port);
+	server_.server.SetHost(host, port);
 
-	const wxString& protocolName = xrc_call(*this, "ID_PROTOCOL", &wxChoice::GetStringSelection);
-	const enum ServerProtocol protocol = CServer::GetProtocolFromName(protocolName);
-	if (protocol != UNKNOWN)
-		server.SetProtocol(protocol);
-	else
-		server.SetProtocol(FTP);
+	std::wstring const protocolName = xrc_call(*this, "ID_PROTOCOL", &wxChoice::GetStringSelection).ToStdWstring();
+	ServerProtocol const protocol = CServer::GetProtocolFromName(protocolName);
+	if (protocol != UNKNOWN) {
+		server_.server.SetProtocol(protocol);
+	}
+	else {
+		server_.server.SetProtocol(FTP);
+	}
 
-	enum LogonType logon_type = CServer::GetLogonTypeFromName(xrc_call(*this, "ID_LOGONTYPE", &wxChoice::GetStringSelection));
-	server.SetLogonType(logon_type);
-
-	server.SetUser(xrc_call(*this, "ID_USER", &wxTextCtrl::GetValue),
-		xrc_call(*this, "ID_PASS", &wxTextCtrl::GetValue));
-	server.SetAccount(xrc_call(*this, "ID_ACCOUNT", &wxTextCtrl::GetValue));
-
-	delete m_pServer;
-	m_pServer = new CServer(server);
+	server_.SetLogonType(GetLogonTypeFromName(xrc_call(*this, "ID_LOGONTYPE", &wxChoice::GetStringSelection).ToStdWstring()));
+	server_.SetUser(xrc_call(*this, "ID_USER", &wxTextCtrl::GetValue).ToStdWstring());
+	server_.credentials.SetPass(xrc_call(*this, "ID_PASS", &wxTextCtrl::GetValue).ToStdWstring());
+	server_.credentials.account_ = xrc_call(*this, "ID_ACCOUNT", &wxTextCtrl::GetValue).ToStdWstring();
 
 	return true;
 }
 
 bool CManualTransfer::VerifyServer()
 {
-	const wxString& host = xrc_call(*this, "ID_HOST", &wxTextCtrl::GetValue);
+	std::wstring const host = xrc_call(*this, "ID_HOST", &wxTextCtrl::GetValue).ToStdWstring();
 	if (host.empty()) {
 		xrc_call(*this, "ID_HOST", &wxTextCtrl::SetFocus);
 		wxMessageBoxEx(_("You have to enter a hostname."));
 		return false;
 	}
 
-	enum LogonType logon_type = CServer::GetLogonTypeFromName(xrc_call(*this, "ID_LOGONTYPE", &wxChoice::GetStringSelection));
+	LogonType logon_type = GetLogonTypeFromName(xrc_call(*this, "ID_LOGONTYPE", &wxChoice::GetStringSelection).ToStdWstring());
 
-	wxString protocolName = xrc_call(*this, "ID_PROTOCOL", &wxChoice::GetStringSelection);
-	enum ServerProtocol protocol = CServer::GetProtocolFromName(protocolName);
+	std::wstring protocolName = xrc_call(*this, "ID_PROTOCOL", &wxChoice::GetStringSelection).ToStdWstring();
+	ServerProtocol protocol = CServer::GetProtocolFromName(protocolName);
 	if (protocol == SFTP &&
-		logon_type == ACCOUNT)
+		logon_type == LogonType::account)
 	{
 		xrc_call(*this, "ID_LOGONTYPE", &wxChoice::SetFocus);
 		wxMessageBoxEx(_("'Account' logontype not supported by selected protocol"));
@@ -429,51 +421,55 @@ bool CManualTransfer::VerifyServer()
 	}
 
 	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 0 &&
-		(logon_type == ACCOUNT || logon_type == NORMAL))
+		(logon_type == LogonType::account || logon_type == LogonType::normal))
 	{
 		xrc_call(*this, "ID_LOGONTYPE", &wxChoice::SetFocus);
 		wxString msg;
-		if (COptions::Get()->OptionFromFzDefaultsXml(OPTION_DEFAULT_KIOSKMODE))
+		if (COptions::Get()->OptionFromFzDefaultsXml(OPTION_DEFAULT_KIOSKMODE) && COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 0) {
 			msg = _("Saving of password has been disabled by your system administrator.");
-		else
+		}
+		else {
 			msg = _("Saving of passwords has been disabled by you.");
+		}
 		msg += _T("\n");
 		msg += _("'Normal' and 'Account' logontypes are not available, using 'Ask for password' instead.");
-		xrc_call(*this, "ID_LOGONTYPE", &wxChoice::SetStringSelection, CServer::GetNameFromLogonType(ASK));
+		xrc_call(*this, "ID_LOGONTYPE", &wxChoice::SetStringSelection, GetNameFromLogonType(LogonType::ask));
 		xrc_call(*this, "ID_PASS", &wxTextCtrl::ChangeValue, _T(""));
-		logon_type = ASK;
+		logon_type = LogonType::ask;
 		wxMessageBoxEx(msg, _("Cannot remember password"), wxICON_INFORMATION, this);
 	}
 
-	CServer server;
+	ServerWithCredentials server;
 
 	// Set selected type
 	server.SetLogonType(logon_type);
 
-	if (protocol != UNKNOWN)
-		server.SetProtocol(protocol);
+	if (protocol != UNKNOWN) {
+		server.server.SetProtocol(protocol);
+	}
 
 	CServerPath path;
-	wxString error;
-	if (!server.ParseUrl(host, xrc_call(*this, "ID_PORT", &wxTextCtrl::GetValue), wxString(), wxString(), error, path)) {
+	std::wstring error;
+	if (!server.ParseUrl(host, xrc_call(*this, "ID_PORT", &wxTextCtrl::GetValue).ToStdWstring(), std::wstring(), std::wstring(), error, path)) {
 		xrc_call(*this, "ID_HOST", &wxTextCtrl::SetFocus);
 		wxMessageBoxEx(error);
 		return false;
 	}
 
-	xrc_call(*this, "ID_HOST", &wxTextCtrl::ChangeValue, server.FormatHost(true));
-	xrc_call(*this, "ID_PORT", &wxTextCtrl::ChangeValue, wxString::Format(_T("%d"), server.GetPort()));
+	xrc_call(*this, "ID_HOST", &wxTextCtrl::ChangeValue, server.Format(ServerFormat::host_only));
+	xrc_call(*this, "ID_PORT", &wxTextCtrl::ChangeValue, wxString::Format(_T("%d"), server.server.GetPort()));
 
-	protocolName = CServer::GetProtocolName(server.GetProtocol());
-	if (protocolName.empty())
+	protocolName = CServer::GetProtocolName(server.server.GetProtocol());
+	if (protocolName.empty()) {
 		CServer::GetProtocolName(FTP);
+	}
 	xrc_call(*this, "ID_PROTOCOL", &wxChoice::SetStringSelection, protocolName);
 
 	// Require username for non-anonymous, non-ask logon type
 	const wxString user = xrc_call(*this, "ID_USER", &wxTextCtrl::GetValue);
-	if (logon_type != ANONYMOUS &&
-		logon_type != ASK &&
-		logon_type != INTERACTIVE &&
+	if (logon_type != LogonType::anonymous &&
+		logon_type != LogonType::ask &&
+		logon_type != LogonType::interactive &&
 		user.empty())
 	{
 		xrc_call(*this, "ID_USER", &wxTextCtrl::SetFocus);
@@ -499,7 +495,7 @@ bool CManualTransfer::VerifyServer()
 	}
 
 	// Require account for account logon type
-	if (logon_type == ACCOUNT &&
+	if (logon_type == LogonType::account &&
 		xrc_call(*this, "ID_ACCOUNT", &wxTextCtrl::GetValue).empty())
 	{
 		xrc_call(*this, "ID_ACCOUNT", &wxTextCtrl::SetFocus);
@@ -520,16 +516,15 @@ void CManualTransfer::OnSelectSite(wxCommandEvent&)
 
 void CManualTransfer::OnSelectedSite(wxCommandEvent& event)
 {
-	std::unique_ptr<CSiteManagerItemData_Site> pData = CSiteManager::GetSiteById(event.GetId());
-	if (!pData)
+	std::unique_ptr<Site> pData = CSiteManager::GetSiteById(event.GetId());
+	if (!pData) {
 		return;
+	}
 
-	delete m_pServer;
-	m_pServer = new CServer(pData->m_server);
-	delete m_pLastSite;
-	m_pLastSite = new CServer(pData->m_server);
+	server_ = pData->server_;
+	lastSite_ = pData->server_;
 
-	xrc_call(*this, "ID_SERVER_SITE_SERVER", &wxStaticText::SetLabel, m_pServer->GetName());
+	xrc_call(*this, "ID_SERVER_SITE_SERVER", &wxStaticText::SetLabel, server_.server.GetName());
 
 	DisplayServer();
 }
