@@ -1,17 +1,17 @@
-#ifndef __MANUAL_TRANSFER_H__
-#define __MANUAL_TRANSFER_H__
+#ifndef FILEZILLA_INTERFACE_MANUAL_TRANSFER_HEADER
+#define FILEZILLA_INTERFACE_MANUAL_TRANSFER_HEADER
 
 #include "dialogex.h"
+#include "serverdata.h"
 
 class CQueueView;
 class CState;
-class CManualTransfer : public wxDialogEx
+class CManualTransfer final : public wxDialogEx
 {
 public:
 	CManualTransfer(CQueueView* pQueueView);
-	virtual ~CManualTransfer();
 
-	void Show(wxWindow* pParent, CState* pState);
+	void Run(wxWindow* pParent, CState* pState);
 
 protected:
 	void DisplayServer();
@@ -24,8 +24,8 @@ protected:
 
 	bool m_local_file_exists;
 
-	CServer *m_pServer;
-	CServer* m_pLastSite;
+	ServerWithCredentials server_;
+	ServerWithCredentials lastSite_;
 
 	CState* m_pState;
 	CQueueView* m_pQueueView;
@@ -42,4 +42,4 @@ protected:
 	void OnLogontypeSelChanged(wxCommandEvent& event);
 };
 
-#endif //__MANUAL_TRANSFER_H__
+#endif
