@@ -8,6 +8,9 @@
 #include "QueueView.h"
 #include "sitemanager.h"
 #include "state.h"
+#if USE_MAC_SANDBOX
+#include "osx_sandbox_userdirs.h"
+#endif
 
 IMPLEMENT_DYNAMIC_CLASS(CMenuBar, wxMenuBar)
 
@@ -72,10 +75,12 @@ CMenuBar* CMenuBar::Load(CMainFrame* pMainFrame)
 	menubar->Check(XRCID("ID_MENU_SERVER_VIEWHIDDEN"), COptions::Get()->GetOptionVal(OPTION_VIEW_HIDDEN_FILES) ? true : false);
 
 	int mode = COptions::Get()->GetOptionVal(OPTION_COMPARISONMODE);
-	if (mode != 1)
+	if (mode != 1) {
 		menubar->Check(XRCID("ID_COMPARE_SIZE"), true);
-	else
+	}
+	else {
 		menubar->Check(XRCID("ID_COMPARE_DATE"), true);
+	}
 
 	menubar->Check(XRCID("ID_COMPARE_HIDEIDENTICAL"), COptions::Get()->GetOptionVal(OPTION_COMPARE_HIDEIDENTICAL) != 0);
 	menubar->Check(XRCID("ID_VIEW_QUICKCONNECT"), COptions::Get()->GetOptionVal(OPTION_SHOW_QUICKCONNECT) != 0);
@@ -134,6 +139,22 @@ CMenuBar* CMenuBar::Load(CMainFrame* pMainFrame)
 	menubar->RegisterOption(OPTION_SPEEDLIMIT_INBOUND);
 	menubar->RegisterOption(OPTION_SPEEDLIMIT_OUTBOUND);
 
+#ifdef FZ_MAC
+	wxMenu* editMenu = nullptr;
+	wxMenuItem* dirsItem = menubar->FindItem(XRCID("ID_MENU_EDIT_SANDBOX_DIRECTORIES"), &editMenu);
+	if (editMenu && dirsItem) {
+#if USE_MAC_SANDBOX
+		editMenu->Bind(wxEVT_COMMAND_MENU_SELECTED, [pMainFrame](wxCommandEvent&)
+			{
+				OSXSandboxUserdirsDialog dlg;
+				dlg.Run(pMainFrame);
+			}, XRCID("ID_MENU_EDIT_SANDBOX_DIRECTORIES"));
+#else		
+		editMenu->Delete(dirsItem);
+#endif
+	}
+#endif
+
 	return menubar;
 }
 
@@ -166,11 +187,11 @@ void CMenuBar::UpdateBookmarkMenu()
 	auto ids = m_bookmark_menu_ids.begin();
 
 	// Insert global bookmarks
-	std::list<wxString> global_bookmarks;
+	std::vector<wxString> global_bookmarks;
 	if (CBookmarksDialog::GetGlobalBookmarks(global_bookmarks) && !global_bookmarks.empty()) {
 		pMenu->AppendSeparator();
 
-		for (std::list<wxString>::const_iterator iter = global_bookmarks.begin(); iter != global_bookmarks.end(); ++iter) {
+		for (auto const& bookmark : global_bookmarks) {
 			int id;
 			if (ids == m_bookmark_menu_ids.end()) {
 				id = wxNewId();
@@ -180,11 +201,11 @@ void CMenuBar::UpdateBookmarkMenu()
 				id = *ids;
 				++ids;
 			}
-			wxString name(*iter);
+			wxString name(bookmark);
 			name.Replace(_T("&"), _T("&&"));
 			pMenu->Append(id, name);
 
-			m_bookmark_menu_id_map_global[id] = *iter;
+			m_bookmark_menu_id_map_global[id] = bookmark;
 		}
 	}
 
@@ -197,7 +218,7 @@ void CMenuBar::UpdateBookmarkMenu()
 	}
 
 	Site site = pState->GetSite();
-	if (!site.m_server) {
+	if (!site.server_) {
 		site = pState->GetLastSite();
 	}
 	if (site.m_bookmarks.empty()) {
@@ -241,7 +262,7 @@ void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 			return;
 
 		Site site = pState->GetSite();
-		if (!site.m_server) {
+		if (!site.server_) {
 			site = pState->GetLastSite();
 		}
 
@@ -251,8 +272,8 @@ void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 
 				pState->SetSyncBrowse(false);
 				if (!bookmark.m_remoteDir.empty() && pState->IsRemoteIdle(true)) {
-					const CServer* pServer = pState->GetServer();
-					if (!pServer || *pServer != site.m_server) {
+					ServerWithCredentials const& server = pState->GetServer();
+					if (!server || server != site.server_) {
 						m_pMainFrame->ConnectToSite(site, bookmark);
 						break;
 					}
@@ -293,8 +314,8 @@ void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 
 		pState->SetSyncBrowse(false);
 		if (!remote_dir.empty() && pState->IsRemoteIdle(true)) {
-			const CServer* pServer = pState->GetServer();
-			if (pServer) {
+			ServerWithCredentials const& server = pState->GetServer();
+			if (server) {
 				CServerPath current_remote_path = pState->GetRemotePath();
 				if (!current_remote_path.empty() && current_remote_path.GetType() != remote_dir.GetType()) {
 					wxMessageBoxEx(_("Selected global bookmark and current server use a different server type.\nUse site-specific bookmarks for this server."), _("Bookmark"), wxICON_EXCLAMATION, this);
@@ -440,32 +461,33 @@ void CMenuBar::UpdateMenubarState()
 		return;
 	}
 
-	const CServer* const pServer = pState->GetServer();
+	ServerWithCredentials const& server = pState->GetServer();
 	const bool idle = pState->IsRemoteIdle();
 
-	Enable(XRCID("ID_MENU_SERVER_DISCONNECT"), pServer && idle);
-	Enable(XRCID("ID_CANCEL"), pServer && !idle);
-	Enable(XRCID("ID_MENU_SERVER_CMD"), pServer && idle);
-	Enable(XRCID("ID_MENU_FILE_COPYSITEMANAGER"), pServer != 0);
-	Enable(XRCID("ID_TOOLBAR_SYNCHRONIZED_BROWSING"), pServer != 0);
+	Enable(XRCID("ID_MENU_SERVER_DISCONNECT"), server && idle);
+	Enable(XRCID("ID_CANCEL"), server && !idle);
+	Enable(XRCID("ID_MENU_SERVER_CMD"), server && idle);
+	Enable(XRCID("ID_MENU_FILE_COPYSITEMANAGER"), server.operator bool());
+	Enable(XRCID("ID_TOOLBAR_SYNCHRONIZED_BROWSING"), server.operator bool());
 
 	Check(XRCID("ID_TOOLBAR_COMPARISON"), pState->GetComparisonManager()->IsComparing());
 	Check(XRCID("ID_TOOLBAR_SYNCHRONIZED_BROWSING"), pState->GetSyncBrowse());
 
 	bool canReconnect;
-	if (pServer || !idle)
+	if (server || !idle)
 		canReconnect = false;
 	else {
-		canReconnect = static_cast<bool>(pState->GetLastSite().m_server);
+		canReconnect = static_cast<bool>(pState->GetLastSite().server_);
 	}
 	Enable(XRCID("ID_MENU_SERVER_RECONNECT"), canReconnect);
 
 	wxMenuItem* pItem = FindItem(XRCID("ID_MENU_TRANSFER_TYPE"));
-	if (!pServer || CServer::ProtocolHasDataTypeConcept(pServer->GetProtocol())) {
+	if (!server || CServer::ProtocolHasDataTypeConcept(server.server.GetProtocol())) {
 		pItem->Enable(true);
 	}
-	else
+	else {
 		pItem->Enable(false);
+	}
 }
 
 bool CMenuBar::ShowItem(int id)
