@@ -1,11 +1,11 @@
-#ifndef __MENU_BAR_H__
-#define __MENU_BAR_H__
+#ifndef FILEZILLA_INTERFACE_MENU_BAR_HEADER
+#define FILEZILLA_INTERFACE_MENU_BAR_HEADER
 
 #include <option_change_event_handler.h>
 #include "state.h"
 
 class CMainFrame;
-class CMenuBar : public wxMenuBar, public CStateEventHandler, public COptionChangeEventHandler
+class CMenuBar final : public wxMenuBar, public CGlobalStateEventHandler, public COptionChangeEventHandler
 {
 public:
 	CMenuBar();
@@ -17,7 +17,6 @@ public:
 	bool HideItem(int id);
 
 	void UpdateBookmarkMenu();
-	void ClearBookmarks();
 
 	std::list<int> m_bookmark_menu_ids;
 	std::map<int, wxString> m_bookmark_menu_id_map_global;
@@ -29,7 +28,7 @@ protected:
 
 	void UpdateSpeedLimitMenuItem();
 
-	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2);
+	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, const wxString& data, const void* data2);
 	virtual void OnOptionsChanged(changed_options_t const& options);
 
 	DECLARE_DYNAMIC_CLASS(CMenuBar)
@@ -40,4 +39,4 @@ protected:
 	std::map<wxMenu*, std::map<int, wxMenuItem*> > m_hidden_items;
 };
 
-#endif //__MENU_BAR_H__
+#endif
