@@ -5,15 +5,16 @@
 
 #include <wx/timer.h>
 
-class CRecursiveOperationStatus : public wxWindow, public CStateEventHandler
+class CRecursiveOperationStatus final : public wxWindow, public CStateEventHandler
 {
 public:
-	CRecursiveOperationStatus(wxWindow* pParent, CState* pState);
+	CRecursiveOperationStatus(wxWindow* pParent, CState& state, bool local);
 
-protected:
+	CRecursiveOperationStatus(CRecursiveOperationStatus const&) = delete;
+private:
 	virtual bool Show(bool show);
 
-	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*);
+	virtual void OnStateChange(t_statechange_notifications notification, const wxString&, const void*);
 
 	void UpdateText();
 
@@ -26,6 +27,8 @@ protected:
 	void OnPaint(wxPaintEvent& ev);
 	void OnCancel(wxCommandEvent& ev);
 	void OnTimer(wxTimerEvent&);
+
+	bool const m_local;
 };
 
 #endif
