@@ -111,8 +111,9 @@ void CRemoteRecursiveOperation::StartRecursiveOperation(OperationMode mode, Acti
 
 bool CRemoteRecursiveOperation::NextOperation()
 {
-	if (m_operationMode == recursive_none)
+	if (m_operationMode == recursive_none) {
 		return false;
+	}
 
 	while (!recursion_roots_.empty()) {
 		auto & root = recursion_roots_.front();
@@ -150,20 +151,24 @@ bool CRemoteRecursiveOperation::NextOperation()
 bool CRemoteRecursiveOperation::BelowRecursionRoot(const CServerPath& path, recursion_root::new_dir &dir)
 {
 	if (!dir.start_dir.empty()) {
-		if (path.IsSubdirOf(dir.start_dir, false))
+		if (path.IsSubdirOf(dir.start_dir, false)) {
 			return true;
-		else
+		}
+		else {
 			return false;
+		}
 	}
 
 	auto & root = recursion_roots_.front();
-	if (path.IsSubdirOf(root.m_remoteStartDir, false))
+	if (path.IsSubdirOf(root.m_remoteStartDir, false)) {
 		return true;
+	}
 
 	// In some cases (chmod from tree for example) it is neccessary to list the
 	// parent first
-	if (path == root.m_remoteStartDir && root.m_allowParent)
+	if (path == root.m_remoteStartDir && root.m_allowParent) {
 		return true;
+	}
 
 	if (dir.link == 2) {
 		dir.start_dir = path;
@@ -229,8 +234,8 @@ void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing*
 
 	++m_processedDirectories;
 
-	const CServer* pServer = m_state.GetServer();
-	wxASSERT(pServer);
+	ServerWithCredentials const& server = m_state.GetServer();
+	wxASSERT(server);
 
 	if (!pDirectoryListing->GetCount() && m_operationMode == recursive_transfer) {
 		if (m_immediate) {
@@ -238,7 +243,7 @@ void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing*
 			m_state.RefreshLocalFile(dir.localDir.GetPath());
 		}
 		else {
-			m_pQueue->QueueFile(true, true, _T(""), _T(""), dir.localDir, CServerPath(), *pServer, -1);
+			m_pQueue->QueueFile(true, true, _T(""), _T(""), dir.localDir, CServerPath(), server, -1);
 			m_pQueue->QueueFile_Finish(false);
 		}
 	}
@@ -345,7 +350,7 @@ void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing*
 					}
 					m_pQueue->QueueFile(!m_immediate, true,
 						entry.name, (entry.name == localFile) ? std::wstring() : localFile,
-						dir.localDir, pDirectoryListing->path, *pServer, entry.size);
+						dir.localDir, pDirectoryListing->path, server, entry.size);
 					added = true;
 				}
 				break;
@@ -412,10 +417,11 @@ void CRemoteRecursiveOperation::StopRecursiveOperation()
 
 void CRemoteRecursiveOperation::ListingFailed(int error)
 {
-	if (m_operationMode == recursive_none || recursion_roots_.empty())
+	if (m_operationMode == recursive_none || recursion_roots_.empty()) {
 		return;
+	}
 
-	if( (error & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED) {
+	if ((error & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED) {
 		// User has cancelled operation
 		StopRecursiveOperation();
 		return;
@@ -432,6 +438,15 @@ void CRemoteRecursiveOperation::ListingFailed(int error)
 		dir.second_try = true;
 		root.m_dirsToVisit.push_front(dir);
 	}
+	else {
+		if (m_operationMode == recursive_delete && dir.doVisit && !dir.subdir.empty()) {
+			// After recursing into directory to delete its contents, delete directory itself
+			// Gets handled in NextOperation
+			recursion_root::new_dir dir2 = dir;
+			dir2.doVisit = false;
+			root.m_dirsToVisit.push_front(dir2);
+		}
+	}
 
 	NextOperation();
 }
@@ -448,8 +463,8 @@ void CRemoteRecursiveOperation::LinkIsNotDir()
 	recursion_root::new_dir dir = root.m_dirsToVisit.front();
 	root.m_dirsToVisit.pop_front();
 
-	const CServer* pServer = m_state.GetServer();
-	if (!pServer) {
+	ServerWithCredentials const& server = m_state.GetServer();
+	if (!server) {
 		NextOperation();
 		return;
 	}
@@ -469,7 +484,7 @@ void CRemoteRecursiveOperation::LinkIsNotDir()
 		if (m_operationMode != recursive_transfer_flatten) {
 			localPath.MakeParent();
 		}
-		m_pQueue->QueueFile(!m_immediate, true, dir.subdir, (dir.subdir == localFile) ? std::wstring() : localFile, localPath, dir.parent, *pServer, -1);
+		m_pQueue->QueueFile(!m_immediate, true, dir.subdir, (dir.subdir == localFile) ? std::wstring() : localFile, localPath, dir.parent, server, -1);
 		m_pQueue->QueueFile_Finish(m_immediate);
 	}
 
