@@ -1,7 +1,7 @@
-#ifndef __OPTIONSPAGE_DATEFORMATTING_H__
-#define __OPTIONSPAGE_DATEFORMATTING_H__
+#ifndef FILEZILLA_INTERFACE_OPTIONSPAGE_DATEFORMATTING_HEADER
+#define FILEZILLA_INTERFACE_OPTIONSPAGE_DATEFORMATTING_HEADER
 
-class COptionsPageDateFormatting : public COptionsPage
+class COptionsPageDateFormatting final : public COptionsPage
 {
 public:
 	virtual wxString GetResourceName() { return _T("ID_SETTINGS_DATEFORMATTING"); }
@@ -17,4 +17,4 @@ protected:
 	void OnRadioChanged(wxCommandEvent& event);
 };
 
-#endif //__OPTIONSPAGE_DATEFORMATTING_H__
+#endif
