@@ -1,5 +1,5 @@
-#ifndef __OPTIONSPAGE_UPDATECHECK_H__
-#define __OPTIONSPAGE_UPDATECHECK_H__
+#ifndef FILEZILLA_INTERFACE_OPTIONSPAGE_UPDATECHECK_HEADER
+#define FILEZILLA_INTERFACE_OPTIONSPAGE_UPDATECHECK_HEADER
 
 #if FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
 
@@ -17,6 +17,6 @@ protected:
 	DECLARE_EVENT_TABLE()
 };
 
-#endif //FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
+#endif
 
-#endif //__OPTIONSPAGE_UPDATECHECK_H__
+#endif
