@@ -1,5 +1,5 @@
-#ifndef __SITEMANAGER_H__
-#define __SITEMANAGER_H__
+#ifndef FILEZILLA_INTERFACE_SITEMANAGER_HEADER
+#define FILEZILLA_INTERFACE_SITEMANAGER_HEADER
 
 #include <wx/treectrl.h>
 
@@ -23,11 +23,11 @@ public:
 class Site final
 {
 public:
-	bool empty() const { return !m_server; }
+	bool empty() const { return !server_; }
 	bool operator==(Site const& s) const;
 	bool operator!=(Site const& s) const { return !(*this == s); }
 
-	CServer m_server;
+	ServerWithCredentials server_;
 	wxString m_comments;
 
 	Bookmark m_default_bookmark;
@@ -52,6 +52,7 @@ public:
 	virtual bool LevelUp() { return true; } // *Ding*
 };
 
+class CLoginManager;
 class CSiteManagerXmlHandler;
 class CSiteManagerDialog;
 class CSiteManager
@@ -65,10 +66,11 @@ public:
 
 	static std::pair<std::unique_ptr<Site>, Bookmark> GetSiteByPath(std::wstring const& sitePath, bool printErrors = true);
 
-	static wxString AddServer(CServer server);
+	static wxString AddServer(ServerWithCredentials server);
 	static bool AddBookmark(std::wstring sitePath, const wxString& name, const wxString &local_dir, const CServerPath &remote_dir, bool sync, bool comparison);
 	static bool ClearBookmarks(std::wstring sitePath);
 
+	static void Rewrite(CLoginManager & loginManager, bool on_failure_set_to_ask);
 
 	static bool UnescapeSitePath(std::wstring path, std::vector<std::wstring>& result);
 	static std::wstring EscapeSegment(std::wstring segment);
@@ -82,6 +84,9 @@ public:
 	static wxString GetColourName(int i);
 
 protected:
+	static void Rewrite(CLoginManager & loginManager, pugi::xml_node element, bool on_failure_set_to_ask);
+	static void Save(pugi::xml_node element, Site const& site);
+
 	static std::pair<std::unique_ptr<Site>, Bookmark> DoGetSiteByPath(std::wstring sitePath, wxString& error);
 
 	static bool Load(CSiteManagerXmlHandler& pHandler);
@@ -99,4 +104,4 @@ protected:
 	static bool LoadPredefined(CSiteManagerXmlHandler& handler);
 };
 
-#endif //__SITEMANAGER_H__
+#endif
