@@ -1,9 +1,9 @@
-#ifndef __SPEEDLIMITS_DIALOG_H__
-#define __SPEEDLIMITS_DIALOG_H__
+#ifndef FILEZILLA_INTERFACE_SPEEDLIMITS_DIALOG_HEADER
+#define FILEZILLA_INTERFACE_SPEEDLIMITS_DIALOG_HEADER
 
 #include "dialogex.h"
 
-class CSpeedLimitsDialog : public wxDialogEx
+class CSpeedLimitsDialog final : public wxDialogEx
 {
 public:
 	CSpeedLimitsDialog() {}
@@ -12,9 +12,9 @@ public:
 	void Run(wxWindow* parent);
 
 protected:
-	DECLARE_EVENT_TABLE();
+	DECLARE_EVENT_TABLE()
 	void OnToggleEnable(wxCommandEvent& event);
 	void OnOK(wxCommandEvent& event);
 };
 
-#endif //__SPEEDLIMITS_DIALOG_H__
+#endif
