@@ -13,6 +13,8 @@
 
 #include <libfilezilla/local_filesys.hpp>
 
+#include <algorithm>
+
 CContextManager CContextManager::m_the_context_manager;
 
 CContextManager::CContextManager()
@@ -143,8 +145,9 @@ void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications
 
 	auto const& handlers = m_handlers[notification];
 	for (auto const& handler : handlers) {
-		if (handler.current_only && pState != GetCurrentContext())
+		if (handler.current_only && pState != GetCurrentContext()) {
 			continue;
+		}
 
 		handler.pHandler->OnStateChange(pState, notification, data, data2);
 	}
@@ -152,16 +155,18 @@ void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications
 
 CState* CContextManager::GetCurrentContext()
 {
-	if (m_current_context == -1)
+	if (m_current_context == -1) {
 		return 0;
+	}
 
 	return m_contexts[m_current_context];
 }
 
 void CContextManager::NotifyAllHandlers(t_statechange_notifications notification, wxString const& data, void const* data2)
 {
-	for (unsigned int i = 0; i < m_contexts.size(); i++)
-		m_contexts[i]->NotifyHandlers(notification, data, data2);
+	for (auto const& context : m_contexts) {
+		context->NotifyHandlers(notification, data, data2);
+	}
 }
 
 void CContextManager::NotifyGlobalHandlers(t_statechange_notifications notification, wxString const& data, void const* data2)
@@ -172,6 +177,18 @@ void CContextManager::NotifyGlobalHandlers(t_statechange_notifications notificat
 	}
 }
 
+void CContextManager::ProcessDirectoryListing(CServer const& server, std::shared_ptr<CDirectoryListing> const& listing, CState const* exempt)
+{
+	for (auto state : m_contexts) {
+		if (state == exempt) {
+			continue;
+		}
+		if (state->GetServer() && state->GetServer().server == server) {
+			state->SetRemoteDir(listing, true);
+		}
+	}
+}
+
 CState::CState(CMainFrame &mainFrame)
 	: m_mainFrame(mainFrame)
 {
@@ -182,9 +199,6 @@ CState::CState(CMainFrame &mainFrame)
 	m_pLocalRecursiveOperation = new CLocalRecursiveOperation(*this);
 	m_pRemoteRecursiveOperation = new CRemoteRecursiveOperation(*this);
 
-	m_sync_browse.is_changing = false;
-	m_sync_browse.compare = false;
-
 	m_localDir.SetPath(std::wstring(1, CLocalPath::path_separator));
 }
 
@@ -198,7 +212,7 @@ CState::~CState()
 
 	// Unregister all handlers
 	for (int i = 0; i < STATECHANGE_MAX; ++i) {
-		wxASSERT(m_handlers[i].empty());
+		wxASSERT(m_handlers[i].handlers.empty());
 	}
 }
 
@@ -208,49 +222,53 @@ CLocalPath CState::GetLocalDir() const
 }
 
 
-bool CState::SetLocalDir(const wxString& dir, wxString *error, bool rememberPreviousSubdir)
+bool CState::SetLocalDir(std::wstring const& dir, std::wstring *error, bool rememberPreviousSubdir)
 {
 	CLocalPath p(m_localDir);
 #ifdef __WXMSW__
-	if (dir == _T("..") && !p.HasParent() && p.HasLogicalParent())
-	{
+	if (dir == _T("..") && !p.HasParent() && p.HasLogicalParent()) {
 		// Parent of C:\ is drive list
-		if (!p.MakeParent())
+		if (!p.MakeParent()) {
 			return false;
+		}
 	}
 	else
 #endif
-	if (!p.ChangePath(dir))
+	if (!p.ChangePath(dir)) {
 		return false;
+	}
 
 	return SetLocalDir(p, error, rememberPreviousSubdir);
 }
 
-bool CState::SetLocalDir(CLocalPath const& dir, wxString *error, bool rememberPreviousSubdir)
+bool CState::SetLocalDir(CLocalPath const& dir, std::wstring *error, bool rememberPreviousSubdir)
 {
-	if (m_sync_browse.is_changing) {
-		wxMessageBoxEx(_T("Cannot change directory, there already is a synchronized browsing operation in progress."), _("Synchronized browsing"));
+	if (m_changeDirFlags.syncbrowse) {
+		wxMessageBoxEx(_("Cannot change directory, there already is a synchronized browsing operation in progress."), _("Synchronized browsing"));
 		return false;
 	}
 
-	if (!dir.Exists(error))
+	if (!dir.Exists(error)) {
 		return false;
+	}
 
 	if (!m_sync_browse.local_root.empty()) {
-		wxASSERT(m_site.m_server);
+		wxASSERT(m_site.server_);
 
 		if (dir != m_sync_browse.local_root && !dir.IsSubdirOf(m_sync_browse.local_root)) {
 			wxString msg = wxString::Format(_("The local directory '%s' is not below the synchronization root (%s).\nDisable synchronized browsing and continue changing the local directory?"),
 					dir.GetPath(),
 					m_sync_browse.local_root.GetPath());
-			if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
+			if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES) {
 				return false;
+			}
 			SetSyncBrowse(false);
 		}
 		else if (!IsRemoteIdle(true)) {
 			wxString msg(_("A remote operation is in progress and synchronized browsing is enabled.\nDisable synchronized browsing and continue changing the local directory?"));
-			if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
+			if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES) {
 				return false;
+			}
 			SetSyncBrowse(false);
 		}
 		else {
@@ -263,8 +281,8 @@ bool CState::SetLocalDir(CLocalPath const& dir, wxString *error, bool rememberPr
 				return false;
 			}
 
-			m_sync_browse.is_changing = true;
-			m_sync_browse.compare = m_pComparisonManager->IsComparing();
+			m_changeDirFlags.syncbrowse = true;
+			m_changeDirFlags.compare = m_pComparisonManager->IsComparing();
 			m_sync_browse.target_path = remote_path;
 			CListCommand *pCommand = new CListCommand(remote_path);
 			m_pCommandQueue->ProcessCommand(pCommand);
@@ -283,8 +301,9 @@ bool CState::SetLocalDir(CLocalPath const& dir, wxString *error, bool rememberPr
 #endif
 			m_previouslyVisitedLocalSubdir = m_localDir.GetLastSegment();
 	}
-	else
+	else {
 		m_previouslyVisitedLocalSubdir = _T("");
+	}
 
 
 	m_localDir = dir;
@@ -299,9 +318,11 @@ bool CState::SetLocalDir(CLocalPath const& dir, wxString *error, bool rememberPr
 bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryListing, bool modified)
 {
 	if (!pDirectoryListing) {
+		m_changeDirFlags.compare = false;
 		SetSyncBrowse(false);
-		if (modified)
+		if (modified) {
 			return false;
+		}
 
 		if (m_pDirectoryListing) {
 			m_pDirectoryListing = 0;
@@ -315,9 +336,12 @@ bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryLi
 
 	if (pDirectoryListing && m_pDirectoryListing &&
 		pDirectoryListing->path == m_pDirectoryListing->path.GetParent())
+	{
 		m_previouslyVisitedRemoteSubdir = m_pDirectoryListing->path.GetLastSegment();
-	else
+	}
+	else {
 		m_previouslyVisitedRemoteSubdir = _T("");
+	}
 
 	if (modified) {
 		if (!m_pDirectoryListing || m_pDirectoryListing->path != pDirectoryListing->path) {
@@ -325,8 +349,9 @@ bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryLi
 			return true;
 		}
 	}
-	else
+	else {
 		m_last_path = pDirectoryListing->path;
+	}
 
 	if (m_pDirectoryListing && m_pDirectoryListing->path == pDirectoryListing->path &&
 		pDirectoryListing->failed())
@@ -339,8 +364,13 @@ bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryLi
 
 	NotifyHandlers(STATECHANGE_REMOTE_DIR, wxString(), &modified);
 
-	if (m_sync_browse.is_changing && !modified) {
-		m_sync_browse.is_changing = false;
+	bool compare = m_changeDirFlags.compare;
+	if (!modified) {
+		m_changeDirFlags.compare = false;
+	}
+
+	if (m_changeDirFlags.syncbrowse && !modified) {
+		m_changeDirFlags.syncbrowse = false;
 		if (m_pDirectoryListing->path != m_sync_browse.remote_root && !m_pDirectoryListing->path.IsSubdirOf(m_sync_browse.remote_root, false)) {
 			SetSyncBrowse(false);
 			wxString msg = wxString::Format(_("Current remote directory (%s) is not below the synchronization root (%s).\nSynchronized browsing has been disabled."),
@@ -355,27 +385,31 @@ bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryLi
 				wxString msg = wxString::Format(_("Could not obtain corresponding local directory for the remote directory '%s'.\nSynchronized browsing has been disabled."),
 					m_pDirectoryListing->path.GetPath());
 				wxMessageBoxEx(msg, _("Synchronized browsing"));
-				return true;
+				compare = false;
 			}
+			else {
+				std::wstring error;
+				if (!local_path.Exists(&error)) {
+					SetSyncBrowse(false);
+					wxString msg = error + _T("\n") + _("Synchronized browsing has been disabled.");
+					wxMessageBoxEx(msg, _("Synchronized browsing"));
+					compare = false;
+				}
+				else {
+					m_localDir = local_path;
 
-			wxString error;
-			if (!local_path.Exists(&error)) {
-				SetSyncBrowse(false);
-				wxString msg = error + _T("\n") + _("Synchronized browsing has been disabled.");
-				wxMessageBoxEx(msg, _("Synchronized browsing"));
-				return true;
+					COptions::Get()->SetOption(OPTION_LASTLOCALDIR, m_localDir.GetPath());
+
+					NotifyHandlers(STATECHANGE_LOCAL_DIR);
+				}
 			}
-
-			m_localDir = local_path;
-
-			COptions::Get()->SetOption(OPTION_LASTLOCALDIR, m_localDir.GetPath());
-
-			NotifyHandlers(STATECHANGE_LOCAL_DIR);
-
-			if (m_sync_browse.compare)
-				m_pComparisonManager->CompareListings();
 		}
 	}
+
+	if (compare && !m_pComparisonManager->IsComparing()) {
+		m_pComparisonManager->CompareListings();
+	}
+
 	return true;
 }
 
@@ -397,23 +431,25 @@ void CState::RefreshLocal()
 	NotifyHandlers(STATECHANGE_LOCAL_DIR);
 }
 
-void CState::RefreshLocalFile(wxString file)
+void CState::RefreshLocalFile(std::wstring const& file)
 {
-	wxString file_name;
+	std::wstring file_name;
 	CLocalPath path(file, &file_name);
-	if (path.empty())
+	if (path.empty()) {
 		return;
+	}
 
-	if (file_name.empty())
-	{
-		if (!path.HasParent())
+	if (file_name.empty()) {
+		if (!path.HasParent()) {
 			return;
+		}
 		path.MakeParent(&file_name);
 		wxASSERT(!file_name.empty());
 	}
 
-	if (path != m_localDir)
+	if (path != m_localDir) {
 		return;
+	}
 
 	NotifyHandlers(STATECHANGE_LOCAL_REFRESH_FILE, file_name);
 }
@@ -440,10 +476,10 @@ void CState::LocalDirCreated(const CLocalPath& path)
 
 void CState::SetSite(Site const& site, CServerPath const& path)
 {
-	if (m_site.m_server) {
-		if (site.m_server && site.m_server == m_site.m_server &&
-			site.m_server.GetName() == m_site.m_server.GetName() &&
-			site.m_server.MaximumMultipleConnections() == m_site.m_server.MaximumMultipleConnections())
+	if (m_site.server_) {
+		if (site.server_ && site.server_ == m_site.server_ &&
+			site.server_.server.GetName() == m_site.server_.server.GetName() &&
+			site.server_.server.MaximumMultipleConnections() == m_site.server_.server.MaximumMultipleConnections())
 		{
 			// Nothing changes
 			return;
@@ -453,16 +489,16 @@ void CState::SetSite(Site const& site, CServerPath const& path)
 		m_pCertificate.reset();
 		m_pSftpEncryptionInfo.reset();
 	}
-	if (site.m_server) {
+	if (site.server_) {
 		if (!path.empty()) {
 			m_last_path = path;
 		}
-		else if (m_last_site.m_server != site.m_server) {
+		else if (m_last_site.server_ != site.server_) {
 			m_last_path.clear();
 		}
 		m_last_site = site;
 	}
-	
+
 	m_site = site;
 
 	UpdateTitle();
@@ -477,9 +513,9 @@ Site const& CState::GetSite() const
 	return m_site;
 }
 
-CServer const* CState::GetServer() const
+ServerWithCredentials const& CState::GetServer() const
 {
-	return m_site.m_server ? &m_site.m_server : 0;
+	return m_site.server_;
 }
 
 wxString CState::GetTitle() const
@@ -487,9 +523,9 @@ wxString CState::GetTitle() const
 	return m_title;
 }
 
-bool CState::Connect(Site const& site, CServerPath const& path)
+bool CState::Connect(Site const& site, CServerPath const& path, bool compare)
 {
-	if (!site.m_server) {
+	if (!site.server_) {
 		return false;
 	}
 	if (!m_pEngine) {
@@ -501,24 +537,29 @@ bool CState::Connect(Site const& site, CServerPath const& path)
 	m_pRemoteRecursiveOperation->StopRecursiveOperation();
 	SetSyncBrowse(false);
 
-	m_pCommandQueue->ProcessCommand(new CConnectCommand(site.m_server));
+	m_pCommandQueue->ProcessCommand(new CConnectCommand(site.server_.server, site.server_.credentials));
 	m_pCommandQueue->ProcessCommand(new CListCommand(path, _T(""), LIST_FLAG_FALLBACK_CURRENT));
 
 	SetSite(site, path);
 
+	m_changeDirFlags.compare = compare;
+
 	return true;
 }
 
 bool CState::Disconnect()
 {
-	if (!m_pEngine)
+	if (!m_pEngine) {
 		return false;
+	}
 
-	if (!IsRemoteConnected())
+	if (!IsRemoteConnected()) {
 		return true;
+	}
 
-	if (!IsRemoteIdle())
+	if (!IsRemoteIdle()) {
 		return false;
+	}
 
 	SetSite(Site());
 	m_pCommandQueue->ProcessCommand(new CDisconnectCommand());
@@ -529,8 +570,9 @@ bool CState::Disconnect()
 bool CState::CreateEngine()
 {
 	wxASSERT(!m_pEngine);
-	if (m_pEngine)
+	if (m_pEngine) {
 		return true;
+	}
 
 	m_pEngine = new CFileZillaEngine(m_mainFrame.GetEngineContext(), m_mainFrame);
 
@@ -551,28 +593,30 @@ void CState::RegisterHandler(CStateEventHandler* pHandler, t_statechange_notific
 {
 	wxASSERT(pHandler);
 	wxASSERT(&pHandler->m_state == this);
-	if (!pHandler || &pHandler->m_state != this)
+	if (!pHandler || &pHandler->m_state != this) {
 		return;
+	}
 	wxASSERT(notification != STATECHANGE_MAX && notification != STATECHANGE_NONE);
 	wxASSERT(pHandler != insertBefore);
 
-	
-	auto &handlers = m_handlers[notification];
-	auto insertionPoint = handlers.end();
 
-	for (auto it = handlers.begin(); it != handlers.end(); ++it) {
-		if (it->pHandler == insertBefore) {
+	auto &handlers = m_handlers[notification];
+
+	wxASSERT(!insertBefore || !handlers.inNotify_);
+
+	auto insertionPoint = handlers.handlers.end();
+
+	for (auto it = handlers.handlers.begin(); it != handlers.handlers.end(); ++it) {
+		if (*it == insertBefore) {
 			insertionPoint = it;
 		}
-		if (it->pHandler == pHandler) {
-			wxASSERT(insertionPoint == handlers.end());
+		if (*it == pHandler) {
+			wxASSERT(insertionPoint == handlers.handlers.end());
 			return;
 		}
 	}
 
-	t_handler handler;
-	handler.pHandler = pHandler;
-	handlers.insert(insertionPoint, handler);
+	handlers.handlers.insert(insertionPoint, pHandler);
 }
 
 void CState::UnregisterHandler(CStateEventHandler* pHandler, t_statechange_notifications notification)
@@ -583,9 +627,15 @@ void CState::UnregisterHandler(CStateEventHandler* pHandler, t_statechange_notif
 	if (notification == STATECHANGE_NONE) {
 		for (int i = 0; i < STATECHANGE_MAX; ++i) {
 			auto &handlers = m_handlers[i];
-			for (auto iter = handlers.begin(); iter != handlers.end(); ++iter) {
-				if (iter->pHandler == pHandler) {
-					handlers.erase(iter);
+			for (auto iter = handlers.handlers.begin(); iter != handlers.handlers.end(); ++iter) {
+				if (*iter == pHandler) {
+					if (handlers.inNotify_) {
+						handlers.compact_ = true;
+						*iter = 0;
+					}
+					else {
+						handlers.handlers.erase(iter);
+					}
 					break;
 				}
 			}
@@ -593,9 +643,15 @@ void CState::UnregisterHandler(CStateEventHandler* pHandler, t_statechange_notif
 	}
 	else {
 		auto &handlers = m_handlers[notification];
-		for (auto iter = handlers.begin(); iter != handlers.end(); ++iter) {
-			if (iter->pHandler == pHandler) {
-				handlers.erase(iter);
+		for (auto iter = handlers.handlers.begin(); iter != handlers.handlers.end(); ++iter) {
+			if (*iter == pHandler) {
+				if (handlers.inNotify_) {
+					handlers.compact_ = true;
+					*iter = 0;
+				}
+				else {
+					handlers.handlers.erase(iter);
+				}
 				return;
 			}
 		}
@@ -606,11 +662,25 @@ void CState::NotifyHandlers(t_statechange_notifications notification, const wxSt
 {
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
 
-	auto const& handlers = m_handlers[notification];
-	for (auto const& handler : handlers) {
-		handler.pHandler->OnStateChange(notification, data, data2);
+	auto & handlers = m_handlers[notification];
+
+	wxASSERT(!handlers.inNotify_);
+
+	handlers.inNotify_ = true;
+	// Can't use iterators as inserting a handler might invalidate them
+	for (size_t i = 0; i < handlers.handlers.size(); ++i) {
+		if (handlers.handlers[i]) {
+			handlers.handlers[i]->OnStateChange(notification, data, data2);
+		}
 	}
 
+	if (handlers.compact_) {
+		handlers.handlers.erase(std::remove(handlers.handlers.begin(), handlers.handlers.end(), nullptr), handlers.handlers.end());
+		handlers.compact_ = false;
+	}
+
+	handlers.inNotify_ = false;
+
 	CContextManager::Get()->NotifyHandlers(this, notification, data, data2);
 }
 
@@ -631,22 +701,24 @@ CGlobalStateEventHandler::~CGlobalStateEventHandler()
 
 void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const wxString& subdir, bool queueOnly)
 {
-	if (!m_site.m_server || !m_pDirectoryListing) {
+	if (!m_site.server_ || !m_pDirectoryListing) {
 		return;
 	}
 
 	CServerPath path = m_pDirectoryListing->path;
-	if (subdir == _T("..") && path.HasParent())
+	if (subdir == _T("..") && path.HasParent()) {
 		path = path.GetParent();
-	else if (!subdir.empty())
-		path.AddSegment(subdir);
+	}
+	else if (!subdir.empty()) {
+		path.AddSegment(subdir.ToStdWstring());
+	}
 
 	UploadDroppedFiles(pFileDataObject, path, queueOnly);
 }
 
 void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const CServerPath& path, bool queueOnly)
 {
-	if (!m_site.m_server) {
+	if (!m_site.server_) {
 		return;
 	}
 
@@ -660,22 +732,22 @@ void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 		return;
 	}
 
-	const wxArrayString& files = pFileDataObject->GetFilenames();
+	wxArrayString const& files = pFileDataObject->GetFilenames();
 
 	for (unsigned int i = 0; i < files.Count(); ++i) {
 		int64_t size;
 		bool is_link;
 		fz::local_filesys::type type = fz::local_filesys::get_file_info(fz::to_native(files[i]), is_link, &size, 0, 0);
 		if (type == fz::local_filesys::file) {
-			wxString localFile;
-			const CLocalPath localPath(files[i], &localFile);
-			m_mainFrame.GetQueue()->QueueFile(queueOnly, false, localFile, wxEmptyString, localPath, path, m_site.m_server, size);
+			std::wstring localFile;
+			const CLocalPath localPath(files[i].ToStdWstring(), &localFile);
+			m_mainFrame.GetQueue()->QueueFile(queueOnly, false, localFile, wxEmptyString, localPath, path, m_site.server_, size);
 			m_mainFrame.GetQueue()->QueueFile_Finish(!queueOnly);
 		}
 		else if (type == fz::local_filesys::dir) {
-			CLocalPath localPath(files[i]);
+			CLocalPath localPath(files[i].ToStdWstring());
 			if (localPath.HasParent()) {
-				
+
 				CServerPath remotePath = path;
 				if (!remotePath.ChangePath(localPath.GetLastSegment())) {
 					continue;
@@ -695,8 +767,9 @@ void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const CLocalPath& path, bool copy)
 {
 	const wxArrayString &files = pFileDataObject->GetFilenames();
-	if (!files.Count())
+	if (!files.Count()) {
 		return;
+	}
 
 #ifdef __WXMSW__
 	int len = 1;
@@ -731,45 +804,53 @@ void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 #else
 	wxString error;
 	for (unsigned int i = 0; i < files.Count(); ++i) {
-		wxString const& file(files[i]);
+		std::wstring const file(files[i].ToStdWstring());
 
 		int64_t size;
 		bool is_link;
 		fz::local_filesys::type type = fz::local_filesys::get_file_info(fz::to_native(file), is_link, &size, 0, 0);
 		if (type == fz::local_filesys::file) {
-			wxString name;
+			std::wstring name;
 			CLocalPath sourcePath(file, &name);
-			if (name.empty())
+			if (name.empty()) {
 				continue;
+			}
 			wxString target = path.GetPath() + name;
-			if (file == target)
+			if (file == target) {
 				continue;
+			}
 
-			if (copy)
+			if (copy) {
 				wxCopyFile(file, target);
-			else
+			}
+			else {
 				wxRenameFile(file, target);
+			}
 		}
 		else if (type == fz::local_filesys::dir) {
 			CLocalPath sourcePath(file);
-			if (sourcePath == path || sourcePath.GetParent() == path)
+			if (sourcePath == path || sourcePath.GetParent() == path) {
 				continue;
+			}
 			if (sourcePath.IsParentOf(path)) {
 				error = _("A directory cannot be dragged into one of its subdirectories.");
 				continue;
 			}
 
-			if (copy)
+			if (copy) {
 				RecursiveCopy(sourcePath, path);
+			}
 			else {
-				if (!sourcePath.HasParent())
+				if (!sourcePath.HasParent()) {
 					continue;
+				}
 				wxRenameFile(file, path.GetPath() + sourcePath.GetLastSegment());
 			}
 		}
 	}
-	if (!error.empty())
+	if (!error.empty()) {
 		wxMessageBoxEx(error, _("Could not complete operation"));
+	}
 #endif
 
 	RefreshLocal();
@@ -777,34 +858,42 @@ void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 
 bool CState::RecursiveCopy(CLocalPath source, const CLocalPath& target)
 {
-	if (source.empty() || target.empty())
+	if (source.empty() || target.empty()) {
 		return false;
+	}
 
-	if (source == target)
+	if (source == target) {
 		return false;
+	}
 
-	if (source.IsParentOf(target))
+	if (source.IsParentOf(target)) {
 		return false;
+	}
 
-	if (!source.HasParent())
+	if (!source.HasParent()) {
 		return false;
+	}
 
-	wxString last_segment;
-	if (!source.MakeParent(&last_segment))
+	std::wstring last_segment;
+	if (!source.MakeParent(&last_segment)) {
 		return false;
+	}
 
-	std::list<wxString> dirsToVisit;
+	std::list<std::wstring> dirsToVisit;
 	dirsToVisit.push_back(last_segment + CLocalPath::path_separator);
 
+	fz::native_string const nsource = fz::to_native(source.GetPath());
+
 	// Process any subdirs which still have to be visited
 	while (!dirsToVisit.empty()) {
-		wxString dirname = dirsToVisit.front();
+		std::wstring dirname = dirsToVisit.front();
 		dirsToVisit.pop_front();
 		wxMkdir(target.GetPath() + dirname);
 
 		fz::local_filesys fs;
-		if (!fs.begin_find_files(fz::to_native(source.GetPath() + dirname), false))
+		if (!fs.begin_find_files(nsource + fz::to_native(dirname), false)) {
 			continue;
+		}
 
 		bool is_dir, is_link;
 		fz::native_string file;
@@ -815,14 +904,16 @@ bool CState::RecursiveCopy(CLocalPath source, const CLocalPath& target)
 			}
 
 			if (is_dir) {
-				if (is_link)
+				if (is_link) {
 					continue;
+				}
 
-				const wxString subDir = dirname + file + CLocalPath::path_separator;
+				std::wstring const subDir = dirname + fz::to_wstring(file) + CLocalPath::path_separator;
 				dirsToVisit.push_back(subDir);
 			}
-			else
+			else {
 				wxCopyFile(source.GetPath() + dirname + file, target.GetPath() + dirname + file);
+			}
 		}
 	}
 
@@ -834,32 +925,38 @@ bool CState::DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, co
 	bool hasDirs = false;
 	bool hasFiles = false;
 	const std::list<CRemoteDataObject::t_fileInfo>& files = pRemoteDataObject->GetFiles();
-	for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
-		if (iter->dir)
+	for (auto const& fileInfo : files) {
+		if (fileInfo.dir) {
 			hasDirs = true;
-		else
+		}
+		else {
 			hasFiles = true;
+		}
 	}
 
 	if (hasDirs) {
-		if (!IsRemoteConnected() || !IsRemoteIdle())
+		if (!IsRemoteConnected() || !IsRemoteIdle()) {
 			return false;
+		}
 	}
 
-	if (hasFiles)
+	if (hasFiles) {
 		m_mainFrame.GetQueue()->QueueFiles(queueOnly, path, *pRemoteDataObject);
+	}
 
-	if (!hasDirs)
+	if (!hasDirs) {
 		return true;
+	}
 
 	recursion_root root(pRemoteDataObject->GetServerPath(), false);
-	for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
-		if (!iter->dir)
+	for (auto const& fileInfo : files) {
+		if (!fileInfo.dir) {
 			continue;
+		}
 
 		CLocalPath newPath(path);
-		newPath.AddSegment(CQueueView::ReplaceInvalidCharacters(iter->name));
-		root.add_dir_to_visit(pRemoteDataObject->GetServerPath(), iter->name, newPath, iter->link);
+		newPath.AddSegment(CQueueView::ReplaceInvalidCharacters(fileInfo.name));
+		root.add_dir_to_visit(pRemoteDataObject->GetServerPath(), fileInfo.name, newPath, fileInfo.link);
 	}
 
 	if (!root.empty()) {
@@ -874,10 +971,11 @@ bool CState::DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, co
 
 bool CState::IsRemoteConnected() const
 {
-	if (!m_pEngine)
+	if (!m_pEngine) {
 		return false;
+	}
 
-	return static_cast<bool>(m_site.m_server);
+	return static_cast<bool>(m_site.server_);
 }
 
 bool CState::IsRemoteIdle(bool ignore_recursive) const
@@ -901,7 +999,10 @@ bool CState::IsLocalIdle(bool ignore_recursive) const
 
 void CState::ListingFailed(int)
 {
-	if (m_sync_browse.is_changing && !m_sync_browse.target_path.empty()) {
+	bool const compare = m_changeDirFlags.compare;
+	m_changeDirFlags.compare = false;
+
+	if (m_changeDirFlags.syncbrowse && !m_sync_browse.target_path.empty()) {
 		wxDialogEx dlg;
 		if (!dlg.Load(0, _T("ID_SYNCBROWSE_NONEXISTING"))) {
 			SetSyncBrowse(false);
@@ -917,6 +1018,7 @@ void CState::ListingFailed(int)
 				m_pCommandQueue->ProcessCommand(new CMkdirCommand(m_sync_browse.target_path));
 				m_pCommandQueue->ProcessCommand(new CListCommand(m_sync_browse.target_path));
 				m_sync_browse.target_path.clear();
+				m_changeDirFlags.compare = compare;
 				return;
 			}
 			else {
@@ -925,22 +1027,25 @@ void CState::ListingFailed(int)
 				SetLocalDir(local);
 			}
 		}
+		else {
+			m_changeDirFlags.syncbrowse = false;
+		}
 	}
-
-	m_sync_browse.is_changing = false;
 }
 
 void CState::LinkIsNotDir(const CServerPath& path, const wxString& subdir)
 {
-	m_sync_browse.is_changing = false;
+	m_changeDirFlags.compare = false;
+	m_changeDirFlags.syncbrowse = false;
 
 	NotifyHandlers(STATECHANGE_REMOTE_LINKNOTDIR, subdir, &path);
 }
 
-bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=_T("")*/, int flags /*=0*/, bool ignore_busy /*=false*/)
+bool CState::ChangeRemoteDir(CServerPath const& path, std::wstring const& subdir, int flags, bool ignore_busy, bool compare)
 {
-	if (!m_site.m_server || !m_pCommandQueue)
+	if (!m_site.server_ || !m_pCommandQueue) {
 		return false;
+	}
 
 	if (!m_sync_browse.local_root.empty()) {
 		CServerPath p(path);
@@ -954,8 +1059,9 @@ bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=
 			wxString msg = wxString::Format(_("The remote directory '%s' is not below the synchronization root (%s).\nDisable synchronized browsing and continue changing the remote directory?"),
 					p.GetPath(),
 					m_sync_browse.remote_root.GetPath());
-			if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
+			if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES) {
 				return false;
+			}
 			SetSyncBrowse(false);
 		}
 		else if (!IsRemoteIdle(true) && !ignore_busy) {
@@ -964,13 +1070,14 @@ bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=
 			return false;
 		}
 		else {
-			wxString error;
+			std::wstring error;
 			CLocalPath local_path = GetSynchronizedDirectory(p);
 			if (local_path.empty()) {
 				wxString msg = wxString::Format(_("Could not obtain corresponding local directory for the remote directory '%s'.\nDisable synchronized browsing and continue changing the remote directory?"),
 					p.GetPath());
-				if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
+				if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES) {
 					return false;
+				}
 				SetSyncBrowse(false);
 			}
 			else if (!local_path.Exists(&error)) {
@@ -998,8 +1105,8 @@ bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=
 						wxMessageBox(wxString::Format(_("The local directory '%s' could not be created."), local_path.GetPath()), _("Synchronized browsing"), wxICON_EXCLAMATION);
 						return false;
 					}
-					m_sync_browse.is_changing = true;
-					m_sync_browse.compare = m_pComparisonManager->IsComparing();
+					m_changeDirFlags.syncbrowse = true;
+					m_changeDirFlags.compare = m_pComparisonManager->IsComparing();
 					m_sync_browse.target_path.clear();
 				}
 				else {
@@ -1007,8 +1114,8 @@ bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=
 				}
 			}
 			else {
-				m_sync_browse.is_changing = true;
-				m_sync_browse.compare = m_pComparisonManager->IsComparing();
+				m_changeDirFlags.syncbrowse = true;
+				m_changeDirFlags.compare = m_pComparisonManager->IsComparing();
 				m_sync_browse.target_path.clear();
 			}
 		}
@@ -1017,19 +1124,24 @@ bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=
 	CListCommand *pCommand = new CListCommand(path, subdir, flags);
 	m_pCommandQueue->ProcessCommand(pCommand);
 
+	if (compare) {
+		m_changeDirFlags.compare = true;
+	}
+
 	return true;
 }
 
-bool CState::SetSyncBrowse(bool enable, const CServerPath& assumed_remote_root /*=CServerPath()*/)
+bool CState::SetSyncBrowse(bool enable, CServerPath const& assumed_remote_root)
 {
-	if (enable != m_sync_browse.local_root.empty())
+	if (enable != m_sync_browse.local_root.empty()) {
 		return enable;
+	}
 
 	if (!enable) {
 		wxASSERT(assumed_remote_root.empty());
 		m_sync_browse.local_root.clear();
 		m_sync_browse.remote_root.clear();
-		m_sync_browse.is_changing = false;
+		m_changeDirFlags.syncbrowse = false;
 
 		NotifyHandlers(STATECHANGE_SYNC_BROWSE);
 		return false;
@@ -1040,15 +1152,15 @@ bool CState::SetSyncBrowse(bool enable, const CServerPath& assumed_remote_root /
 		return false;
 	}
 
-	m_sync_browse.is_changing = false;
+	m_changeDirFlags.syncbrowse = false;
 	m_sync_browse.local_root = m_localDir;
 
-	if (assumed_remote_root.empty())
+	if (assumed_remote_root.empty()) {
 		m_sync_browse.remote_root = m_pDirectoryListing->path;
+	}
 	else {
 		m_sync_browse.remote_root = assumed_remote_root;
-		m_sync_browse.is_changing = true;
-		m_sync_browse.compare = false;
+		m_changeDirFlags.syncbrowse = true;
 	}
 
 	while (m_sync_browse.local_root.HasParent() && m_sync_browse.remote_root.HasParent() &&
@@ -1064,17 +1176,19 @@ bool CState::SetSyncBrowse(bool enable, const CServerPath& assumed_remote_root /
 
 CLocalPath CState::GetSynchronizedDirectory(CServerPath remote_path)
 {
-	std::list<wxString> segments;
+	std::list<std::wstring> segments;
 	while (remote_path.HasParent() && remote_path != m_sync_browse.remote_root) {
 		segments.push_front(remote_path.GetLastSegment());
 		remote_path = remote_path.GetParent();
 	}
-	if (remote_path != m_sync_browse.remote_root)
+	if (remote_path != m_sync_browse.remote_root) {
 		return CLocalPath();
+	}
 
 	CLocalPath local_path = m_sync_browse.local_root;
-	for (std::list<wxString>::const_iterator iter = segments.begin(); iter != segments.end(); ++iter)
-		local_path.AddSegment(*iter);
+	for (auto const& segment : segments) {
+		local_path.AddSegment(segment);
+	}
 
 	return local_path;
 }
@@ -1082,29 +1196,33 @@ CLocalPath CState::GetSynchronizedDirectory(CServerPath remote_path)
 
 CServerPath CState::GetSynchronizedDirectory(CLocalPath local_path)
 {
-	std::list<wxString> segments;
+	std::list<std::wstring> segments;
 	while (local_path.HasParent() && local_path != m_sync_browse.local_root) {
-		wxString last;
+		std::wstring last;
 		local_path.MakeParent(&last);
 		segments.push_front(last);
 	}
-	if (local_path != m_sync_browse.local_root)
+	if (local_path != m_sync_browse.local_root) {
 		return CServerPath();
+	}
 
 	CServerPath remote_path = m_sync_browse.remote_root;
-	for (std::list<wxString>::const_iterator iter = segments.begin(); iter != segments.end(); ++iter)
-		remote_path.AddSegment(*iter);
+	for (auto const& segment : segments) {
+		remote_path.AddSegment(segment);
+	}
 
 	return remote_path;
 }
 
 bool CState::RefreshRemote()
 {
-	if (!m_pCommandQueue)
+	if (!m_pCommandQueue) {
 		return false;
+	}
 
-	if (!IsRemoteConnected() || !IsRemoteIdle(true))
+	if (!IsRemoteConnected() || !IsRemoteIdle(true)) {
 		return false;
+	}
 
 	return ChangeRemoteDir(GetRemotePath(), _T(""), LIST_FLAG_REFRESH);
 }
@@ -1137,21 +1255,21 @@ void CState::SetSecurityInfo(CSftpEncryptionNotification const& info)
 
 void CState::UpdateSite(wxString const& oldPath, Site const& newSite)
 {
-	if (newSite.m_path.empty() || !newSite.m_server) {
+	if (newSite.m_path.empty() || !newSite.server_) {
 		return;
 	}
 
 	bool changed = false;
-	if (m_site.m_server && m_site != newSite) {
-		if (m_site.m_path == oldPath && m_site.m_server.EqualsNoPass(newSite.m_server)) {
+	if (m_site.server_ && m_site != newSite) {
+		if (m_site.m_path == oldPath && m_site.server_ == newSite.server_) {
 			m_site = newSite;
 			changed = true;
 		}
 	}
-	if (m_last_site.m_server && m_last_site != newSite) {
-		if (m_last_site.m_path == oldPath && m_last_site.m_server.EqualsNoPass(newSite.m_server)) {
+	if (m_last_site.server_ && m_last_site != newSite) {
+		if (m_last_site.m_path == oldPath && m_last_site.server_ == newSite.server_) {
 			m_last_site = newSite;
-			if (!m_site.m_server) {
+			if (!m_site.server_) {
 				// Active site has precedence over historic data
 				changed = true;
 			}
@@ -1165,13 +1283,13 @@ void CState::UpdateSite(wxString const& oldPath, Site const& newSite)
 
 void CState::UpdateTitle()
 {
-	if (m_site.m_server) {
-		wxString const& name = m_site.m_server.GetName();
+	if (m_site.server_) {
+		wxString const& name = m_site.server_.server.GetName();
 		m_title.clear();
 		if (!name.empty()) {
 			m_title = name + _T(" - ");
 		}
-		m_title += m_site.m_server.Format(ServerFormat::with_user_and_optional_port);
+		m_title += m_site.server_.Format(ServerFormat::with_user_and_optional_port);
 	}
 	else {
 		m_title = _("Not connected");
