@@ -1,5 +1,5 @@
-#ifndef __STATE_H__
-#define __STATE_H__
+#ifndef FILEZILLA_INTERFACE_STATE_HEADER
+#define FILEZILLA_INTERFACE_STATE_HEADER
 
 #include "local_path.h"
 #include "sitemanager.h"
@@ -40,6 +40,9 @@ enum t_statechange_notifications
 	STATECHANGE_CHANGEDCONTEXT, /* Currently active context changed */
 	STATECHANGE_REMOVECONTEXT, /* Right before deleting a context */
 	STATECHANGE_GLOBALBOOKMARKS,
+	STATECHANGE_REWRITE_CREDENTIALS,
+
+	STATECHANGE_QUITNOW,
 
 	STATECHANGE_MAX
 };
@@ -81,6 +84,8 @@ public:
 
 	void SetCurrentContext(CState* pState);
 
+	void ProcessDirectoryListing(CServer const& server, std::shared_ptr<CDirectoryListing> const& listing, CState const* exempt);
+
 protected:
 	CContextManager();
 
@@ -125,7 +130,7 @@ public:
 	const CServerPath GetRemotePath() const;
 
 	Site const& GetSite() const;
-	CServer const* GetServer() const;
+	ServerWithCredentials const& GetServer() const;
 	wxString GetTitle() const;
 
 	void RefreshLocal();
@@ -208,11 +213,14 @@ protected:
 
 	CComparisonManager* m_pComparisonManager;
 
-	struct t_handler
+	struct t_handlersForNotification
 	{
-		CStateEventHandler* pHandler;
+		std::vector<CStateEventHandler*> handlers;
+		bool compact_{};
+		bool inNotify_{};
 	};
-	std::vector<t_handler> m_handlers[STATECHANGE_MAX];
+
+	t_handlersForNotification m_handlers[STATECHANGE_MAX];
 
 	CLocalPath GetSynchronizedDirectory(CServerPath remote_path);
 	CServerPath GetSynchronizedDirectory(CLocalPath local_path);
