@@ -72,7 +72,7 @@ Impl& GetImpl()
 wxString CTimeFormat::Format(fz::datetime const& time)
 {
 	wxString ret;
-	if (time.empty()) {
+	if (!time.empty()) {
 		if (time.get_accuracy() > fz::datetime::days) {
 			ret = FormatDateTime(time);
 		}
@@ -95,4 +95,4 @@ wxString CTimeFormat::FormatDate(fz::datetime const& time)
 	Impl& impl = GetImpl();
 
 	return time.format(impl.m_dateFormat.ToStdWstring(), fz::datetime::local);
-}
\ No newline at end of file
+}
