@@ -286,14 +286,14 @@ int CUpdater::ContinueDownload()
 
 bool CUpdater::CreateConnectCommand(wxString const& url)
 {
-	CServer s;
+	ServerWithCredentials s;
 	CServerPath path;
 	std::wstring error;
-	if (!s.ParseUrl(url.ToStdWstring(), 0, std::wstring(), std::wstring(), error, path) || (s.GetProtocol() != HTTP && s.GetProtocol() != HTTPS)) {
+	if (!s.ParseUrl(url.ToStdWstring(), 0, std::wstring(), std::wstring(), error, path) || (s.server.GetProtocol() != HTTP && s.server.GetProtocol() != HTTPS)) {
 		return false;
 	}
 
-	pending_commands_.emplace_back(new CConnectCommand(s));
+	pending_commands_.emplace_back(new CConnectCommand(s.server, s.credentials));
 	return true;
 }
 
@@ -301,10 +301,10 @@ bool CUpdater::CreateTransferCommand(wxString const& url, std::wstring const& lo
 {
 	CFileTransferCommand::t_transferSettings transferSettings;
 
-	CServer s;
+	ServerWithCredentials s;
 	CServerPath path;
 	std::wstring error;
-	if (!s.ParseUrl(url.ToStdWstring(), 0, std::wstring(), std::wstring(), error, path) || (s.GetProtocol() != HTTP && s.GetProtocol() != HTTPS)) {
+	if (!s.ParseUrl(url.ToStdWstring(), 0, std::wstring(), std::wstring(), error, path) || (s.server.GetProtocol() != HTTP && s.server.GetProtocol() != HTTPS)) {
 		return false;
 	}
 	std::wstring file = path.GetLastSegment();
@@ -321,8 +321,9 @@ void CUpdater::OnEngineEvent(CFileZillaEngine* engine)
 
 void CUpdater::DoOnEngineEvent(CFileZillaEngine* engine)
 {
-	if (!engine_ || engine_ != engine)
+	if (!engine_ || engine_ != engine) {
 		return;
+	}
 
 	std::unique_ptr<CNotification> notification;
 	while( (notification = engine_->GetNextNotification()) ) {
@@ -530,18 +531,18 @@ wxString CUpdater::GetLocalFile(build const& b, bool allow_existing)
 
 void CUpdater::ProcessData(CDataNotification& dataNotification)
 {
-	if( state_ != UpdaterState::checking ) {
+	if (state_ != UpdaterState::checking) {
 		return;
 	}
 
 	int len;
 	char* data = dataNotification.Detach(len);
 
-	if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
+	if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
 		log_ += wxString::Format(_T("ProcessData %d\n"), len);
 	}
 
-	if( raw_version_information_.size() + len > 0x40000) {
+	if (raw_version_information_.size() + len > 0x40000) {
 		log_ += _("Received version information is too large");
 		engine_->Cancel();
 		SetState(UpdaterState::failed);
@@ -557,7 +558,7 @@ void CUpdater::ProcessData(CDataNotification& dataNotification)
 		}
 	}
 
-	if( state_ == UpdaterState::checking ) {
+	if (state_ == UpdaterState::checking) {
 		raw_version_information_ += wxString(data, wxConvUTF8, len);
 	}
 	delete [] data;
