@@ -3,6 +3,8 @@
 
 #if FZ_MANUALUPDATECHECK
 
+#include "notification.h"
+
 #include <wx/timer.h>
 
 struct build
@@ -27,7 +29,9 @@ struct version_information
 
 	build available_;
 
-	wxString changelog;
+	wxString changelog_;
+
+	wxString resources_;
 };
 
 enum class UpdaterState
@@ -47,7 +51,7 @@ public:
 };
 
 class CFileZillaEngineContext;
-class CUpdater : public wxEvtHandler
+class CUpdater final : public wxEvtHandler, private EngineNotificationHandler
 {
 public:
 	CUpdater(CUpdateHandler& parent, CFileZillaEngineContext& engine_context);
@@ -61,7 +65,8 @@ public:
 
 	UpdaterState GetState() const { return state_; }
 	build AvailableBuild() const { return version_information_.available_; }
-	wxString GetChangelog() const { return version_information_.changelog; }
+	wxString GetChangelog() const { return version_information_.changelog_; }
+	wxString GetResources() const { return version_information_.resources_; }
 
 	wxString DownloadedFile() const;
 
@@ -78,14 +83,14 @@ public:
 	void RunIfNeeded();
 
 protected:
-	int Download(wxString const& url, wxString const& local_file = _T(""));
+	int Download(wxString const& url, std::wstring const& local_file = std::wstring());
 	int ContinueDownload();
 
 	void AutoRunIfNeeded();
 	bool Run();
 
 	bool CreateConnectCommand(wxString const& url);
-	bool CreateTransferCommand(wxString const& url, wxString const& local_file);
+	bool CreateTransferCommand(wxString const& url, std::wstring const& local_file);
 
 	wxString GetUrl();
 	void ProcessNotification(std::unique_ptr<CNotification> && notification);
@@ -97,14 +102,16 @@ protected:
 
 	bool VerifyChecksum(wxString const& file, int64_t size, wxString const& checksum);
 
-	wxString GetTempFile() const;
+	std::wstring GetTempFile() const;
 	wxString GetFilename(wxString const& url) const;
 	wxString GetLocalFile(build const& b, bool allow_existing);
 
 	void SetState( UpdaterState s );
 
+	virtual void OnEngineEvent(CFileZillaEngine* engine);
+	void DoOnEngineEvent(CFileZillaEngine* engine);
+
 	DECLARE_EVENT_TABLE()
-	void OnEngineEvent(wxFzEvent& event);
 	void OnTimer(wxTimerEvent& ev);
 
 	UpdaterState state_;
