@@ -1,15 +1,27 @@
 #include <filezilla.h>
 #include "welcome_dialog.h"
 #include "buildinfo.h"
-#include <wx/hyperlink.h>
 #include "Options.h"
+#include "themeprovider.h"
 #include "xrc_helper.h"
+#if USE_MAC_SANDBOX
+#include "osx_sandbox_userdirs.h"
+#endif
+#include <wx/hyperlink.h>
 
 BEGIN_EVENT_TABLE(CWelcomeDialog, wxDialogEx)
 EVT_TIMER(wxID_ANY, CWelcomeDialog::OnTimer)
 END_EVENT_TABLE()
 
-bool CWelcomeDialog::Run(wxWindow* parent, bool force /*=false*/, bool delay /*=false*/)
+void CWelcomeDialog::RunDelayed(wxWindow* parent)
+{
+	CWelcomeDialog * dlg = new CWelcomeDialog;
+	dlg->parent_ = parent;
+	dlg->m_delayedShowTimer.SetOwner(dlg);
+	dlg->m_delayedShowTimer.Start(10, true);
+}
+
+bool CWelcomeDialog::Run(wxWindow* parent, bool force)
 {
 	const wxString ownVersion = CBuildInfo::GetVersion();
 	wxString greetingVersion = COptions::Get()->GetOption(OPTION_GREETINGVERSION);
@@ -19,8 +31,6 @@ bool CWelcomeDialog::Run(wxWindow* parent, bool force /*=false*/, bool delay /*=
 
 	if (!force) {
 		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
-			if (delay)
-				delete this;
 			return true;
 		}
 
@@ -28,23 +38,22 @@ bool CWelcomeDialog::Run(wxWindow* parent, bool force /*=false*/, bool delay /*=
 			CBuildInfo::ConvertToVersionNumber(ownVersion.c_str()) <= CBuildInfo::ConvertToVersionNumber(greetingVersion.c_str()))
 		{
 			// Been there done that
-			if (delay)
-				delete this;
 			return true;
 		}
-		COptions::Get()->SetOption(OPTION_GREETINGVERSION, ownVersion);
+		COptions::Get()->SetOption(OPTION_GREETINGVERSION, ownVersion.ToStdWstring());
 
-		if (greetingVersion.empty() && !COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE))
+		if (greetingVersion.empty() && !COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE)) {
 			COptions::Get()->SetOption(OPTION_PROMPTPASSWORDSAVE, 1);
+		}
 	}
 
 	if (!Load(parent, _T("ID_WELCOME"))) {
-		if (delay) {
-			delete this;
-		}
 		return false;
 	}
 
+	wxBitmap bmp = CThemeProvider::Get()->CreateBitmap("ART_FILEZILLA", wxString(), CThemeProvider::GetIconSize(iconSizeLarge));
+	xrc_call(*this, "ID_FILEZILLA_LOGO", &wxStaticBitmap::SetBitmap, bmp);
+
 	InitFooter(force ? wxString() : resources);
 
 	xrc_call(*this, "ID_FZVERSION", &wxStaticText::SetLabel, _T("FileZilla ") + CBuildInfo::GetVersion());
@@ -70,19 +79,21 @@ bool CWelcomeDialog::Run(wxWindow* parent, bool force /*=false*/, bool delay /*=
 
 	GetSizer()->Fit(this);
 
-	if (delay) {
-		m_delayedShowTimer.SetOwner(this);
-		m_delayedShowTimer.Start(10, true);
-	}
-	else
-		ShowModal();
+	ShowModal();
 
 	return true;
 }
 
 void CWelcomeDialog::OnTimer(wxTimerEvent&)
 {
-	ShowModal();
+	Run(parent_, false);
+#if USE_MAC_SANDBOX
+        if (OSXSandboxUserdirs::Get().GetDirs().empty()) {
+                OSXSandboxUserdirsDialog dlg;
+                dlg.Run(parent_);
+        }
+#endif
+
 	Destroy();
 }
 
@@ -123,5 +134,7 @@ void CWelcomeDialog::InitFooter(wxString const& resources)
 			}
 		}
 	}
+#else
+	(void)resources;
 #endif
 }
