@@ -4,11 +4,10 @@
 #include "Options.h"
 #include <wx/ffile.h>
 #include <wx/log.h>
-#include <wx/base64.h>
 
 #include <libfilezilla/local_filesys.hpp>
 
-CXmlFile::CXmlFile(wxString const& fileName, wxString const& root)
+CXmlFile::CXmlFile(std::wstring const& fileName, std::string const& root)
 {
 	if (!root.empty()) {
 		m_rootName = root;
@@ -16,26 +15,21 @@ CXmlFile::CXmlFile(wxString const& fileName, wxString const& root)
 	SetFileName(fileName);
 }
 
-void CXmlFile::SetFileName(const wxString& name)
+void CXmlFile::SetFileName(std::wstring const& name)
 {
-	wxASSERT(!name.empty());
+	assert(!name.empty());
 	m_fileName = name;
 	m_modificationTime = fz::datetime();
 }
 
-CXmlFile::~CXmlFile()
-{
-	Close();
-}
-
-pugi::xml_node CXmlFile::Load()
+pugi::xml_node CXmlFile::Load(bool overwriteInvalid)
 {
 	Close();
 	m_error.clear();
 
 	wxCHECK(!m_fileName.empty(), m_element);
 
-	wxString redirectedName = GetRedirectedName();
+	std::wstring redirectedName = GetRedirectedName();
 
 	GetXmlFile(redirectedName);
 	if (!m_element) {
@@ -50,8 +44,17 @@ pugi::xml_node CXmlFile::Load()
 		// Try the backup file
 		GetXmlFile(redirectedName + _T("~"));
 		if (!m_element) {
-			// Loading backup failed. If both original and backup file are empty, create new file.
+			// Loading backup failed.
+
+			// Create new one if we are allowed to create empty file
+			bool createEmpty = overwriteInvalid;
+
+			// Also, if both original and backup file are empty, create new file.
 			if (fz::local_filesys::get_size(fz::to_native(redirectedName)) <= 0 && fz::local_filesys::get_size(fz::to_native(redirectedName + _T("~"))) <= 0) {
+				createEmpty = true;
+			}
+
+			if (createEmpty) {
 				m_error.clear();
 				CreateEmpty();
 				m_modificationTime = fz::local_filesys::get_modification_time(fz::to_native(redirectedName));
@@ -74,8 +77,8 @@ pugi::xml_node CXmlFile::Load()
 		if (!res) {
 			// Could not restore backup, give up.
 			Close();
-			m_error = err;
-			m_error += _T("\n") + wxString::Format(_("The valid backup file %s could not be restored"), redirectedName + _T("~"));
+			m_error = err.ToStdWstring();
+			m_error += _T("\n") + wxString::Format(_("The valid backup file %s could not be restored"), redirectedName + _T("~")).ToStdWstring();
 			m_modificationTime.clear();
 			return m_element;
 		}
@@ -93,11 +96,11 @@ bool CXmlFile::Modified()
 {
 	wxCHECK(!m_fileName.empty(), false);
 
-	if (!m_modificationTime.empty())
+	if (m_modificationTime.empty())
 		return true;
 
 	fz::datetime const modificationTime = fz::local_filesys::get_modification_time(fz::to_native(m_fileName));
-	if (modificationTime.empty() && modificationTime == m_modificationTime)
+	if (!modificationTime.empty() && modificationTime == m_modificationTime)
 		return false;
 
 	return true;
@@ -117,15 +120,15 @@ void CXmlFile::UpdateMetadata()
 
 	SetTextAttribute(m_element, "version", CBuildInfo::GetVersion());
 
-	wxString const platform =
-#ifdef __WXMSW__
-		_T("windows");
-#elif defined(__WXMAX__)
-		_T("mac");
+	std::string const platform =
+#ifdef FZ_WINDOWS
+		"windows";
+#elif defined(FZ_MAC)
+		"mac";
 #else
-		_T("*nix");
+		"*nix";
 #endif
-	SetTextAttribute(m_element, "platform", platform);
+	SetTextAttributeUtf8(m_element, "platform", platform);
 }
 
 bool CXmlFile::Save(bool printError)
@@ -141,7 +144,7 @@ bool CXmlFile::Save(bool printError)
 	m_modificationTime = fz::local_filesys::get_modification_time(fz::to_native(m_fileName));
 
 	if (!res && printError) {
-		wxASSERT(!m_error.empty());
+		assert(!m_error.empty());
 
 		wxString msg = wxString::Format(_("Could not write \"%s\":"), m_fileName);
 		wxMessageBoxEx(msg + _T("\n") + m_error, _("Error writing xml file"), wxICON_ERROR);
@@ -157,27 +160,18 @@ pugi::xml_node CXmlFile::CreateEmpty()
 	decl.append_attribute("version") = "1.0";
 	decl.append_attribute("encoding") = "UTF-8";
 
-	m_element = m_document.append_child(m_rootName.utf8_str());
+	m_element = m_document.append_child(m_rootName.c_str());
 	return m_element;
 }
 
-wxString ConvLocal(const char *value)
+pugi::xml_node AddTextElement(pugi::xml_node node, const char* name, std::string const& value, bool overwrite)
 {
-	return wxString(wxConvUTF8.cMB2WC(value), *wxConvCurrent);
+	return AddTextElementUtf8(node, name, fz::to_utf8(value), overwrite);
 }
 
-pugi::xml_node AddTextElement(pugi::xml_node node, const char* name, const wxString& value, bool overwrite)
+pugi::xml_node AddTextElement(pugi::xml_node node, const char* name, std::wstring const& value, bool overwrite)
 {
-	pugi::xml_node ret;
-
-	wxASSERT(node);
-
-	wxScopedCharBuffer utf8 = value.utf8_str();
-	if (utf8) {
-		ret = AddTextElementRaw(node, name, utf8, overwrite);
-	}
-
-	return ret;
+	return AddTextElementUtf8(node, name, fz::to_utf8(value), overwrite);
 }
 
 void AddTextElement(pugi::xml_node node, const char* name, int64_t value, bool overwrite)
@@ -189,82 +183,66 @@ void AddTextElement(pugi::xml_node node, const char* name, int64_t value, bool o
 	child.text().set(static_cast<long long>(value));
 }
 
-pugi::xml_node AddTextElementRaw(pugi::xml_node node, const char* name, const char* value, bool overwrite)
+pugi::xml_node AddTextElementUtf8(pugi::xml_node node, const char* name, std::string const& value, bool overwrite)
 {
-	wxASSERT(node);
-	wxASSERT(value);
+	assert(node);
 
 	if (overwrite) {
 		node.remove_child(name);
 	}
 
 	auto element = node.append_child(name);
-	if (*value)
-		element.text().set(value);
+	if (!value.empty()) {
+		element.text().set(value.c_str());
+	}
 
 	return element;
 }
 
-void AddTextElement(pugi::xml_node node, const wxString& value)
+void AddTextElement(pugi::xml_node node, std::string const& value)
 {
-	wxASSERT(node);
-	wxASSERT(value);
+	AddTextElementUtf8(node, fz::to_utf8(value));
+}
 
-	wxScopedCharBuffer utf8 = value.utf8_str();
-	if (!utf8)
-		return;
-
-	AddTextElementRaw(node, utf8);
+void AddTextElement(pugi::xml_node node, std::wstring const& value)
+{
+	AddTextElementUtf8(node, fz::to_utf8(value));
 }
 
 void AddTextElement(pugi::xml_node node, int64_t value)
 {
-	wxASSERT(node);
+	assert(node);
 	node.text().set(static_cast<long long>(value));
 }
 
-void AddTextElementRaw(pugi::xml_node node, const char* value)
+void AddTextElementUtf8(pugi::xml_node node, std::string const& value)
 {
-	wxASSERT(node);
-	wxASSERT(value);
-
-	if (*value)
-		node.text().set(value);
-	else {
-		node.text().set("");
-	}
+	assert(node);
+	node.text().set(value.c_str());
 }
 
-wxString GetTextElement_Trimmed(pugi::xml_node node, const char* name)
+std::wstring GetTextElement_Trimmed(pugi::xml_node node, const char* name)
 {
-	wxString t = GetTextElement(node, name);
-	t.Trim(true);
-	t.Trim(false);
-
-	return t;
+	return fz::trimmed(GetTextElement(node, name));
 }
 
-wxString GetTextElement(pugi::xml_node node, const char* name)
+std::wstring GetTextElement(pugi::xml_node node, const char* name)
 {
 	wxASSERT(node);
 
-	return ConvLocal(node.child_value(name));
+	return fz::to_wstring_from_utf8(node.child_value(name));
 }
 
-wxString GetTextElement_Trimmed(pugi::xml_node node)
+std::wstring GetTextElement_Trimmed(pugi::xml_node node)
 {
-	wxString t = GetTextElement(node);
-	t.Trim(true);
-	t.Trim(false);
-
-	return t;
+	return fz::trimmed(GetTextElement(node));
 }
 
-wxString GetTextElement(pugi::xml_node node)
+std::wstring GetTextElement(pugi::xml_node node)
 {
 	wxASSERT(node);
 
-	return ConvLocal(node.child_value());
+	return fz::to_wstring_from_utf8(node.child_value());
 }
 
 int64_t GetTextElementInt(pugi::xml_node node, const char* name, int defValue /*=0*/)
@@ -280,8 +258,8 @@ bool GetTextElementBool(pugi::xml_node node, const char* name, bool defValue /*=
 }
 
 // Opens the specified XML file if it exists or creates a new one otherwise.
-// Returns 0 on error.
-bool CXmlFile::GetXmlFile(wxString const& file)
+// Returns false on error.
+bool CXmlFile::GetXmlFile(std::wstring const& file)
 {
 	Close();
 
@@ -292,34 +270,34 @@ bool CXmlFile::GetXmlFile(wxString const& file)
 	// File exists, open it
 	auto result = m_document.load_file(static_cast<wchar_t const*>(file.c_str()));
 	if (!result) {
-		m_error += wxString::Format(_T("%s at offset %lld."), wxString(result.description()), static_cast<long long>(result.offset));
+		m_error += fz::sprintf(_T("%s at offset %d."), result.description(), result.offset);
 		return false;
 	}
 
-	m_element = m_document.child(m_rootName.utf8_str());
+	m_element = m_document.child(m_rootName.c_str());
 	if (!m_element) {
 		if (m_document.first_child()) { // Beware: parse_declaration and parse_doctype can break this
 			// Not created by FileZilla3
 			Close();
-			m_error = _("Unknown root element, the file does not appear to be generated by FileZilla.");
+			m_error = _("Unknown root element, the file does not appear to be generated by FileZilla.").ToStdWstring();
 			return false;
 		}
-		m_element = m_document.append_child(m_rootName.utf8_str());
+		m_element = m_document.append_child(m_rootName.c_str());
 	}
 
 	return true;
 }
 
-wxString CXmlFile::GetRedirectedName() const
+std::wstring CXmlFile::GetRedirectedName() const
 {
-	wxString redirectedName = m_fileName;
+	std::wstring redirectedName = m_fileName;
 	bool isLink = false;
 	if (fz::local_filesys::get_file_info(fz::to_native(redirectedName), isLink, 0, 0, 0) == fz::local_filesys::file) {
 		if (isLink) {
-			CLocalPath target(fz::local_filesys::get_link_target(fz::to_native(redirectedName)));
+			CLocalPath target(fz::to_wstring(fz::local_filesys::get_link_target(fz::to_native(redirectedName))));
 			if (!target.empty()) {
 				redirectedName = target.GetPath();
-				redirectedName.RemoveLast();
+				redirectedName.pop_back();
 			}
 		}
 	}
@@ -347,7 +325,7 @@ bool CXmlFile::SaveXmlFile()
 			res = wxCopyFile(redirectedName, redirectedName + _T("~"));
 		}
 		if (!res) {
-			m_error = _("Failed to create backup copy of xml file");
+			m_error = _("Failed to create backup copy of xml file").ToStdWstring();
 			return false;
 		}
 	}
@@ -360,7 +338,7 @@ bool CXmlFile::SaveXmlFile()
 			wxLogNull null;
 			wxRenameFile(redirectedName + _T("~"), redirectedName);
 		}
-		m_error = _("Failed to write xml file");
+		m_error = _("Failed to write xml file").ToStdWstring();
 		return false;
 	}
 
@@ -370,251 +348,277 @@ bool CXmlFile::SaveXmlFile()
 	return true;
 }
 
-bool GetServer(pugi::xml_node node, CServer& server)
+bool GetServer(pugi::xml_node node, ServerWithCredentials & server)
 {
 	wxASSERT(node);
 
-	wxString host = GetTextElement(node, "Host");
-	if (host.empty())
+	std::wstring host = GetTextElement(node, "Host");
+	if (host.empty()) {
 		return false;
+	}
 
 	int port = GetTextElementInt(node, "Port");
-	if (port < 1 || port > 65535)
+	if (port < 1 || port > 65535) {
 		return false;
+	}
 
-	if (!server.SetHost(host, port))
+	if (!server.server.SetHost(host, port)) {
 		return false;
+	}
 
 	int const protocol = GetTextElementInt(node, "Protocol");
 	if (protocol < 0 || protocol > ServerProtocol::MAX_VALUE) {
 		return false;
 	}
-	server.SetProtocol(static_cast<ServerProtocol>(protocol));
+	server.server.SetProtocol(static_cast<ServerProtocol>(protocol));
 
 	int type = GetTextElementInt(node, "Type");
-	if (type < 0 || type >= SERVERTYPE_MAX)
+	if (type < 0 || type >= SERVERTYPE_MAX) {
 		return false;
+	}
 
-	server.SetType((enum ServerType)type);
+	server.server.SetType(static_cast<ServerType>(type));
 
 	int logonType = GetTextElementInt(node, "Logontype");
-	if (logonType < 0)
+	if (logonType < 0 || logonType >= static_cast<int>(LogonType::count)) {
 		return false;
+	}
 
-	server.SetLogonType((enum LogonType)logonType);
+	server.SetLogonType(static_cast<LogonType>(logonType));
+	
+	if (server.credentials.logonType_ != LogonType::anonymous) {
+		std::wstring user = GetTextElement(node, "User");
+		if (user.empty() && server.credentials.logonType_ != LogonType::interactive && server.credentials.logonType_ != LogonType::ask) {
+			return false;
+		}
 
-	if (server.GetLogonType() != ANONYMOUS) {
-		wxString user = GetTextElement(node, "User");
-
-		wxString pass, key;
-		if ((long)NORMAL == logonType || (long)ACCOUNT == logonType) {
-			auto  passElement = node.child("Pass");
+		std::wstring pass, key;
+		if (server.credentials.logonType_ == LogonType::normal || server.credentials.logonType_ == LogonType::account) {
+			auto passElement = node.child("Pass");
 			if (passElement) {
-				pass = GetTextElement(passElement);
-
-				wxString encoding = GetTextAttribute(passElement, "encoding");
+				std::wstring encoding = GetTextAttribute(passElement, "encoding");
 
 				if (encoding == _T("base64")) {
-					wxMemoryBuffer buf = wxBase64Decode(pass);
-					if (!buf.IsEmpty()) {
-						pass = wxString::FromUTF8(static_cast<const char*>(buf.GetData()), buf.GetDataLen());
-					}
-					else {
+					std::string decoded = fz::base64_decode(passElement.child_value());
+					pass = fz::to_wstring_from_utf8(decoded);
+				}
+				else if (encoding == _T("crypt")) {
+					pass = fz::to_wstring_from_utf8(passElement.child_value());
+					server.credentials.encrypted_ = public_key::from_base64(passElement.attribute("pubkey").value());
+					if (!server.credentials.encrypted_) {
 						pass.clear();
+						server.SetLogonType(LogonType::ask);
 					}
 				}
 				else if (!encoding.empty()) {
-					pass.clear();
-					server.SetLogonType(ASK);
+					server.SetLogonType(LogonType::ask);
+				}
+				else {
+					pass = GetTextElement(passElement);
 				}
 			}
-		} else if ((long)KEY == logonType) {
+		}
+		else if (server.credentials.logonType_ == LogonType::key) {
 			key = GetTextElement(node, "Keyfile");
 
 			// password should be empty if we're using a key file
-			pass = wxString();
+			pass.clear();
 
-			server.SetKeyFile(key);
+			server.credentials.keyFile_ = key;
 		}
 
-		if (!server.SetUser(user, pass))
-			return false;
+		server.SetUser(user);
+		server.credentials.SetPass(pass);
 
-		if ((long)ACCOUNT == logonType) {
-			wxString account = GetTextElement(node, "Account");
-			if (account.empty())
-				return false;
-			if (!server.SetAccount(account))
-				return false;
-		}
+		server.credentials.account_ = GetTextElement(node, "Account");
 	}
 
 	int timezoneOffset = GetTextElementInt(node, "TimezoneOffset");
-	if (!server.SetTimezoneOffset(timezoneOffset))
+	if (!server.server.SetTimezoneOffset(timezoneOffset)) {
 		return false;
+	}
 
 	wxString pasvMode = GetTextElement(node, "PasvMode");
-	if (pasvMode == _T("MODE_PASSIVE"))
-		server.SetPasvMode(MODE_PASSIVE);
-	else if (pasvMode == _T("MODE_ACTIVE"))
-		server.SetPasvMode(MODE_ACTIVE);
-	else
-		server.SetPasvMode(MODE_DEFAULT);
+	if (pasvMode == _T("MODE_PASSIVE")) {
+		server.server.SetPasvMode(MODE_PASSIVE);
+	}
+	else if (pasvMode == _T("MODE_ACTIVE")) {
+		server.server.SetPasvMode(MODE_ACTIVE);
+	}
+	else {
+		server.server.SetPasvMode(MODE_DEFAULT);
+	}
 
 	int maximumMultipleConnections = GetTextElementInt(node, "MaximumMultipleConnections");
-	server.MaximumMultipleConnections(maximumMultipleConnections);
+	server.server.MaximumMultipleConnections(maximumMultipleConnections);
 
 	wxString encodingType = GetTextElement(node, "EncodingType");
-	if (encodingType == _T("Auto"))
-		server.SetEncodingType(ENCODING_AUTO);
-	else if (encodingType == _T("UTF-8"))
-		server.SetEncodingType(ENCODING_UTF8);
+	if (encodingType == _T("Auto")) {
+		server.server.SetEncodingType(ENCODING_AUTO);
+	}
+	else if (encodingType == _T("UTF-8")) {
+		server.server.SetEncodingType(ENCODING_UTF8);
+	}
 	else if (encodingType == _T("Custom")) {
-		wxString customEncoding = GetTextElement(node, "CustomEncoding");
-		if (customEncoding.empty())
+		std::wstring customEncoding = GetTextElement(node, "CustomEncoding");
+		if (customEncoding.empty()) {
 			return false;
-		if (!server.SetEncodingType(ENCODING_CUSTOM, customEncoding))
+		}
+		if (!server.server.SetEncodingType(ENCODING_CUSTOM, customEncoding)) {
 			return false;
+		}
+	}
+	else {
+		server.server.SetEncodingType(ENCODING_AUTO);
 	}
-	else
-		server.SetEncodingType(ENCODING_AUTO);
 
-	if (CServer::SupportsPostLoginCommands(server.GetProtocol())) {
-		std::vector<wxString> postLoginCommands;
+	if (CServer::SupportsPostLoginCommands(server.server.GetProtocol())) {
+		std::vector<std::wstring> postLoginCommands;
 		auto element = node.child("PostLoginCommands");
 		if (element) {
 			for (auto commandElement = element.child("Command"); commandElement; commandElement = commandElement.next_sibling("Command")) {
-				wxString command = ConvLocal(commandElement.child_value());
+				std::wstring command = fz::to_wstring_from_utf8(commandElement.child_value());
 				if (!command.empty()) {
-					postLoginCommands.push_back(command);
+					postLoginCommands.emplace_back(std::move(command));
 				}
 			}
 		}
-		if (!server.SetPostLoginCommands(postLoginCommands))
+		if (!server.server.SetPostLoginCommands(postLoginCommands)) {
 			return false;
+		}
 	}
 
-	server.SetBypassProxy(GetTextElementInt(node, "BypassProxy", false) == 1);
-	server.SetName(GetTextElement_Trimmed(node, "Name"));
+	server.server.SetBypassProxy(GetTextElementInt(node, "BypassProxy", false) == 1);
+	server.server.SetName(GetTextElement_Trimmed(node, "Name"));
 
-	if (server.GetName().empty())
-		server.SetName(GetTextElement_Trimmed(node));
+	if (server.server.GetName().empty()) {
+		server.server.SetName(GetTextElement_Trimmed(node));
+	}
 
 	return true;
 }
 
-void SetServer(pugi::xml_node node, const CServer& server)
+void SetServer(pugi::xml_node node, ServerWithCredentials const& server)
 {
-	if (!node)
+	if (!node) {
 		return;
-
-	bool kiosk_mode = COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 0;
+	}
 
 	for (auto child = node.first_child(); child; child = node.first_child()) {
 		node.remove_child(child);
 	}
 
-	AddTextElement(node, "Host", server.GetHost());
-	AddTextElement(node, "Port", server.GetPort());
-	AddTextElement(node, "Protocol", server.GetProtocol());
-	AddTextElement(node, "Type", server.GetType());
+	AddTextElement(node, "Host", server.server.GetHost());
+	AddTextElement(node, "Port", server.server.GetPort());
+	AddTextElement(node, "Protocol", server.server.GetProtocol());
+	AddTextElement(node, "Type", server.server.GetType());
 
-	enum LogonType logonType = server.GetLogonType();
+	ProtectedCredentials credentials = server.credentials;
 
-	if (server.GetLogonType() != ANONYMOUS) {
-		AddTextElement(node, "User", server.GetUser());
+	if (credentials.logonType_ != LogonType::anonymous) {
+		AddTextElement(node, "User", server.server.GetUser());
 
-		if (server.GetLogonType() == NORMAL || server.GetLogonType() == ACCOUNT) {
-			if (kiosk_mode)
-				logonType = ASK;
+		credentials.Protect();
+
+		if (credentials.logonType_ == LogonType::normal || credentials.logonType_ == LogonType::account) {
+			std::string pass = fz::to_utf8(credentials.GetPass());
+
+			if (credentials.encrypted_) {
+				pugi::xml_node passElement = AddTextElementUtf8(node, "Pass", pass);
+				if (passElement) {
+					SetTextAttribute(passElement, "encoding", _T("crypt"));
+					SetTextAttributeUtf8(passElement, "pubkey", credentials.encrypted_.to_base64());
+				}
+			}
 			else {
-				wxString pass = server.GetPass();
-				auto const& buf = pass.utf8_str(); // wxWidgets has such an ugly string API....
-				std::string utf8(buf.data(), buf.length());
-
-				wxString base64 = wxBase64Encode(utf8.c_str(), utf8.size());
-				pugi::xml_node passElement = AddTextElement(node, "Pass", base64);
+				pugi::xml_node passElement = AddTextElementUtf8(node, "Pass", fz::base64_encode(pass));
 				if (passElement) {
 					SetTextAttribute(passElement, "encoding", _T("base64"));
 				}
+			}
 
-				if (server.GetLogonType() == ACCOUNT)
-					AddTextElement(node, "Account", server.GetAccount());
+			if (credentials.logonType_ == LogonType::account) {
+				AddTextElement(node, "Account", credentials.account_);
 			}
 		}
-		else if (server.GetLogonType() == KEY)
-		{
-			AddTextElement(node, "Keyfile", server.GetKeyFile());
+		else if (!credentials.keyFile_.empty()) {
+			AddTextElement(node, "Keyfile", credentials.keyFile_);
 		}
 	}
-	AddTextElement(node, "Logontype", logonType);
+	AddTextElement(node, "Logontype", static_cast<int>(credentials.logonType_));
 
-	AddTextElement(node, "TimezoneOffset", server.GetTimezoneOffset());
-	switch (server.GetPasvMode())
+	AddTextElement(node, "TimezoneOffset", server.server.GetTimezoneOffset());
+	switch (server.server.GetPasvMode())
 	{
 	case MODE_PASSIVE:
-		AddTextElementRaw(node, "PasvMode", "MODE_PASSIVE");
+		AddTextElementUtf8(node, "PasvMode", "MODE_PASSIVE");
 		break;
 	case MODE_ACTIVE:
-		AddTextElementRaw(node, "PasvMode", "MODE_ACTIVE");
+		AddTextElementUtf8(node, "PasvMode", "MODE_ACTIVE");
 		break;
 	default:
-		AddTextElementRaw(node, "PasvMode", "MODE_DEFAULT");
+		AddTextElementUtf8(node, "PasvMode", "MODE_DEFAULT");
 		break;
 	}
-	AddTextElement(node, "MaximumMultipleConnections", server.MaximumMultipleConnections());
+	AddTextElement(node, "MaximumMultipleConnections", server.server.MaximumMultipleConnections());
 
-	switch (server.GetEncodingType())
+	switch (server.server.GetEncodingType())
 	{
 	case ENCODING_AUTO:
-		AddTextElementRaw(node, "EncodingType", "Auto");
+		AddTextElementUtf8(node, "EncodingType", "Auto");
 		break;
 	case ENCODING_UTF8:
-		AddTextElementRaw(node, "EncodingType", "UTF-8");
+		AddTextElementUtf8(node, "EncodingType", "UTF-8");
 		break;
 	case ENCODING_CUSTOM:
-		AddTextElementRaw(node, "EncodingType", "Custom");
-		AddTextElement(node, "CustomEncoding", server.GetCustomEncoding());
+		AddTextElementUtf8(node, "EncodingType", "Custom");
+		AddTextElement(node, "CustomEncoding", server.server.GetCustomEncoding());
 		break;
 	}
 
-	if (CServer::SupportsPostLoginCommands(server.GetProtocol())) {
-		std::vector<wxString> const& postLoginCommands = server.GetPostLoginCommands();
+	if (CServer::SupportsPostLoginCommands(server.server.GetProtocol())) {
+		std::vector<std::wstring> const& postLoginCommands = server.server.GetPostLoginCommands();
 		if (!postLoginCommands.empty()) {
 			auto element = node.append_child("PostLoginCommands");
-			for (std::vector<wxString>::const_iterator iter = postLoginCommands.begin(); iter != postLoginCommands.end(); ++iter) {
-				AddTextElement(element, "Command", *iter);
-			}				
+			for (auto const& command : postLoginCommands) {
+				AddTextElement(element, "Command", command);
+			}
 		}
 	}
 
-	AddTextElementRaw(node, "BypassProxy", server.GetBypassProxy() ? "1" : "0");
-	const wxString& name = server.GetName();
-	if (!name.empty())
+	AddTextElementUtf8(node, "BypassProxy", server.server.GetBypassProxy() ? "1" : "0");
+	std::wstring const& name = server.server.GetName();
+	if (!name.empty()) {
 		AddTextElement(node, "Name", name);
+	}
 }
 
-void SetTextAttribute(pugi::xml_node node, const char* name, const wxString& value)
+void SetTextAttribute(pugi::xml_node node, char const* name, std::string const& value)
 {
-	wxASSERT(node);
+	SetTextAttributeUtf8(node, name, fz::to_utf8(value));
+}
 
-	wxScopedCharBuffer utf8 = value.utf8_str();
-	if (!utf8)
-		return;
+void SetTextAttribute(pugi::xml_node node, char const* name, std::wstring const& value)
+{
+	SetTextAttributeUtf8(node, name, fz::to_utf8(value));
+}
 
+void SetTextAttributeUtf8(pugi::xml_node node, char const* name, std::string const& utf8)
+{
+	assert(node);
 	auto attribute = node.attribute(name);
 	if (!attribute) {
 		attribute = node.append_attribute(name);
 	}
-	attribute.set_value(utf8);
+	attribute.set_value(utf8.c_str());
 }
 
-wxString GetTextAttribute(pugi::xml_node node, const char* name)
+std::wstring GetTextAttribute(pugi::xml_node node, char const* name)
 {
-	wxASSERT(node);
+	assert(node);
 
 	const char* value = node.attribute(name).value();
-	return ConvLocal(value);
+	return fz::to_wstring_from_utf8(value);
 }
 
 pugi::xml_node FindElementWithAttribute(pugi::xml_node node, const char* element, const char* attribute, const char* value)
@@ -622,8 +626,9 @@ pugi::xml_node FindElementWithAttribute(pugi::xml_node node, const char* element
 	pugi::xml_node child = element ? node.child(element) : node.first_child();
 	while (child) {
 		const char* nodeVal = child.attribute(attribute).value();
-		if (nodeVal && !strcmp(value, nodeVal))
+		if (nodeVal && !strcmp(value, nodeVal)) {
 			return child;
+		}
 
 		child = element ? child.next_sibling(element) : child.next_sibling();
 	}
@@ -666,8 +671,9 @@ struct xml_memory_writer : pugi::xml_writer
 
 size_t CXmlFile::GetRawDataLength()
 {
-	if (!m_document)
+	if (!m_document) {
 		return 0;
+	}
 
 	xml_memory_writer writer;
 	m_document.save(writer);
@@ -689,7 +695,7 @@ bool CXmlFile::ParseData(char* data)
 {
 	Close();
 	m_document.load_string(data);
-	m_element = m_document.child(m_rootName.mb_str());
+	m_element = m_document.child(m_rootName.c_str());
 	if (!m_element) {
 		Close();
 	}
@@ -701,6 +707,6 @@ bool CXmlFile::IsFromFutureVersion() const
 	if (!m_element) {
 		return false;
 	}
-	wxString const version = GetTextAttribute(m_element, "version");
+	std::wstring const version = GetTextAttribute(m_element, "version");
 	return CBuildInfo::ConvertToVersionNumber(CBuildInfo::GetVersion().c_str()) < CBuildInfo::ConvertToVersionNumber(version.c_str());
 }
