@@ -403,7 +403,18 @@ int cmdline_process_param(const char *p, char *value,
 	SAVEABLE(0);
 	conf_set_int(conf, CONF_tryagent, FALSE);
     }
-
+    if (!strcmp(p, "-share")) {
+	RETURN(1);
+	UNAVAILABLE_IN(TOOLTYPE_NONNETWORK);
+	SAVEABLE(0);
+	conf_set_int(conf, CONF_ssh_connection_sharing, TRUE);
+    }
+    if (!strcmp(p, "-noshare")) {
+	RETURN(1);
+	UNAVAILABLE_IN(TOOLTYPE_NONNETWORK);
+	SAVEABLE(0);
+	conf_set_int(conf, CONF_ssh_connection_sharing, FALSE);
+    }
     if (!strcmp(p, "-A")) {
 	RETURN(1);
 	UNAVAILABLE_IN(TOOLTYPE_FILETRANSFER | TOOLTYPE_NONNETWORK);
@@ -609,6 +620,18 @@ int cmdline_process_param(const char *p, char *value,
 	conf_set_str(conf, CONF_proxy_telnet_command, value);
     }
 
+#ifdef _WINDOWS
+    /*
+     * Cross-tool options only available on Windows.
+     */
+    if (!strcmp(p, "-restrict-acl") || !strcmp(p, "-restrict_acl") ||
+        !strcmp(p, "-restrictacl")) {
+	RETURN(1);
+        restrict_process_acl();
+        restricted_acl = TRUE;
+    }
+#endif
+
     return ret;			       /* unrecognised */
 }
 
