@@ -346,6 +346,34 @@ void sk_getaddr(SockAddr addr, char *buf, int buflen)
     }
 }
 
+/*
+ * This constructs a SockAddr that points at one specific sub-address
+ * of a parent SockAddr. The returned SockAddr does not own all its
+ * own memory: it points into the old one's data structures, so it
+ * MUST NOT be used after the old one is freed, and it MUST NOT be
+ * passed to sk_addr_free. (The latter is why it's returned by value
+ * rather than dynamically allocated - that should clue in anyone
+ * writing a call to it that something is weird about it.)
+ */
+static struct SockAddr_tag sk_extractaddr_tmp(
+    SockAddr addr, const SockAddrStep *step)
+{
+    struct SockAddr_tag toret;
+    toret = *addr;                    /* structure copy */
+    toret.refcount = 1;
+
+    if (addr->superfamily == IP) {
+#ifndef NO_IPV6
+        toret.ais = step->ai;
+#else
+	assert(SOCKADDR_FAMILY(addr, *step) == AF_INET);
+        toret.addresses += step->curraddr;
+#endif
+    }
+
+    return toret;
+}
+
 int sk_addr_needs_port(SockAddr addr)
 {
     if (addr->superfamily == UNRESOLVED || addr->superfamily == UNIX) {
@@ -565,7 +593,11 @@ static int try_connect(Actual_Socket sock)
     if (sock->s >= 0)
         close(sock->s);
 
-    plug_log(sock->plug, 0, sock->addr, sock->port, NULL, 0);
+    {
+        struct SockAddr_tag thisaddr = sk_extractaddr_tmp(
+            sock->addr, &sock->step);
+        plug_log(sock->plug, 0, &thisaddr, sock->port, NULL, 0);
+    }
 
     /*
      * Open socket.
@@ -743,8 +775,11 @@ static int try_connect(Actual_Socket sock)
      */
     add234(sktree, sock);
 
-    if (err)
-	plug_log(sock->plug, 1, sock->addr, sock->port, strerror(err), err);
+    if (err) {
+        struct SockAddr_tag thisaddr = sk_extractaddr_tmp(
+            sock->addr, &sock->step);
+	plug_log(sock->plug, 1, &thisaddr, sock->port, strerror(err), err);
+    }
     return err;
 }
 
@@ -1255,7 +1290,7 @@ static void sk_tcp_write_eof(Socket sock)
     uxsel_tell(s);
 }
 
-static int net_select_result(int fd, int event)
+static void net_select_result(int fd, int event)
 {
     int ret;
     char buf[20480];		       /* nice big buffer for plenty of speed */
@@ -1266,7 +1301,7 @@ static int net_select_result(int fd, int event)
     /* Find the Socket structure */
     s = find234(sktree, &fd, cmpforsearch);
     if (!s)
-	return 1;		       /* boggle */
+	return;		       /* boggle */
 
     noise_ultralight(event);
 
@@ -1283,9 +1318,9 @@ static int net_select_result(int fd, int event)
 	    ret = recv(s->s, buf, toRecv, MSG_OOB);
 	    noise_ultralight(ret);
 	    if (ret <= 0) {
-                return plug_closing(s->plug,
-				    ret == 0 ? "Internal networking trouble" :
-				    strerror(errno), errno, 0);
+                plug_closing(s->plug,
+			     ret == 0 ? "Internal networking trouble" :
+			     strerror(errno), errno, 0);
 	    } else {
 		UpdateQuota(0, ret);
 		if (fz_timer_check(&s->recv_timer))
@@ -1299,7 +1334,7 @@ static int net_select_result(int fd, int event)
                     sk_addr_free(s->addr);
                     s->addr = NULL;
                 }
-		return plug_receive(s->plug, 2, buf, ret);
+		plug_receive(s->plug, 2, buf, ret);
 	    }
 	    break;
 	}
@@ -1372,25 +1407,11 @@ static int net_select_result(int fd, int event)
 	    }
 	}
 	if (ret < 0) {
-            /*
-             * An error at this point _might_ be an error reported
-             * by a non-blocking connect(). So before we return a
-             * panic status to the user, let's just see whether
-             * that's the case.
-             */
-            int err = errno;
-	    if (s->addr) {
-		plug_log(s->plug, 1, s->addr, s->port, strerror(err), err);
-		while (s->addr && sk_nextaddr(s->addr, &s->step)) {
-		    err = try_connect(s);
-		}
-	    }
-            if (err != 0)
-                return plug_closing(s->plug, strerror(err), err, 0);
+            plug_closing(s->plug, strerror(errno), errno, 0);
 	} else if (0 == ret) {
             s->incomingeof = TRUE;     /* stop trying to read now */
             uxsel_tell(s);
-	    return plug_closing(s->plug, NULL, 0, 0);
+	    plug_closing(s->plug, NULL, 0, 0);
 	} else {
 	    UpdateQuota(0, ret);
 	    if (fz_timer_check(&s->recv_timer))
@@ -1404,18 +1425,59 @@ static int net_select_result(int fd, int event)
                 sk_addr_free(s->addr);
                 s->addr = NULL;
             }
-	    return plug_receive(s->plug, atmark ? 0 : 1, buf, ret);
+	    plug_receive(s->plug, atmark ? 0 : 1, buf, ret);
 	}
 	break;
       case 2:			       /* writable */
 	if (!s->connected) {
 	    /*
 	     * select() reports a socket as _writable_ when an
-	     * asynchronous connection is completed.
+	     * asynchronous connect() attempt either completes or
+	     * fails. So first we must find out which.
 	     */
+            {
+                int err;
+                socklen_t errlen = sizeof(err);
+                char *errmsg = NULL;
+                if (getsockopt(s->s, SOL_SOCKET, SO_ERROR, &err, &errlen)<0) {
+                    errmsg = dupprintf("getsockopt(SO_ERROR): %s",
+                                       strerror(errno));
+                    err = errno;       /* got to put something in here */
+                } else if (err != 0) {
+                    errmsg = dupstr(strerror(err));
+                }
+                if (errmsg) {
+                    /*
+                     * The asynchronous connection attempt failed.
+                     * Report the problem via plug_log, and try again
+                     * with the next candidate address, if we have
+                     * more than one.
+                     */
+                    struct SockAddr_tag thisaddr;
+                    assert(s->addr);
+
+                    thisaddr = sk_extractaddr_tmp(s->addr, &s->step);
+                    plug_log(s->plug, 1, &thisaddr, s->port, errmsg, err);
+
+                    while (err && s->addr && sk_nextaddr(s->addr, &s->step)) {
+                        err = try_connect(s);
+                    }
+                    if (err)
+                        plug_closing(s->plug, strerror(err), err, 0);
+                    if (!s->connected)
+                        return;      /* another async attempt in progress */
+                }
+            }
+
+            /*
+             * If we get here, we've managed to make a connection.
+             */
+            if (s->addr) {
+                sk_addr_free(s->addr);
+                s->addr = NULL;
+            }
 	    s->connected = s->writable = 1;
 	    uxsel_tell(s);
-	    break;
 	} else {
 	    int bufsize_before, bufsize_after;
 	    s->writable = 1;
@@ -1427,8 +1489,6 @@ static int net_select_result(int fd, int event)
 	}
 	break;
     }
-
-    return 1;
 }
 
 /*
@@ -1591,7 +1651,8 @@ SockAddr unix_sock_addr(const char *path)
 
     if (n < 0)
 	ret->error = "snprintf failed";
-    else if (n >= sizeof ret->hostname)
+    else if (n >= sizeof ret->hostname ||
+             n >= sizeof(((struct sockaddr_un *)0)->sun_path))
 	ret->error = "socket pathname too long";
 
 #ifndef NO_IPV6
