@@ -2,8 +2,11 @@
  * winsftp.c: the Windows-specific parts of PSFTP and PSCP.
  */
 
+#include <winsock2.h> /* need to put this first, for winelib builds */
 #include <assert.h>
 
+#define NEED_DECLARATION_OF_SELECT
+
 #include "putty.h"
 #include "psftp.h"
 #include "ssh.h"
@@ -554,7 +557,6 @@ char *do_select(SOCKET skt, int startup)
     }
     return NULL;
 }
-extern int select_result(WPARAM, LPARAM);
 
 int do_eventsel_loop(HANDLE other_event)
 {
@@ -603,7 +605,6 @@ int do_eventsel_loop(HANDLE other_event)
 	WSANETWORKEVENTS things;
 	SOCKET socket;
 	extern SOCKET first_socket(int *), next_socket(int *);
-	extern int select_result(WPARAM, LPARAM);
 	int i, socketstate;
 
 	/*
@@ -808,23 +809,11 @@ char *ssh_sftp_get_cmdline(const char *prompt, int no_fds_ok)
     return ctx->line;
 }
 
-void platform_psftp_post_option_setup(void)
+void platform_psftp_pre_conn_setup(void)
 {
-#if !defined UNPROTECT && !defined NO_SECURITY
-    /*
-     * Protect our process.
-     */
-    {
-        char *error = NULL;
-        if (!setprocessacl(error)) {
-            char *message = dupprintf("Could not restrict process ACL: %s",
-                                      error);
-            logevent(NULL, message);
-            sfree(message);
-            sfree(error);
-        }
+    if (restricted_acl) {
+	logevent(NULL, "Running with restricted process ACL");
     }
-#endif
 }
 
 /* ----------------------------------------------------------------------
